/// autogenerated analytical inverse kinematics code from ikfast program
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// generated 2010-11-03 15:37:15.733851
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object:
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -shared -Wl,-soname,ik.so -o ik.so ik.cpp
#include <cmath>
#include <cassert>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IK2PI  6.28318530717959
#define IKPI  3.14159265358979
#define IKPI_2  1.57079632679490

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// defined when creating a shared object/dll
#ifdef IKFAST_CLIBRARY
#ifdef _MSC_VER
#define IKFAST_API extern "C" __declspec(dllexport)
#else
#define IKFAST_API extern "C"
#endif
#else
#define IKFAST_API
#endif

typedef double IKReal;
class IKSolution
{
public:
    /// Gets a solution given its free parameters
    /// \param pfree The free parameters required, range is in [-pi,pi]
    void GetSolution(IKReal* psolution, const IKReal* pfree) const {
        for(size_t i = 0; i < basesol.size(); ++i) {
            if( basesol[i].freeind < 0 )
                psolution[i] = basesol[i].foffset;
            else {
                assert(pfree != NULL);
                psolution[i] = pfree[basesol[i].freeind]*basesol[i].fmul + basesol[i].foffset;
                if( psolution[i] > IKPI )
                    psolution[i] -= IK2PI;
                else if( psolution[i] < -IKPI )
                    psolution[i] += IK2PI;
            }
        }
    }

    /// Gets the free parameters the solution requires to be set before a full solution can be returned
    /// \return vector of indices indicating the free parameters
    const std::vector<int>& GetFree() const { return vfree; }

    struct VARIABLE
    {
        VARIABLE() : freeind(-1), fmul(0), foffset(0) {}
        VARIABLE(int freeind, IKReal fmul, IKReal foffset) : freeind(freeind), fmul(fmul), foffset(foffset) {}
        int freeind;
        IKReal fmul, foffset; ///< joint value is fmul*sol[freeind]+foffset
    };

    std::vector<VARIABLE> basesol;       ///< solution and their offsets if joints are mimiced
    std::vector<int> vfree;
};

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

inline float IKasin(float f)
{
assert( f > -1.001f && f < 1.001f ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asinf(f);
}
inline double IKasin(double f)
{
assert( f > -1.001 && f < 1.001 ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline float IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
assert( f > -1.001f && f < 1.001f ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0.0f;
return acosf(f);
}
inline double IKacos(double f)
{
assert( f > -1.001 && f < 1.001 ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0.0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        assert(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) )
        return 0;
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        assert(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) )
        return 0;
    return atan2(fy,fx);
}

IKFAST_API int getNumFreeParameters() { return 1; }
IKFAST_API int* getFreeParameters() { static int freeparams[] = {2}; return freeparams; }
IKFAST_API int getNumJoints() { return 7; }

IKFAST_API int getIKRealSize() { return sizeof(IKReal); }

IKFAST_API int getIKType() { return 1; }

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void fk(const IKReal* j, IKReal* eetrans, IKReal* eerot) {
    IKReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13;
    x0=IKcos(j[0]);
    x1=IKcos(j[1]);
    x2=IKcos(j[2]);
    x3=IKcos(j[3]);
    x4=IKsin(j[4]);
    x5=IKsin(j[6]);
    x6=IKcos(j[4]);
    x7=IKcos(j[5]);
    x8=IKcos(j[6]);
    x9=IKsin(j[0]);
    x10=IKsin(j[2]);
    x11=IKsin(j[3]);
    x12=IKsin(j[1]);
    x13=IKsin(j[5]);
    eerot[0]=((((x0)*(x1)*(x2)*(x3)*(x4)*(x5)))+(((x2)*(x5)*(x6)*(x9)))+(((x0)*(x1)*(x2)*(x3)*(x6)*(x7)*(x8)))+(((-1.00000000000000)*(x0)*(x12)*(x13)*(x3)*(x8)))+(((x10)*(x3)*(x4)*(x5)*(x9)))+(((-1.00000000000000)*(x0)*(x1)*(x11)*(x13)*(x2)*(x8)))+(((-1.00000000000000)*(x2)*(x4)*(x7)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x1)*(x10)*(x5)*(x6)))+(((x0)*(x1)*(x10)*(x4)*(x7)*(x8)))+(((-1.00000000000000)*(x10)*(x11)*(x13)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x11)*(x12)*(x6)*(x7)*(x8)))+(((-1.00000000000000)*(x0)*(x11)*(x12)*(x4)*(x5)))+(((x10)*(x3)*(x6)*(x7)*(x8)*(x9))));
    eerot[1]=((((-1.00000000000000)*(x0)*(x11)*(x12)*(x5)*(x6)*(x7)))+(((x10)*(x3)*(x5)*(x6)*(x7)*(x9)))+(((x0)*(x1)*(x2)*(x3)*(x5)*(x6)*(x7)))+(((-1.00000000000000)*(x0)*(x12)*(x13)*(x3)*(x5)))+(((x0)*(x1)*(x10)*(x4)*(x5)*(x7)))+(((x0)*(x11)*(x12)*(x4)*(x8)))+(((x0)*(x1)*(x10)*(x6)*(x8)))+(((-1.00000000000000)*(x0)*(x1)*(x11)*(x13)*(x2)*(x5)))+(((-1.00000000000000)*(x2)*(x4)*(x5)*(x7)*(x9)))+(((-1.00000000000000)*(x2)*(x6)*(x8)*(x9)))+(((-1.00000000000000)*(x10)*(x11)*(x13)*(x5)*(x9)))+(((-1.00000000000000)*(x10)*(x3)*(x4)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x1)*(x2)*(x3)*(x4)*(x8))));
    eerot[2]=((((x10)*(x13)*(x3)*(x6)*(x9)))+(((x0)*(x1)*(x11)*(x2)*(x7)))+(((x0)*(x12)*(x3)*(x7)))+(((-1.00000000000000)*(x13)*(x2)*(x4)*(x9)))+(((x10)*(x11)*(x7)*(x9)))+(((x0)*(x1)*(x10)*(x13)*(x4)))+(((-1.00000000000000)*(x0)*(x11)*(x12)*(x13)*(x6)))+(((x0)*(x1)*(x13)*(x2)*(x3)*(x6))));
    eetrans[0]=((0.220000000000000)+(((0.300000000000000)*(x10)*(x11)*(x9)))+(((-0.0450000000000000)*(x10)*(x3)*(x9)))+(((0.220000000000000)*(x0)*(x1)*(x11)*(x2)*(x7)))+(((-0.220000000000000)*(x0)*(x11)*(x12)*(x13)*(x6)))+(((0.220000000000000)*(x10)*(x11)*(x7)*(x9)))+(((-0.220000000000000)*(x13)*(x2)*(x4)*(x9)))+(((0.300000000000000)*(x0)*(x1)*(x11)*(x2)))+(((0.0450000000000000)*(x10)*(x9)))+(((0.220000000000000)*(x0)*(x1)*(x10)*(x13)*(x4)))+(((-0.0450000000000000)*(x0)*(x1)*(x2)*(x3)))+(((0.220000000000000)*(x0)*(x1)*(x13)*(x2)*(x3)*(x6)))+(((0.0450000000000000)*(x0)*(x1)*(x2)))+(((0.220000000000000)*(x10)*(x13)*(x3)*(x6)*(x9)))+(((0.0450000000000000)*(x0)*(x11)*(x12)))+(((0.300000000000000)*(x0)*(x12)*(x3)))+(((0.550000000000000)*(x0)*(x12)))+(((0.220000000000000)*(x0)*(x12)*(x3)*(x7))));
    eerot[3]=((((-1.00000000000000)*(x0)*(x2)*(x5)*(x6)))+(((x0)*(x10)*(x11)*(x13)*(x8)))+(((-1.00000000000000)*(x1)*(x10)*(x5)*(x6)*(x9)))+(((x1)*(x2)*(x3)*(x6)*(x7)*(x8)*(x9)))+(((x0)*(x2)*(x4)*(x7)*(x8)))+(((-1.00000000000000)*(x12)*(x13)*(x3)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x3)*(x6)*(x7)*(x8)))+(((x1)*(x10)*(x4)*(x7)*(x8)*(x9)))+(((-1.00000000000000)*(x11)*(x12)*(x6)*(x7)*(x8)*(x9)))+(((x1)*(x2)*(x3)*(x4)*(x5)*(x9)))+(((-1.00000000000000)*(x1)*(x11)*(x13)*(x2)*(x8)*(x9)))+(((-1.00000000000000)*(x11)*(x12)*(x4)*(x5)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x3)*(x4)*(x5))));
    eerot[4]=((((x0)*(x10)*(x3)*(x4)*(x8)))+(((-1.00000000000000)*(x1)*(x11)*(x13)*(x2)*(x5)*(x9)))+(((x11)*(x12)*(x4)*(x8)*(x9)))+(((x0)*(x10)*(x11)*(x13)*(x5)))+(((x1)*(x10)*(x4)*(x5)*(x7)*(x9)))+(((x1)*(x2)*(x3)*(x5)*(x6)*(x7)*(x9)))+(((x0)*(x2)*(x6)*(x8)))+(((-1.00000000000000)*(x12)*(x13)*(x3)*(x5)*(x9)))+(((-1.00000000000000)*(x1)*(x2)*(x3)*(x4)*(x8)*(x9)))+(((x1)*(x10)*(x6)*(x8)*(x9)))+(((x0)*(x2)*(x4)*(x5)*(x7)))+(((-1.00000000000000)*(x11)*(x12)*(x5)*(x6)*(x7)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x3)*(x5)*(x6)*(x7))));
    eerot[5]=((((-1.00000000000000)*(x11)*(x12)*(x13)*(x6)*(x9)))+(((x1)*(x13)*(x2)*(x3)*(x6)*(x9)))+(((x1)*(x11)*(x2)*(x7)*(x9)))+(((x0)*(x13)*(x2)*(x4)))+(((-1.00000000000000)*(x0)*(x10)*(x13)*(x3)*(x6)))+(((x12)*(x3)*(x7)*(x9)))+(((x1)*(x10)*(x13)*(x4)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x11)*(x7))));
    eetrans[1]=((0.140000000000000)+(((0.0450000000000000)*(x11)*(x12)*(x9)))+(((0.220000000000000)*(x1)*(x11)*(x2)*(x7)*(x9)))+(((-0.220000000000000)*(x11)*(x12)*(x13)*(x6)*(x9)))+(((-0.220000000000000)*(x0)*(x10)*(x11)*(x7)))+(((-0.0450000000000000)*(x1)*(x2)*(x3)*(x9)))+(((0.300000000000000)*(x12)*(x3)*(x9)))+(((0.300000000000000)*(x1)*(x11)*(x2)*(x9)))+(((-0.300000000000000)*(x0)*(x10)*(x11)))+(((0.220000000000000)*(x1)*(x13)*(x2)*(x3)*(x6)*(x9)))+(((0.220000000000000)*(x12)*(x3)*(x7)*(x9)))+(((0.220000000000000)*(x1)*(x10)*(x13)*(x4)*(x9)))+(((0.550000000000000)*(x12)*(x9)))+(((0.0450000000000000)*(x0)*(x10)*(x3)))+(((-0.220000000000000)*(x0)*(x10)*(x13)*(x3)*(x6)))+(((0.220000000000000)*(x0)*(x13)*(x2)*(x4)))+(((0.0450000000000000)*(x1)*(x2)*(x9)))+(((-0.0450000000000000)*(x0)*(x10))));
    eerot[6]=((((-1.00000000000000)*(x12)*(x2)*(x3)*(x6)*(x7)*(x8)))+(((-1.00000000000000)*(x10)*(x12)*(x4)*(x7)*(x8)))+(((-1.00000000000000)*(x1)*(x13)*(x3)*(x8)))+(((-1.00000000000000)*(x1)*(x11)*(x4)*(x5)))+(((-1.00000000000000)*(x12)*(x2)*(x3)*(x4)*(x5)))+(((-1.00000000000000)*(x1)*(x11)*(x6)*(x7)*(x8)))+(((x11)*(x12)*(x13)*(x2)*(x8)))+(((x10)*(x12)*(x5)*(x6))));
    eerot[7]=((((-1.00000000000000)*(x10)*(x12)*(x4)*(x5)*(x7)))+(((-1.00000000000000)*(x1)*(x13)*(x3)*(x5)))+(((-1.00000000000000)*(x10)*(x12)*(x6)*(x8)))+(((x12)*(x2)*(x3)*(x4)*(x8)))+(((x11)*(x12)*(x13)*(x2)*(x5)))+(((-1.00000000000000)*(x1)*(x11)*(x5)*(x6)*(x7)))+(((x1)*(x11)*(x4)*(x8)))+(((-1.00000000000000)*(x12)*(x2)*(x3)*(x5)*(x6)*(x7))));
    eerot[8]=((((-1.00000000000000)*(x1)*(x11)*(x13)*(x6)))+(((x1)*(x3)*(x7)))+(((-1.00000000000000)*(x11)*(x12)*(x2)*(x7)))+(((-1.00000000000000)*(x10)*(x12)*(x13)*(x4)))+(((-1.00000000000000)*(x12)*(x13)*(x2)*(x3)*(x6))));
    eetrans[2]=((0.346000000000000)+(((-0.220000000000000)*(x1)*(x11)*(x13)*(x6)))+(((-0.220000000000000)*(x10)*(x12)*(x13)*(x4)))+(((0.300000000000000)*(x1)*(x3)))+(((0.550000000000000)*(x1)))+(((-0.220000000000000)*(x12)*(x13)*(x2)*(x3)*(x6)))+(((-0.300000000000000)*(x11)*(x12)*(x2)))+(((0.0450000000000000)*(x12)*(x2)*(x3)))+(((0.220000000000000)*(x1)*(x3)*(x7)))+(((-0.0450000000000000)*(x12)*(x2)))+(((-0.220000000000000)*(x11)*(x12)*(x2)*(x7)))+(((0.0450000000000000)*(x1)*(x11))));
}

class IKSolver {
public:
IKReal j0, cj0, sj0,
j1, cj1, sj1,
j3, cj3, sj3,
j4, cj4, sj4,
j5, cj5, sj5,
j6, cj6, sj6,
j2, cj2, sj2,
new_r00, r00,
new_r01, r01,
new_r02, r02,
new_r10, r10,
new_r11, r11,
new_r12, r12,
new_r20, r20,
new_r21, r21,
new_r22, r22,
new_px, new_py, new_pz, px, py, pz;

bool ik(const IKReal* eetrans, const IKReal* eerot, const IKReal* pfree, std::vector<IKSolution>& vsolutions) {
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    vsolutions.resize(0); vsolutions.reserve(8);
    j2=pfree[0]; cj2=cos(pfree[0]); sj2=sin(pfree[0]);
    r00 = eerot[0*3+0];
    r01 = eerot[0*3+1];
    r02 = eerot[0*3+2];
    r10 = eerot[1*3+0];
    r11 = eerot[1*3+1];
    r12 = eerot[1*3+2];
    r20 = eerot[2*3+0];
    r21 = eerot[2*3+1];
    r22 = eerot[2*3+2];
    px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
    
    new_r00=r00;
    new_r01=((-1.00000000000000)*(r02));
    new_r02=r01;
    new_px=((-0.220000000000000)+(px)+(((-0.220000000000000)*(r02))));
    new_r10=r10;
    new_r11=((-1.00000000000000)*(r12));
    new_r12=r11;
    new_py=((-0.140000000000000)+(py)+(((-0.220000000000000)*(r12))));
    new_r20=r20;
    new_r21=((-1.00000000000000)*(r22));
    new_r22=r21;
    new_pz=((-0.346000000000000)+(pz)+(((-0.220000000000000)*(r22))));
    r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
    {
        if( 1 )
        {
        {
        IKReal j3array[2], cj3array[2], sj3array[2];
        bool j3valid[2]={false};
        IKReal x0=(pz)*(pz);
        IKReal x1=((2.98679637348110)*(x0));
        IKReal x2=(py)*(py);
        IKReal x3=((2.98679637348110)*(x2));
        IKReal x4=(px)*(px);
        IKReal x5=((2.98679637348110)*(x4));
        IKReal x6=((-1.18441410190393)+(x3)+(x1)+(x5));
        if( (x6) < -1.0001 || (x6) > 1.0001 )
            continue;
        IKReal x7=IKasin(x6);
        j3array[0]=((-1.34027003705633)+(x7));
        sj3array[0]=IKsin(j3array[0]);
        cj3array[0]=IKcos(j3array[0]);
        j3array[1]=((1.80132261653346)+(((-1.00000000000000)*(x7))));
        sj3array[1]=IKsin(j3array[1]);
        cj3array[1]=IKcos(j3array[1]);
        if( j3array[0] > IKPI )
            j3array[0]-=IK2PI;
        else if( j3array[0] < -IKPI )
            j3array[0]+=IK2PI;
        j3valid[0] = true;
        if( j3array[1] > IKPI )
            j3array[1]-=IK2PI;
        else if( j3array[1] < -IKPI )
            j3array[1]+=IK2PI;
        j3valid[1] = true;
        if( j3valid[0] && j3valid[1] && IKabs(cj3array[0]-cj3array[1]) < 0.0001 && IKabs(sj3array[0]-sj3array[1]) < 0.0001 )
            j3valid[1]=false;
        for(int ij3 = 0; ij3 < 2; ++ij3)
        {
        if( !j3valid[ij3] ) { continue; }
        j3 = j3array[ij3]; cj3 = cj3array[ij3]; sj3 = sj3array[ij3];
        
        {
        IKReal j0array[2], cj0array[2], sj0array[2];
        bool j0valid[2]={false};
        IKReal x8=x4;
        IKReal x9=x2;
        IKReal x10=((x9)+(x8));
        if( (x10) < (IKReal)-0.00001 )
            continue;
        IKReal x11=IKsqrt(x10);
        IKReal x12=IKabs(x11);
        IKReal x13=((IKabs(x12) != 0)?((IKReal)1/(x12)):(IKReal)1.0e30);
        IKReal x14=((0.0450000000000000)*(cj3)*(sj2));
        IKReal x15=((0.0450000000000000)*(sj2));
        IKReal x16=((0.300000000000000)*(sj2)*(sj3));
        IKReal x17=((x15)+(x16));
        IKReal x18=((((-1.00000000000000)*(x17)))+(x14));
        IKReal x19=((x13)*(x18));
        if( (x19) < -1.0001 || (x19) > 1.0001 )
            continue;
        IKReal x20=IKasin(x19);
        IKReal x21=((-1.00000000000000)*(py));
        IKReal x22=IKatan2(x21, px);
        j0array[0]=((((-1.00000000000000)*(x20)))+(((-1.00000000000000)*(x22))));
        sj0array[0]=IKsin(j0array[0]);
        cj0array[0]=IKcos(j0array[0]);
        j0array[1]=((3.14159265358979)+(((-1.00000000000000)*(x22)))+(x20));
        sj0array[1]=IKsin(j0array[1]);
        cj0array[1]=IKcos(j0array[1]);
        if( j0array[0] > IKPI )
            j0array[0]-=IK2PI;
        else if( j0array[0] < -IKPI )
            j0array[0]+=IK2PI;
        j0valid[0] = true;
        if( j0array[1] > IKPI )
            j0array[1]-=IK2PI;
        else if( j0array[1] < -IKPI )
            j0array[1]+=IK2PI;
        j0valid[1] = true;
        if( j0valid[0] && j0valid[1] && IKabs(cj0array[0]-cj0array[1]) < 0.0001 && IKabs(sj0array[0]-sj0array[1]) < 0.0001 )
            j0valid[1]=false;
        for(int ij0 = 0; ij0 < 2; ++ij0)
        {
        if( !j0valid[ij0] ) { continue; }
        j0 = j0array[ij0]; cj0 = cj0array[ij0]; sj0 = sj0array[ij0];
        
        {
            IKReal j1eval[1];
            j1eval[0]=(((((py)*(py))*((sj0)*(sj0))))+(((2.00000000000000)*(cj0)*(px)*(py)*(sj0)))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px)))));
            if( IKabs(j1eval[0]) < 0.000001  )
            {
                {
                    if( 1 )
                    {
                    {
                        IKReal j1eval[1];
                        j1eval[0]=((((sj2)*((pz)*(pz))))+(((sj2)*((cj0)*(cj0))*((px)*(px))))+(((2.00000000000000)*(cj0)*(px)*(py)*(sj0)*(sj2)))+(((sj2)*((py)*(py))*((sj0)*(sj0)))));
                        if( IKabs(j1eval[0]) < 0.000001  )
                        {
                            {
                            IKReal evalcond[4];
                                IKReal x23=((px)*(sj0));
                                IKReal x24=((cj0)*(py));
                                IKReal x25=((((-1.00000000000000)*(x24)))+(x23));
                                evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                evalcond[2]=x25;
                                evalcond[3]=x25;
                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                {
                                {
                                    IKReal j1eval[1];
                                    j1eval[0]=(((((py)*(py))*((sj0)*(sj0))))+(((2.00000000000000)*(cj0)*(px)*(py)*(sj0)))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px)))));
                                    if( IKabs(j1eval[0]) < 0.000001  )
                                    {
                                        {
                                            if( 1 )
                                            {
                                            {
                                                IKReal j1eval[1];
                                                j1eval[0]=((((-0.0450000000000000)*(pz)))+(((-0.300000000000000)*(cj3)*(py)*(sj0)))+(((-0.300000000000000)*(cj0)*(cj3)*(px)))+(((-0.0450000000000000)*(cj0)*(px)*(sj3)))+(((-0.300000000000000)*(pz)*(sj3)))+(((-0.550000000000000)*(cj0)*(px)))+(((-0.550000000000000)*(py)*(sj0)))+(((-0.0450000000000000)*(py)*(sj0)*(sj3)))+(((0.0450000000000000)*(cj3)*(pz))));
                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j1eval[1];
                                                            j1eval[0]=((((0.550000000000000)*(pz)))+(((-0.300000000000000)*(py)*(sj0)*(sj3)))+(((-0.300000000000000)*(cj0)*(px)*(sj3)))+(((-0.0450000000000000)*(py)*(sj0)))+(((0.0450000000000000)*(cj3)*(py)*(sj0)))+(((0.0450000000000000)*(cj0)*(cj3)*(px)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3)))+(((-0.0450000000000000)*(cj0)*(px))));
                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                IKReal evalcond[4];
                                                                    IKReal x26=x23;
                                                                    IKReal x27=x24;
                                                                    IKReal x28=((((-1.00000000000000)*(x27)))+(x26));
                                                                    evalcond[0]=((IKabs(((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979)))))))+(IKabs(pz)));
                                                                    evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                    evalcond[2]=x28;
                                                                    evalcond[3]=x28;
                                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                                    {
                                                                    {
                                                                        IKReal j1eval[1];
                                                                        j1eval[0]=((((0.0765000000000000)*(py)*(sj0)))+(((0.0765000000000000)*(cj0)*(px))));
                                                                        if( IKabs(j1eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                {
                                                                                    IKReal j1eval[2];
                                                                                    j1eval[0]=cj0;
                                                                                    j1eval[1]=((((0.0765000000000000)*(px)*((cj0)*(cj0))))+(((0.0765000000000000)*(cj0)*(py)*(sj0))));
                                                                                    if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                        IKReal evalcond[5];
                                                                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                            evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                            evalcond[2]=px;
                                                                                            evalcond[3]=px;
                                                                                            evalcond[4]=((-1.00000000000000)*(px));
                                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=py;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=py;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((0.850000000000000)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))), ((-11.1111111111111)*(((IKabs(py) != 0)?(pow(py,-2.00000000000000)):(IKReal)1.0e30))*(((((0.935000000000000)*(py)))+(((-1.00000000000000)*(py)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                    bool j1valid[1]={false};
                                                                                                    j1array[0]=IKatan2(((1.17647058823529)*(py)), ((13.0718954248366)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((px)*(px))))+(((-0.250000000000000)*((py)*(py))))))));
                                                                                                    sj1array[0]=IKsin(j1array[0]);
                                                                                                    cj1array[0]=IKcos(j1array[0]);
                                                                                                    if( j1array[0] > IKPI )
                                                                                                        j1array[0]-=IK2PI;
                                                                                                    else if( j1array[0] < -IKPI )
                                                                                                        j1array[0]+=IK2PI;
                                                                                                    j1valid[0] = true;
                                                                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                            IKReal x70=((-1.00000000000000)*(px));
                                                                                            evalcond[0]=((IKfmod(((1.57079632679490)+(3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                            evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                            evalcond[2]=x70;
                                                                                            evalcond[3]=x70;
                                                                                            evalcond[4]=x70;
                                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=py;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=py;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((-0.850000000000000)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))), ((-11.1111111111111)*(((IKabs(py) != 0)?(pow(py,-2.00000000000000)):(IKReal)1.0e30))*(((((-0.935000000000000)*(py)))+(((py)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                    bool j1valid[1]={false};
                                                                                                    j1array[0]=IKatan2(((-1.17647058823529)*(py)), ((-13.0718954248366)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((px)*(px))))+(((-0.250000000000000)*((py)*(py))))))));
                                                                                                    sj1array[0]=IKsin(j1array[0]);
                                                                                                    cj1array[0]=IKcos(j1array[0]);
                                                                                                    if( j1array[0] > IKPI )
                                                                                                        j1array[0]-=IK2PI;
                                                                                                    else if( j1array[0] < -IKPI )
                                                                                                        j1array[0]+=IK2PI;
                                                                                                    j1valid[0] = true;
                                                                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[2];
                                                                                                j1eval[0]=sj0;
                                                                                                j1eval[1]=((((0.0765000000000000)*(py)*((sj0)*(sj0))))+(((0.0765000000000000)*(cj0)*(px)*(sj0))));
                                                                                                if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                    IKReal evalcond[5];
                                                                                                        IKReal x71=x21;
                                                                                                        evalcond[0]=((IKfmod(((3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                                        evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                                        evalcond[2]=x71;
                                                                                                        evalcond[3]=x71;
                                                                                                        evalcond[4]=x71;
                                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=px;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    {
                                                                                                                        IKReal j1eval[1];
                                                                                                                        j1eval[0]=px;
                                                                                                                        if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                                        {
                                                                                                                            {
                                                                                                                                if( 1 )
                                                                                                                                {
                                                                                                                                continue;
                                                                                                                            
                                                                                                                            } else
                                                                                                                            {
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        } else
                                                                                                                        {
                                                                                                                            {
                                                                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                            bool j1valid[1]={false};
                                                                                                                            j1array[0]=IKatan2(((0.850000000000000)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))), ((-11.1111111111111)*(((IKabs(px) != 0)?(pow(px,-2.00000000000000)):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))+(((0.935000000000000)*(px)))))));
                                                                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                                                                            if( j1array[0] > IKPI )
                                                                                                                                j1array[0]-=IK2PI;
                                                                                                                            else if( j1array[0] < -IKPI )
                                                                                                                                j1array[0]+=IK2PI;
                                                                                                                            j1valid[0] = true;
                                                                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                            {
                                                                                                                            if( !j1valid[ij1] ) { continue; }
                                                                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                            
                                                                                                                            rotationfunction0(vsolutions);
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        }
                                                                                                                    
                                                                                                                    }
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((1.17647058823529)*(px)), ((13.0718954248366)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((py)*(py))))+(((-0.250000000000000)*((px)*(px))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                        evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                                        evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                                        evalcond[2]=py;
                                                                                                        evalcond[3]=py;
                                                                                                        evalcond[4]=((-1.00000000000000)*(py));
                                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=px;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    {
                                                                                                                        IKReal j1eval[1];
                                                                                                                        j1eval[0]=px;
                                                                                                                        if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                                        {
                                                                                                                            {
                                                                                                                                if( 1 )
                                                                                                                                {
                                                                                                                                continue;
                                                                                                                            
                                                                                                                            } else
                                                                                                                            {
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        } else
                                                                                                                        {
                                                                                                                            {
                                                                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                            bool j1valid[1]={false};
                                                                                                                            j1array[0]=IKatan2(((-0.850000000000000)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))), ((-11.1111111111111)*(((IKabs(px) != 0)?(pow(px,-2.00000000000000)):(IKReal)1.0e30))*(((((-0.935000000000000)*(px)))+(((px)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                                                                            if( j1array[0] > IKPI )
                                                                                                                                j1array[0]-=IK2PI;
                                                                                                                            else if( j1array[0] < -IKPI )
                                                                                                                                j1array[0]+=IK2PI;
                                                                                                                            j1valid[0] = true;
                                                                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                            {
                                                                                                                            if( !j1valid[ij1] ) { continue; }
                                                                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                            
                                                                                                                            rotationfunction0(vsolutions);
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        }
                                                                                                                    
                                                                                                                    }
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((-1.17647058823529)*(px)), ((-13.0718954248366)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((py)*(py))))+(((-0.250000000000000)*((px)*(px))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        continue;
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                    bool j1valid[1]={false};
                                                                                                    j1array[0]=IKatan2(((1.17647058823529)*(py)*(((IKabs(sj0) != 0)?((IKReal)1/(sj0)):(IKReal)1.0e30))), ((((IKabs(((((0.0765000000000000)*(py)*((sj0)*(sj0))))+(((0.0765000000000000)*(cj0)*(px)*(sj0))))) != 0)?((IKReal)1/(((((0.0765000000000000)*(py)*((sj0)*(sj0))))+(((0.0765000000000000)*(cj0)*(px)*(sj0)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(py)*(((((1.10000000000000)*(cj0)*(px)))+(((1.10000000000000)*(py)*(sj0)))))))+(((0.850000000000000)*(sj0)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                    sj1array[0]=IKsin(j1array[0]);
                                                                                                    cj1array[0]=IKcos(j1array[0]);
                                                                                                    if( j1array[0] > IKPI )
                                                                                                        j1array[0]-=IK2PI;
                                                                                                    else if( j1array[0] < -IKPI )
                                                                                                        j1array[0]+=IK2PI;
                                                                                                    j1valid[0] = true;
                                                                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                        bool j1valid[1]={false};
                                                                                        j1array[0]=IKatan2(((1.17647058823529)*(px)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))), ((((IKabs(((((0.0765000000000000)*(px)*((cj0)*(cj0))))+(((0.0765000000000000)*(cj0)*(py)*(sj0))))) != 0)?((IKReal)1/(((((0.0765000000000000)*(px)*((cj0)*(cj0))))+(((0.0765000000000000)*(cj0)*(py)*(sj0)))))):(IKReal)1.0e30))*(((((0.850000000000000)*(cj0)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))+(((-1.00000000000000)*(px)*(((((1.10000000000000)*(cj0)*(px)))+(((1.10000000000000)*(py)*(sj0)))))))))));
                                                                                        sj1array[0]=IKsin(j1array[0]);
                                                                                        cj1array[0]=IKcos(j1array[0]);
                                                                                        if( j1array[0] > IKPI )
                                                                                            j1array[0]-=IK2PI;
                                                                                        else if( j1array[0] < -IKPI )
                                                                                            j1array[0]+=IK2PI;
                                                                                        j1valid[0] = true;
                                                                                        for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                        {
                                                                                        if( !j1valid[ij1] ) { continue; }
                                                                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                            bool j1valid[1]={false};
                                                                            j1array[0]=IKatan2(((((1.17647058823529)*(py)*(sj0)))+(((1.17647058823529)*(cj0)*(px)))), ((((IKabs(((((0.0765000000000000)*(py)*(sj0)))+(((0.0765000000000000)*(cj0)*(px))))) != 0)?((IKReal)1/(((((0.0765000000000000)*(py)*(sj0)))+(((0.0765000000000000)*(cj0)*(px)))))):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((py)*(py))))+(((0.850000000000000)*((px)*(px))))+(((((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))*(((((1.10000000000000)*(cj0)*(px)))+(((1.10000000000000)*(py)*(sj0)))))))))));
                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                            if( j1array[0] > IKPI )
                                                                                j1array[0]-=IK2PI;
                                                                            else if( j1array[0] < -IKPI )
                                                                                j1array[0]+=IK2PI;
                                                                            j1valid[0] = true;
                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                            {
                                                                            if( !j1valid[ij1] ) { continue; }
                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    continue;
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                bool j1valid[1]={false};
                                                                IKReal x72=((0.300000000000000)*(sj3));
                                                                IKReal x73=((0.0450000000000000)+(x72));
                                                                IKReal x74=((0.0450000000000000)*(cj3));
                                                                IKReal x75=((((-1.00000000000000)*(x74)))+(x73));
                                                                IKReal x76=((0.550000000000000)*(pz));
                                                                IKReal x77=((1.00000000000000)*(py)*(sj0)*(x74));
                                                                IKReal x78=((1.00000000000000)*(cj0)*(px)*(x74));
                                                                IKReal x79=((0.300000000000000)*(cj3)*(pz));
                                                                IKReal x80=((0.0450000000000000)*(pz)*(sj3));
                                                                IKReal x81=((x77)+(x76)+(x79)+(x78)+(x80));
                                                                IKReal x82=((0.0450000000000000)*(cj0)*(px));
                                                                IKReal x83=((py)*(sj0)*(x72));
                                                                IKReal x84=((0.0450000000000000)*(py)*(sj0));
                                                                IKReal x85=((cj0)*(px)*(x72));
                                                                IKReal x86=((x82)+(x83)+(x84)+(x85));
                                                                IKReal x87=((x81)+(((-1.00000000000000)*(x86))));
                                                                IKReal x88=((IKabs(x87) != 0)?((IKReal)1/(x87)):(IKReal)1.0e30);
                                                                j1array[0]=IKatan2(((-1.00000000000000)*(x88)*(((((x75)*(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))))+(((pz)*(((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))))))), ((x88)*((((pz)*(pz))+(((-1.00000000000000)*((x75)*(x75))))))));
                                                                sj1array[0]=IKsin(j1array[0]);
                                                                cj1array[0]=IKcos(j1array[0]);
                                                                if( j1array[0] > IKPI )
                                                                    j1array[0]-=IK2PI;
                                                                else if( j1array[0] < -IKPI )
                                                                    j1array[0]+=IK2PI;
                                                                j1valid[0] = true;
                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                {
                                                                if( !j1valid[ij1] ) { continue; }
                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                    bool j1valid[1]={false};
                                                    IKReal x89=((0.0450000000000000)*(sj3));
                                                    IKReal x90=((0.300000000000000)*(cj3));
                                                    IKReal x91=((0.550000000000000)+(x89)+(x90));
                                                    IKReal x92=((0.0450000000000000)*(cj3)*(pz));
                                                    IKReal x93=((0.550000000000000)*(py)*(sj0));
                                                    IKReal x94=((1.00000000000000)*(cj0)*(px)*(x89));
                                                    IKReal x95=((0.300000000000000)*(pz)*(sj3));
                                                    IKReal x96=((1.00000000000000)*(py)*(sj0)*(x89));
                                                    IKReal x97=((0.550000000000000)*(cj0)*(px));
                                                    IKReal x98=((0.0450000000000000)*(pz));
                                                    IKReal x99=((py)*(sj0)*(x90));
                                                    IKReal x100=((cj0)*(px)*(x90));
                                                    IKReal x101=((x100)+(x99)+(x98)+(x93)+(x95)+(x94)+(x97)+(x96));
                                                    IKReal x102=((((-1.00000000000000)*(x101)))+(x92));
                                                    IKReal x103=((IKabs(x102) != 0)?((IKReal)1/(x102)):(IKReal)1.0e30);
                                                    j1array[0]=IKatan2(((x103)*((((pz)*(pz))+(((-1.00000000000000)*((x91)*(x91))))))), ((x103)*(((((-1.00000000000000)*(x91)*(((0.0450000000000000)+(((-0.0450000000000000)*(cj3)))+(((0.300000000000000)*(sj3)))))))+(((pz)*(((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))))))));
                                                    sj1array[0]=IKsin(j1array[0]);
                                                    cj1array[0]=IKcos(j1array[0]);
                                                    if( j1array[0] > IKPI )
                                                        j1array[0]-=IK2PI;
                                                    else if( j1array[0] < -IKPI )
                                                        j1array[0]+=IK2PI;
                                                    j1valid[0] = true;
                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                    {
                                                    if( !j1valid[ij1] ) { continue; }
                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                        }
                                        }
                                    
                                    } else
                                    {
                                        {
                                        IKReal j1array[1], cj1array[1], sj1array[1];
                                        bool j1valid[1]={false};
                                        IKReal x104=x2;
                                        IKReal x105=(sj0)*(sj0);
                                        IKReal x106=((x104)*(x105));
                                        IKReal x107=((2.00000000000000)*(cj0)*(px)*(py)*(sj0));
                                        IKReal x108=x0;
                                        IKReal x109=(cj0)*(cj0);
                                        IKReal x110=x4;
                                        IKReal x111=((x109)*(x110));
                                        IKReal x112=((x111)+(x108)+(x106)+(x107));
                                        IKReal x113=((IKabs(x112) != 0)?((IKReal)1/(x112)):(IKReal)1.0e30);
                                        IKReal x114=((0.0450000000000000)*(sj3));
                                        IKReal x115=((0.300000000000000)*(cj3));
                                        IKReal x116=((0.550000000000000)+(x115)+(x114));
                                        IKReal x117=((cj0)*(px));
                                        IKReal x118=((py)*(sj0));
                                        IKReal x119=((x117)+(x118));
                                        IKReal x120=((0.300000000000000)*(sj3));
                                        IKReal x121=((0.0450000000000000)+(x120));
                                        IKReal x122=((0.0450000000000000)*(cj3));
                                        IKReal x123=((((-1.00000000000000)*(x122)))+(x121));
                                        j1array[0]=IKatan2(((x113)*(((((-1.00000000000000)*(pz)*(x123)))+(((x116)*(x119)))))), ((x113)*(((((pz)*(x116)))+(((x119)*(x123)))))));
                                        sj1array[0]=IKsin(j1array[0]);
                                        cj1array[0]=IKcos(j1array[0]);
                                        if( j1array[0] > IKPI )
                                            j1array[0]-=IK2PI;
                                        else if( j1array[0] < -IKPI )
                                            j1array[0]+=IK2PI;
                                        j1valid[0] = true;
                                        for(int ij1 = 0; ij1 < 1; ++ij1)
                                        {
                                        if( !j1valid[ij1] ) { continue; }
                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                        
                                        rotationfunction0(vsolutions);
                                        }
                                        }
                                    
                                    }
                                
                                }
                            
                            } else
                            {
                                IKReal x124=((cj0)*(py));
                                IKReal x125=((px)*(sj0));
                                evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                evalcond[2]=((((-1.00000000000000)*(x124)))+(x125));
                                evalcond[3]=((((-1.00000000000000)*(x125)))+(x124));
                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                {
                                {
                                    IKReal j1eval[1];
                                    j1eval[0]=(((((py)*(py))*((sj0)*(sj0))))+(((2.00000000000000)*(cj0)*(px)*(py)*(sj0)))+((pz)*(pz))+((((cj0)*(cj0))*((px)*(px)))));
                                    if( IKabs(j1eval[0]) < 0.000001  )
                                    {
                                        {
                                            if( 1 )
                                            {
                                            {
                                                IKReal j1eval[1];
                                                j1eval[0]=((((-0.300000000000000)*(cj3)*(py)*(sj0)))+(((-0.300000000000000)*(cj0)*(cj3)*(px)))+(((-0.0450000000000000)*(cj0)*(px)*(sj3)))+(((0.300000000000000)*(pz)*(sj3)))+(((-0.0450000000000000)*(cj3)*(pz)))+(((-0.550000000000000)*(cj0)*(px)))+(((-0.550000000000000)*(py)*(sj0)))+(((0.0450000000000000)*(pz)))+(((-0.0450000000000000)*(py)*(sj0)*(sj3))));
                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j1eval[1];
                                                            j1eval[0]=((((0.550000000000000)*(pz)))+(((0.0450000000000000)*(cj0)*(px)))+(((0.300000000000000)*(py)*(sj0)*(sj3)))+(((0.0450000000000000)*(py)*(sj0)))+(((0.300000000000000)*(cj0)*(px)*(sj3)))+(((-0.0450000000000000)*(cj0)*(cj3)*(px)))+(((0.300000000000000)*(cj3)*(pz)))+(((-0.0450000000000000)*(cj3)*(py)*(sj0)))+(((0.0450000000000000)*(pz)*(sj3))));
                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                IKReal evalcond[4];
                                                                    IKReal x126=x124;
                                                                    IKReal x127=x125;
                                                                    evalcond[0]=((IKabs(((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979)))))))+(IKabs(pz)));
                                                                    evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                    evalcond[2]=((((-1.00000000000000)*(x126)))+(x127));
                                                                    evalcond[3]=((((-1.00000000000000)*(x127)))+(x126));
                                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                                    {
                                                                    {
                                                                        IKReal j1eval[1];
                                                                        j1eval[0]=((((-0.0765000000000000)*(cj0)*(px)))+(((-0.0765000000000000)*(py)*(sj0))));
                                                                        if( IKabs(j1eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                {
                                                                                    IKReal j1eval[2];
                                                                                    j1eval[0]=cj0;
                                                                                    j1eval[1]=((((-0.0765000000000000)*(px)*((cj0)*(cj0))))+(((-0.0765000000000000)*(cj0)*(py)*(sj0))));
                                                                                    if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                        IKReal evalcond[4];
                                                                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                            evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                            evalcond[2]=px;
                                                                                            evalcond[3]=((-1.00000000000000)*(px));
                                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=py;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=py;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((0.850000000000000)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))), ((11.1111111111111)*(((IKabs(py) != 0)?(pow(py,-2.00000000000000)):(IKReal)1.0e30))*(((((0.935000000000000)*(py)))+(((-1.00000000000000)*(py)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                    bool j1valid[1]={false};
                                                                                                    j1array[0]=IKatan2(((1.17647058823529)*(py)), ((-13.0718954248366)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((px)*(px))))+(((-0.250000000000000)*((py)*(py))))))));
                                                                                                    sj1array[0]=IKsin(j1array[0]);
                                                                                                    cj1array[0]=IKcos(j1array[0]);
                                                                                                    if( j1array[0] > IKPI )
                                                                                                        j1array[0]-=IK2PI;
                                                                                                    else if( j1array[0] < -IKPI )
                                                                                                        j1array[0]+=IK2PI;
                                                                                                    j1valid[0] = true;
                                                                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                            evalcond[0]=((IKfmod(((1.57079632679490)+(3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                            evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                            evalcond[2]=((-1.00000000000000)*(px));
                                                                                            evalcond[3]=px;
                                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=py;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=py;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((-0.850000000000000)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))), ((11.1111111111111)*(((IKabs(py) != 0)?(pow(py,-2.00000000000000)):(IKReal)1.0e30))*(((((-0.935000000000000)*(py)))+(((py)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                    bool j1valid[1]={false};
                                                                                                    j1array[0]=IKatan2(((-1.17647058823529)*(py)), ((13.0718954248366)*(((IKabs(py) != 0)?((IKReal)1/(py)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((px)*(px))))+(((-0.250000000000000)*((py)*(py))))))));
                                                                                                    sj1array[0]=IKsin(j1array[0]);
                                                                                                    cj1array[0]=IKcos(j1array[0]);
                                                                                                    if( j1array[0] > IKPI )
                                                                                                        j1array[0]-=IK2PI;
                                                                                                    else if( j1array[0] < -IKPI )
                                                                                                        j1array[0]+=IK2PI;
                                                                                                    j1valid[0] = true;
                                                                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[2];
                                                                                                j1eval[0]=sj0;
                                                                                                j1eval[1]=((((-0.0765000000000000)*(py)*((sj0)*(sj0))))+(((-0.0765000000000000)*(cj0)*(px)*(sj0))));
                                                                                                if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                    IKReal evalcond[4];
                                                                                                        evalcond[0]=((IKfmod(((3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                                        evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                                        evalcond[2]=((-1.00000000000000)*(py));
                                                                                                        evalcond[3]=py;
                                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=px;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    {
                                                                                                                        IKReal j1eval[1];
                                                                                                                        j1eval[0]=px;
                                                                                                                        if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                                        {
                                                                                                                            {
                                                                                                                                if( 1 )
                                                                                                                                {
                                                                                                                                continue;
                                                                                                                            
                                                                                                                            } else
                                                                                                                            {
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        } else
                                                                                                                        {
                                                                                                                            {
                                                                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                            bool j1valid[1]={false};
                                                                                                                            j1array[0]=IKatan2(((0.850000000000000)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))), ((11.1111111111111)*(((IKabs(px) != 0)?(pow(px,-2.00000000000000)):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))+(((0.935000000000000)*(px)))))));
                                                                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                                                                            if( j1array[0] > IKPI )
                                                                                                                                j1array[0]-=IK2PI;
                                                                                                                            else if( j1array[0] < -IKPI )
                                                                                                                                j1array[0]+=IK2PI;
                                                                                                                            j1valid[0] = true;
                                                                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                            {
                                                                                                                            if( !j1valid[ij1] ) { continue; }
                                                                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                            
                                                                                                                            rotationfunction0(vsolutions);
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        }
                                                                                                                    
                                                                                                                    }
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((1.17647058823529)*(px)), ((-13.0718954248366)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((py)*(py))))+(((-0.250000000000000)*((px)*(px))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                        evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                                        evalcond[1]=((0.722500000000000)+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py)))));
                                                                                                        evalcond[2]=py;
                                                                                                        evalcond[3]=((-1.00000000000000)*(py));
                                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=px;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    {
                                                                                                                        IKReal j1eval[1];
                                                                                                                        j1eval[0]=px;
                                                                                                                        if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                                        {
                                                                                                                            {
                                                                                                                                if( 1 )
                                                                                                                                {
                                                                                                                                continue;
                                                                                                                            
                                                                                                                            } else
                                                                                                                            {
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        } else
                                                                                                                        {
                                                                                                                            {
                                                                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                            bool j1valid[1]={false};
                                                                                                                            j1array[0]=IKatan2(((-0.850000000000000)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))), ((11.1111111111111)*(((IKabs(px) != 0)?(pow(px,-2.00000000000000)):(IKReal)1.0e30))*(((((-0.935000000000000)*(px)))+(((px)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                                                                            if( j1array[0] > IKPI )
                                                                                                                                j1array[0]-=IK2PI;
                                                                                                                            else if( j1array[0] < -IKPI )
                                                                                                                                j1array[0]+=IK2PI;
                                                                                                                            j1valid[0] = true;
                                                                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                            {
                                                                                                                            if( !j1valid[ij1] ) { continue; }
                                                                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                            
                                                                                                                            rotationfunction0(vsolutions);
                                                                                                                            }
                                                                                                                            }
                                                                                                                        
                                                                                                                        }
                                                                                                                    
                                                                                                                    }
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                                bool j1valid[1]={false};
                                                                                                                j1array[0]=IKatan2(((-1.17647058823529)*(px)), ((13.0718954248366)*(((IKabs(px) != 0)?((IKReal)1/(px)):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((py)*(py))))+(((-0.250000000000000)*((px)*(px))))))));
                                                                                                                sj1array[0]=IKsin(j1array[0]);
                                                                                                                cj1array[0]=IKcos(j1array[0]);
                                                                                                                if( j1array[0] > IKPI )
                                                                                                                    j1array[0]-=IK2PI;
                                                                                                                else if( j1array[0] < -IKPI )
                                                                                                                    j1array[0]+=IK2PI;
                                                                                                                j1valid[0] = true;
                                                                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        continue;
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                                    bool j1valid[1]={false};
                                                                                                    j1array[0]=IKatan2(((1.17647058823529)*(py)*(((IKabs(sj0) != 0)?((IKReal)1/(sj0)):(IKReal)1.0e30))), ((((IKabs(((((-0.0765000000000000)*(py)*((sj0)*(sj0))))+(((-0.0765000000000000)*(cj0)*(px)*(sj0))))) != 0)?((IKReal)1/(((((-0.0765000000000000)*(py)*((sj0)*(sj0))))+(((-0.0765000000000000)*(cj0)*(px)*(sj0)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(py)*(((((1.10000000000000)*(cj0)*(px)))+(((1.10000000000000)*(py)*(sj0)))))))+(((0.850000000000000)*(sj0)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))))));
                                                                                                    sj1array[0]=IKsin(j1array[0]);
                                                                                                    cj1array[0]=IKcos(j1array[0]);
                                                                                                    if( j1array[0] > IKPI )
                                                                                                        j1array[0]-=IK2PI;
                                                                                                    else if( j1array[0] < -IKPI )
                                                                                                        j1array[0]+=IK2PI;
                                                                                                    j1valid[0] = true;
                                                                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j1array[1], cj1array[1], sj1array[1];
                                                                                        bool j1valid[1]={false};
                                                                                        j1array[0]=IKatan2(((1.17647058823529)*(px)*(((IKabs(cj0) != 0)?((IKReal)1/(cj0)):(IKReal)1.0e30))), ((((IKabs(((((-0.0765000000000000)*(px)*((cj0)*(cj0))))+(((-0.0765000000000000)*(cj0)*(py)*(sj0))))) != 0)?((IKReal)1/(((((-0.0765000000000000)*(px)*((cj0)*(cj0))))+(((-0.0765000000000000)*(cj0)*(py)*(sj0)))))):(IKReal)1.0e30))*(((((0.850000000000000)*(cj0)*(((0.212500000000000)+((px)*(px))+((py)*(py))))))+(((-1.00000000000000)*(px)*(((((1.10000000000000)*(cj0)*(px)))+(((1.10000000000000)*(py)*(sj0)))))))))));
                                                                                        sj1array[0]=IKsin(j1array[0]);
                                                                                        cj1array[0]=IKcos(j1array[0]);
                                                                                        if( j1array[0] > IKPI )
                                                                                            j1array[0]-=IK2PI;
                                                                                        else if( j1array[0] < -IKPI )
                                                                                            j1array[0]+=IK2PI;
                                                                                        j1valid[0] = true;
                                                                                        for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                                        {
                                                                                        if( !j1valid[ij1] ) { continue; }
                                                                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                            bool j1valid[1]={false};
                                                                            j1array[0]=IKatan2(((((1.17647058823529)*(py)*(sj0)))+(((1.17647058823529)*(cj0)*(px)))), ((((IKabs(((((-0.0765000000000000)*(cj0)*(px)))+(((-0.0765000000000000)*(py)*(sj0))))) != 0)?((IKReal)1/(((((-0.0765000000000000)*(cj0)*(px)))+(((-0.0765000000000000)*(py)*(sj0)))))):(IKReal)1.0e30))*(((0.180625000000000)+(((0.850000000000000)*((py)*(py))))+(((0.850000000000000)*((px)*(px))))+(((((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))*(((((1.10000000000000)*(cj0)*(px)))+(((1.10000000000000)*(py)*(sj0)))))))))));
                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                            if( j1array[0] > IKPI )
                                                                                j1array[0]-=IK2PI;
                                                                            else if( j1array[0] < -IKPI )
                                                                                j1array[0]+=IK2PI;
                                                                            j1valid[0] = true;
                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                            {
                                                                            if( !j1valid[ij1] ) { continue; }
                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    continue;
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                bool j1valid[1]={false};
                                                                IKReal x128=((0.300000000000000)*(sj3));
                                                                IKReal x129=((0.0450000000000000)+(x128));
                                                                IKReal x130=((0.0450000000000000)*(cj3));
                                                                IKReal x131=((x129)+(((-1.00000000000000)*(x130))));
                                                                IKReal x132=((0.550000000000000)*(pz));
                                                                IKReal x133=((0.0450000000000000)*(cj0)*(px));
                                                                IKReal x134=((py)*(sj0)*(x128));
                                                                IKReal x135=((0.0450000000000000)*(py)*(sj0));
                                                                IKReal x136=((cj0)*(px)*(x128));
                                                                IKReal x137=((0.300000000000000)*(cj3)*(pz));
                                                                IKReal x138=((0.0450000000000000)*(pz)*(sj3));
                                                                IKReal x139=((x135)+(x134)+(x137)+(x136)+(x133)+(x132)+(x138));
                                                                IKReal x140=((1.00000000000000)*(py)*(sj0)*(x130));
                                                                IKReal x141=((1.00000000000000)*(cj0)*(px)*(x130));
                                                                IKReal x142=((x140)+(x141));
                                                                IKReal x143=((((-1.00000000000000)*(x142)))+(x139));
                                                                IKReal x144=((IKabs(x143) != 0)?((IKReal)1/(x143)):(IKReal)1.0e30);
                                                                j1array[0]=IKatan2(((x144)*(((((-1.00000000000000)*(pz)*(((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))))+(((x131)*(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))))))), ((x144)*((((pz)*(pz))+(((-1.00000000000000)*((x131)*(x131))))))));
                                                                sj1array[0]=IKsin(j1array[0]);
                                                                cj1array[0]=IKcos(j1array[0]);
                                                                if( j1array[0] > IKPI )
                                                                    j1array[0]-=IK2PI;
                                                                else if( j1array[0] < -IKPI )
                                                                    j1array[0]+=IK2PI;
                                                                j1valid[0] = true;
                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                {
                                                                if( !j1valid[ij1] ) { continue; }
                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                    bool j1valid[1]={false};
                                                    IKReal x145=((0.0450000000000000)*(sj3));
                                                    IKReal x146=((0.300000000000000)*(cj3));
                                                    IKReal x147=((0.550000000000000)+(x145)+(x146));
                                                    IKReal x148=((0.300000000000000)*(pz)*(sj3));
                                                    IKReal x149=((0.0450000000000000)*(pz));
                                                    IKReal x150=((x148)+(x149));
                                                    IKReal x151=((0.550000000000000)*(py)*(sj0));
                                                    IKReal x152=((1.00000000000000)*(cj0)*(px)*(x145));
                                                    IKReal x153=((1.00000000000000)*(py)*(sj0)*(x145));
                                                    IKReal x154=((0.550000000000000)*(cj0)*(px));
                                                    IKReal x155=((1.00000000000000)*(cj3)*(x149));
                                                    IKReal x156=((py)*(sj0)*(x146));
                                                    IKReal x157=((cj0)*(px)*(x146));
                                                    IKReal x158=((x157)+(x156)+(x155)+(x154)+(x153)+(x152)+(x151));
                                                    IKReal x159=((((-1.00000000000000)*(x158)))+(x150));
                                                    IKReal x160=((IKabs(x159) != 0)?((IKReal)1/(x159)):(IKReal)1.0e30);
                                                    j1array[0]=IKatan2(((x160)*(((((-1.00000000000000)*((x147)*(x147))))+((pz)*(pz))))), ((x160)*(((((pz)*(((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))))+(((x147)*(((0.0450000000000000)+(((-0.0450000000000000)*(cj3)))+(((0.300000000000000)*(sj3)))))))))));
                                                    sj1array[0]=IKsin(j1array[0]);
                                                    cj1array[0]=IKcos(j1array[0]);
                                                    if( j1array[0] > IKPI )
                                                        j1array[0]-=IK2PI;
                                                    else if( j1array[0] < -IKPI )
                                                        j1array[0]+=IK2PI;
                                                    j1valid[0] = true;
                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                    {
                                                    if( !j1valid[ij1] ) { continue; }
                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                        }
                                        }
                                    
                                    } else
                                    {
                                        {
                                        IKReal j1array[1], cj1array[1], sj1array[1];
                                        bool j1valid[1]={false};
                                        IKReal x161=x2;
                                        IKReal x162=(sj0)*(sj0);
                                        IKReal x163=((x161)*(x162));
                                        IKReal x164=((2.00000000000000)*(cj0)*(px)*(py)*(sj0));
                                        IKReal x165=x0;
                                        IKReal x166=(cj0)*(cj0);
                                        IKReal x167=x4;
                                        IKReal x168=((x166)*(x167));
                                        IKReal x169=((x168)+(x163)+(x164)+(x165));
                                        IKReal x170=((IKabs(x169) != 0)?((IKReal)1/(x169)):(IKReal)1.0e30);
                                        IKReal x171=((cj0)*(px));
                                        IKReal x172=((py)*(sj0));
                                        IKReal x173=((x171)+(x172));
                                        IKReal x174=((0.300000000000000)*(sj3));
                                        IKReal x175=((0.0450000000000000)+(x174));
                                        IKReal x176=((0.0450000000000000)*(cj3));
                                        IKReal x177=((((-1.00000000000000)*(x176)))+(x175));
                                        IKReal x178=((0.0450000000000000)*(sj3));
                                        IKReal x179=((0.300000000000000)*(cj3));
                                        IKReal x180=((0.550000000000000)+(x179)+(x178));
                                        j1array[0]=IKatan2(((x170)*(((((pz)*(x177)))+(((x173)*(x180)))))), ((x170)*(((((pz)*(x180)))+(((-1.00000000000000)*(x173)*(x177)))))));
                                        sj1array[0]=IKsin(j1array[0]);
                                        cj1array[0]=IKcos(j1array[0]);
                                        if( j1array[0] > IKPI )
                                            j1array[0]-=IK2PI;
                                        else if( j1array[0] < -IKPI )
                                            j1array[0]+=IK2PI;
                                        j1valid[0] = true;
                                        for(int ij1 = 0; ij1 < 1; ++ij1)
                                        {
                                        if( !j1valid[ij1] ) { continue; }
                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                        
                                        rotationfunction0(vsolutions);
                                        }
                                        }
                                    
                                    }
                                
                                }
                            
                            } else
                            {
                                if( 1 )
                                {
                                {
                                    IKReal j1eval[1];
                                    j1eval[0]=((((cj2)*((py)*(py))*((sj0)*(sj0))))+(((cj2)*((cj0)*(cj0))*((px)*(px))))+(((2.00000000000000)*(cj0)*(cj2)*(px)*(py)*(sj0)))+(((cj2)*((pz)*(pz)))));
                                    if( IKabs(j1eval[0]) < 0.000001  )
                                    {
                                        {
                                        IKReal evalcond[4];
                                            IKReal x181=((0.300000000000000)*(sj3));
                                            IKReal x182=((cj0)*(py));
                                            IKReal x183=((0.0450000000000000)*(cj3));
                                            IKReal x184=((px)*(sj0));
                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                            evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                            evalcond[2]=((-0.0450000000000000)+(((-1.00000000000000)*(x182)))+(((-1.00000000000000)*(x181)))+(x184)+(x183));
                                            evalcond[3]=((0.0450000000000000)+(((-1.00000000000000)*(x184)))+(((-1.00000000000000)*(x183)))+(x181)+(x182));
                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                            {
                                            {
                                                IKReal j1eval[1];
                                                j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j1eval[2];
                                                            j1eval[0]=((((0.550000000000000)*(pz)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3))));
                                                            j1eval[1]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                            if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j1eval[2];
                                                                        j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                                        j1eval[1]=((((-0.300000000000000)*(cj3)*(py)*(sj0)))+(((-0.300000000000000)*(cj0)*(cj3)*(px)))+(((-0.0450000000000000)*(cj0)*(px)*(sj3)))+(((-0.550000000000000)*(cj0)*(px)))+(((-0.550000000000000)*(py)*(sj0)))+(((-0.0450000000000000)*(py)*(sj0)*(sj3))));
                                                                        if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                                        {
                                                                            {
                                                                            IKReal evalcond[5];
                                                                                IKReal x185=x181;
                                                                                IKReal x186=x183;
                                                                                IKReal x187=((-0.0450000000000000)+(x186));
                                                                                IKReal x188=((x185)+(py));
                                                                                IKReal x189=((((-1.00000000000000)*(x188)))+(x187));
                                                                                evalcond[0]=((IKabs(px))+(IKabs(((IKfmod(((3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979)))))));
                                                                                evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                                                                evalcond[2]=x189;
                                                                                evalcond[3]=((0.0450000000000000)+(((-1.00000000000000)*(x186)))+(x188));
                                                                                evalcond[4]=x189;
                                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                {
                                                                                {
                                                                                    IKReal j1eval[1];
                                                                                    j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                                                    if( IKabs(j1eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=pz;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=pz;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                                bool j1valid[2]={false};
                                                                                                                IKReal x190=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                                cj1array[0]=((((0.193181818181818)*(x190)))+(((0.0818181818181818)*(py)*(x190)))+(((0.909090909090909)*(x190)*((py)*(py))))+(((0.909090909090909)*(pz))));
                                                                                                                if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                                {
                                                                                                                    j1valid[0] = j1valid[1] = true;
                                                                                                                    j1array[0] = IKacos(cj1array[0]);
                                                                                                                    sj1array[0] = IKsin(j1array[0]);
                                                                                                                    cj1array[1] = cj1array[0];
                                                                                                                    j1array[1] = -j1array[0];
                                                                                                                    sj1array[1] = -sj1array[0];
                                                                                                                }
                                                                                                                else if( isnan(cj1array[0]) )
                                                                                                                {
                                                                                                                    // probably any value will work
                                                                                                                    j1valid[0] = true;
                                                                                                                    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                                }
                                                                                                                if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                                    j1valid[1]=false;
                                                                                                                for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                    bool j1valid[2]={false};
                                                                                                    IKReal x191=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                    cj1array[0]=((((0.0450000000000000)*(sj3)*(x191)))+(((0.300000000000000)*(cj3)*(x191)))+(((0.550000000000000)*(x191))));
                                                                                                    if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                    {
                                                                                                        j1valid[0] = j1valid[1] = true;
                                                                                                        j1array[0] = IKacos(cj1array[0]);
                                                                                                        sj1array[0] = IKsin(j1array[0]);
                                                                                                        cj1array[1] = cj1array[0];
                                                                                                        j1array[1] = -j1array[0];
                                                                                                        sj1array[1] = -sj1array[0];
                                                                                                    }
                                                                                                    else if( isnan(cj1array[0]) )
                                                                                                    {
                                                                                                        // probably any value will work
                                                                                                        j1valid[0] = true;
                                                                                                        cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                    }
                                                                                                    if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                        j1valid[1]=false;
                                                                                                    for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                        bool j1valid[2]={false};
                                                                                        cj1array[0]=((pz)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30)));
                                                                                        if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                        {
                                                                                            j1valid[0] = j1valid[1] = true;
                                                                                            j1array[0] = IKacos(cj1array[0]);
                                                                                            sj1array[0] = IKsin(j1array[0]);
                                                                                            cj1array[1] = cj1array[0];
                                                                                            j1array[1] = -j1array[0];
                                                                                            sj1array[1] = -sj1array[0];
                                                                                        }
                                                                                        else if( isnan(cj1array[0]) )
                                                                                        {
                                                                                            // probably any value will work
                                                                                            j1valid[0] = true;
                                                                                            cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                        }
                                                                                        if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                            j1valid[1]=false;
                                                                                        for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                        {
                                                                                        if( !j1valid[ij1] ) { continue; }
                                                                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                                IKReal x192=x181;
                                                                                IKReal x193=x183;
                                                                                IKReal x194=((0.0450000000000000)+(x192));
                                                                                IKReal x195=((x193)+(px));
                                                                                IKReal x196=((((-1.00000000000000)*(x195)))+(x194));
                                                                                evalcond[0]=((IKabs(((IKfmod(((((0.500000000000000)*(3.14159265358979)))+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))))))+(IKabs(py)));
                                                                                evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                                                                evalcond[2]=((-0.0450000000000000)+(((-1.00000000000000)*(x192)))+(x195));
                                                                                evalcond[3]=x196;
                                                                                evalcond[4]=x196;
                                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                {
                                                                                {
                                                                                    IKReal j1eval[1];
                                                                                    j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                                                    if( IKabs(j1eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=pz;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=pz;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                                bool j1valid[2]={false};
                                                                                                                IKReal x197=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                                cj1array[0]=((((0.193181818181818)*(x197)))+(((-0.0818181818181818)*(px)*(x197)))+(((0.909090909090909)*(x197)*((px)*(px))))+(((0.909090909090909)*(pz))));
                                                                                                                if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                                {
                                                                                                                    j1valid[0] = j1valid[1] = true;
                                                                                                                    j1array[0] = IKacos(cj1array[0]);
                                                                                                                    sj1array[0] = IKsin(j1array[0]);
                                                                                                                    cj1array[1] = cj1array[0];
                                                                                                                    j1array[1] = -j1array[0];
                                                                                                                    sj1array[1] = -sj1array[0];
                                                                                                                }
                                                                                                                else if( isnan(cj1array[0]) )
                                                                                                                {
                                                                                                                    // probably any value will work
                                                                                                                    j1valid[0] = true;
                                                                                                                    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                                }
                                                                                                                if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                                    j1valid[1]=false;
                                                                                                                for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                    bool j1valid[2]={false};
                                                                                                    IKReal x198=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                    cj1array[0]=((((0.0450000000000000)*(sj3)*(x198)))+(((0.300000000000000)*(cj3)*(x198)))+(((0.550000000000000)*(x198))));
                                                                                                    if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                    {
                                                                                                        j1valid[0] = j1valid[1] = true;
                                                                                                        j1array[0] = IKacos(cj1array[0]);
                                                                                                        sj1array[0] = IKsin(j1array[0]);
                                                                                                        cj1array[1] = cj1array[0];
                                                                                                        j1array[1] = -j1array[0];
                                                                                                        sj1array[1] = -sj1array[0];
                                                                                                    }
                                                                                                    else if( isnan(cj1array[0]) )
                                                                                                    {
                                                                                                        // probably any value will work
                                                                                                        j1valid[0] = true;
                                                                                                        cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                    }
                                                                                                    if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                        j1valid[1]=false;
                                                                                                    for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                        bool j1valid[2]={false};
                                                                                        cj1array[0]=((pz)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30)));
                                                                                        if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                        {
                                                                                            j1valid[0] = j1valid[1] = true;
                                                                                            j1array[0] = IKacos(cj1array[0]);
                                                                                            sj1array[0] = IKsin(j1array[0]);
                                                                                            cj1array[1] = cj1array[0];
                                                                                            j1array[1] = -j1array[0];
                                                                                            sj1array[1] = -sj1array[0];
                                                                                        }
                                                                                        else if( isnan(cj1array[0]) )
                                                                                        {
                                                                                            // probably any value will work
                                                                                            j1valid[0] = true;
                                                                                            cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                        }
                                                                                        if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                            j1valid[1]=false;
                                                                                        for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                        {
                                                                                        if( !j1valid[ij1] ) { continue; }
                                                                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                continue;
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                            bool j1valid[1]={false};
                                                                            IKReal x199=((cj0)*(px));
                                                                            IKReal x200=((py)*(sj0));
                                                                            IKReal x201=((x199)+(x200));
                                                                            j1array[0]=IKatan2(((x201)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30))), ((-1.00000000000000)*(pz)*(x201)*(((IKabs(((((-0.0450000000000000)*(sj3)*(x199)))+(((-0.300000000000000)*(cj3)*(x200)))+(((-0.550000000000000)*(x200)))+(((-0.300000000000000)*(cj3)*(x199)))+(((-0.0450000000000000)*(sj3)*(x200)))+(((-0.550000000000000)*(x199))))) != 0)?((IKReal)1/(((((-0.0450000000000000)*(sj3)*(x199)))+(((-0.300000000000000)*(cj3)*(x200)))+(((-0.550000000000000)*(x200)))+(((-0.300000000000000)*(cj3)*(x199)))+(((-0.0450000000000000)*(sj3)*(x200)))+(((-0.550000000000000)*(x199)))))):(IKReal)1.0e30))));
                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                            if( j1array[0] > IKPI )
                                                                                j1array[0]-=IK2PI;
                                                                            else if( j1array[0] < -IKPI )
                                                                                j1array[0]+=IK2PI;
                                                                            j1valid[0] = true;
                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                            {
                                                                            if( !j1valid[ij1] ) { continue; }
                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                bool j1valid[1]={false};
                                                                j1array[0]=IKatan2(((-1.00000000000000)*(pz)*(((IKabs(((((0.550000000000000)*(pz)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3))))) != 0)?((IKReal)1/(((((0.550000000000000)*(pz)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))), ((pz)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30))));
                                                                sj1array[0]=IKsin(j1array[0]);
                                                                cj1array[0]=IKcos(j1array[0]);
                                                                if( j1array[0] > IKPI )
                                                                    j1array[0]-=IK2PI;
                                                                else if( j1array[0] < -IKPI )
                                                                    j1array[0]+=IK2PI;
                                                                j1valid[0] = true;
                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                {
                                                                if( !j1valid[ij1] ) { continue; }
                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                    bool j1valid[1]={false};
                                                    IKReal x202=((0.0450000000000000)*(sj3));
                                                    IKReal x203=((0.300000000000000)*(cj3));
                                                    IKReal x204=((0.550000000000000)+(x203)+(x202));
                                                    IKReal x205=((IKabs(x204) != 0)?((IKReal)1/(x204)):(IKReal)1.0e30);
                                                    j1array[0]=IKatan2(((x205)*(((((cj0)*(px)))+(((py)*(sj0)))))), ((pz)*(x205)));
                                                    sj1array[0]=IKsin(j1array[0]);
                                                    cj1array[0]=IKcos(j1array[0]);
                                                    if( j1array[0] > IKPI )
                                                        j1array[0]-=IK2PI;
                                                    else if( j1array[0] < -IKPI )
                                                        j1array[0]+=IK2PI;
                                                    j1valid[0] = true;
                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                    {
                                                    if( !j1valid[ij1] ) { continue; }
                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                            IKReal x206=((0.300000000000000)*(sj3));
                                            IKReal x207=((px)*(sj0));
                                            IKReal x208=((0.0450000000000000)+(x207)+(x206));
                                            IKReal x209=((0.0450000000000000)*(cj3));
                                            IKReal x210=((cj0)*(py));
                                            IKReal x211=((x210)+(x209));
                                            IKReal x212=((x208)+(((-1.00000000000000)*(x211))));
                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                            evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                            evalcond[2]=x212;
                                            evalcond[3]=x212;
                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                            {
                                            {
                                                IKReal j1eval[1];
                                                j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j1eval[2];
                                                            j1eval[0]=((((0.550000000000000)*(pz)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3))));
                                                            j1eval[1]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                            if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j1eval[2];
                                                                        j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                                        j1eval[1]=((((-0.300000000000000)*(cj3)*(py)*(sj0)))+(((-0.300000000000000)*(cj0)*(cj3)*(px)))+(((-0.0450000000000000)*(cj0)*(px)*(sj3)))+(((-0.550000000000000)*(cj0)*(px)))+(((-0.550000000000000)*(py)*(sj0)))+(((-0.0450000000000000)*(py)*(sj0)*(sj3))));
                                                                        if( IKabs(j1eval[0]) < 0.000001  || IKabs(j1eval[1]) < 0.000001  )
                                                                        {
                                                                            {
                                                                            IKReal evalcond[5];
                                                                                IKReal x213=x206;
                                                                                IKReal x214=((0.0450000000000000)+(x213));
                                                                                IKReal x215=x209;
                                                                                IKReal x216=((x215)+(py));
                                                                                IKReal x217=((x214)+(((-1.00000000000000)*(x216))));
                                                                                evalcond[0]=((IKabs(px))+(IKabs(((IKfmod(((3.14159265358979)+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979)))))));
                                                                                evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                                                                evalcond[2]=x217;
                                                                                evalcond[3]=x217;
                                                                                evalcond[4]=x217;
                                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                {
                                                                                {
                                                                                    IKReal j1eval[1];
                                                                                    j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                                                    if( IKabs(j1eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=pz;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=pz;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                                bool j1valid[2]={false};
                                                                                                                IKReal x218=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                                cj1array[0]=((((-0.0818181818181818)*(py)*(x218)))+(((0.909090909090909)*(x218)*((py)*(py))))+(((0.193181818181818)*(x218)))+(((0.909090909090909)*(pz))));
                                                                                                                if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                                {
                                                                                                                    j1valid[0] = j1valid[1] = true;
                                                                                                                    j1array[0] = IKacos(cj1array[0]);
                                                                                                                    sj1array[0] = IKsin(j1array[0]);
                                                                                                                    cj1array[1] = cj1array[0];
                                                                                                                    j1array[1] = -j1array[0];
                                                                                                                    sj1array[1] = -sj1array[0];
                                                                                                                }
                                                                                                                else if( isnan(cj1array[0]) )
                                                                                                                {
                                                                                                                    // probably any value will work
                                                                                                                    j1valid[0] = true;
                                                                                                                    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                                }
                                                                                                                if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                                    j1valid[1]=false;
                                                                                                                for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                    bool j1valid[2]={false};
                                                                                                    IKReal x219=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                    cj1array[0]=((((0.550000000000000)*(x219)))+(((0.0450000000000000)*(sj3)*(x219)))+(((0.300000000000000)*(cj3)*(x219))));
                                                                                                    if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                    {
                                                                                                        j1valid[0] = j1valid[1] = true;
                                                                                                        j1array[0] = IKacos(cj1array[0]);
                                                                                                        sj1array[0] = IKsin(j1array[0]);
                                                                                                        cj1array[1] = cj1array[0];
                                                                                                        j1array[1] = -j1array[0];
                                                                                                        sj1array[1] = -sj1array[0];
                                                                                                    }
                                                                                                    else if( isnan(cj1array[0]) )
                                                                                                    {
                                                                                                        // probably any value will work
                                                                                                        j1valid[0] = true;
                                                                                                        cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                    }
                                                                                                    if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                        j1valid[1]=false;
                                                                                                    for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                        bool j1valid[2]={false};
                                                                                        cj1array[0]=((pz)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30)));
                                                                                        if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                        {
                                                                                            j1valid[0] = j1valid[1] = true;
                                                                                            j1array[0] = IKacos(cj1array[0]);
                                                                                            sj1array[0] = IKsin(j1array[0]);
                                                                                            cj1array[1] = cj1array[0];
                                                                                            j1array[1] = -j1array[0];
                                                                                            sj1array[1] = -sj1array[0];
                                                                                        }
                                                                                        else if( isnan(cj1array[0]) )
                                                                                        {
                                                                                            // probably any value will work
                                                                                            j1valid[0] = true;
                                                                                            cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                        }
                                                                                        if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                            j1valid[1]=false;
                                                                                        for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                        {
                                                                                        if( !j1valid[ij1] ) { continue; }
                                                                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                                IKReal x220=x206;
                                                                                IKReal x221=((0.0450000000000000)+(x220)+(px));
                                                                                IKReal x222=x209;
                                                                                IKReal x223=((x221)+(((-1.00000000000000)*(x222))));
                                                                                evalcond[0]=((IKabs(((IKfmod(((((0.500000000000000)*(3.14159265358979)))+(j0)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))))))+(IKabs(py)));
                                                                                evalcond[1]=((0.396550000000000)+(((0.0765000000000000)*(sj3)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((pz)*(pz))))+(((0.325950000000000)*(cj3))));
                                                                                evalcond[2]=x223;
                                                                                evalcond[3]=x223;
                                                                                evalcond[4]=((x222)+(((-1.00000000000000)*(x221))));
                                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  && IKabs(evalcond[4]) < 0.000010  )
                                                                                {
                                                                                {
                                                                                    IKReal j1eval[1];
                                                                                    j1eval[0]=((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))));
                                                                                    if( IKabs(j1eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            {
                                                                                                IKReal j1eval[1];
                                                                                                j1eval[0]=pz;
                                                                                                if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                {
                                                                                                    {
                                                                                                        if( 1 )
                                                                                                        {
                                                                                                        {
                                                                                                            IKReal j1eval[1];
                                                                                                            j1eval[0]=pz;
                                                                                                            if( IKabs(j1eval[0]) < 0.000001  )
                                                                                                            {
                                                                                                                {
                                                                                                                    if( 1 )
                                                                                                                    {
                                                                                                                    continue;
                                                                                                                
                                                                                                                } else
                                                                                                                {
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            } else
                                                                                                            {
                                                                                                                {
                                                                                                                IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                                bool j1valid[2]={false};
                                                                                                                IKReal x224=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                                cj1array[0]=((((0.193181818181818)*(x224)))+(((0.0818181818181818)*(px)*(x224)))+(((0.909090909090909)*(x224)*((px)*(px))))+(((0.909090909090909)*(pz))));
                                                                                                                if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                                {
                                                                                                                    j1valid[0] = j1valid[1] = true;
                                                                                                                    j1array[0] = IKacos(cj1array[0]);
                                                                                                                    sj1array[0] = IKsin(j1array[0]);
                                                                                                                    cj1array[1] = cj1array[0];
                                                                                                                    j1array[1] = -j1array[0];
                                                                                                                    sj1array[1] = -sj1array[0];
                                                                                                                }
                                                                                                                else if( isnan(cj1array[0]) )
                                                                                                                {
                                                                                                                    // probably any value will work
                                                                                                                    j1valid[0] = true;
                                                                                                                    cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                                }
                                                                                                                if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                                    j1valid[1]=false;
                                                                                                                for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                                {
                                                                                                                if( !j1valid[ij1] ) { continue; }
                                                                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                                
                                                                                                                rotationfunction0(vsolutions);
                                                                                                                }
                                                                                                                }
                                                                                                            
                                                                                                            }
                                                                                                        
                                                                                                        }
                                                                                                    
                                                                                                    } else
                                                                                                    {
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                    {
                                                                                                    IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                                    bool j1valid[2]={false};
                                                                                                    IKReal x225=((IKabs(pz) != 0)?((IKReal)1/(pz)):(IKReal)1.0e30);
                                                                                                    cj1array[0]=((((0.550000000000000)*(x225)))+(((0.0450000000000000)*(sj3)*(x225)))+(((0.300000000000000)*(cj3)*(x225))));
                                                                                                    if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                                    {
                                                                                                        j1valid[0] = j1valid[1] = true;
                                                                                                        j1array[0] = IKacos(cj1array[0]);
                                                                                                        sj1array[0] = IKsin(j1array[0]);
                                                                                                        cj1array[1] = cj1array[0];
                                                                                                        j1array[1] = -j1array[0];
                                                                                                        sj1array[1] = -sj1array[0];
                                                                                                    }
                                                                                                    else if( isnan(cj1array[0]) )
                                                                                                    {
                                                                                                        // probably any value will work
                                                                                                        j1valid[0] = true;
                                                                                                        cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                                    }
                                                                                                    if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                                        j1valid[1]=false;
                                                                                                    for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                                    {
                                                                                                    if( !j1valid[ij1] ) { continue; }
                                                                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                                    
                                                                                                    rotationfunction0(vsolutions);
                                                                                                    }
                                                                                                    }
                                                                                                
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j1array[2], cj1array[2], sj1array[2];
                                                                                        bool j1valid[2]={false};
                                                                                        cj1array[0]=((pz)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30)));
                                                                                        if( cj1array[0] >= -1.0001 && cj1array[0] <= 1.0001 )
                                                                                        {
                                                                                            j1valid[0] = j1valid[1] = true;
                                                                                            j1array[0] = IKacos(cj1array[0]);
                                                                                            sj1array[0] = IKsin(j1array[0]);
                                                                                            cj1array[1] = cj1array[0];
                                                                                            j1array[1] = -j1array[0];
                                                                                            sj1array[1] = -sj1array[0];
                                                                                        }
                                                                                        else if( isnan(cj1array[0]) )
                                                                                        {
                                                                                            // probably any value will work
                                                                                            j1valid[0] = true;
                                                                                            cj1array[0] = 1; sj1array[0] = 0; j1array[0] = 0;
                                                                                        }
                                                                                        if( j1valid[0] && j1valid[1] && IKabs(cj1array[0]-cj1array[1]) < 0.0001 && IKabs(sj1array[0]-sj1array[1]) < 0.0001 )
                                                                                            j1valid[1]=false;
                                                                                        for(int ij1 = 0; ij1 < 2; ++ij1)
                                                                                        {
                                                                                        if( !j1valid[ij1] ) { continue; }
                                                                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                continue;
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j1array[1], cj1array[1], sj1array[1];
                                                                            bool j1valid[1]={false};
                                                                            IKReal x226=((cj0)*(px));
                                                                            IKReal x227=((py)*(sj0));
                                                                            IKReal x228=((x227)+(x226));
                                                                            j1array[0]=IKatan2(((x228)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30))), ((-1.00000000000000)*(pz)*(x228)*(((IKabs(((((-0.0450000000000000)*(sj3)*(x227)))+(((-0.0450000000000000)*(sj3)*(x226)))+(((-0.550000000000000)*(x226)))+(((-0.550000000000000)*(x227)))+(((-0.300000000000000)*(cj3)*(x227)))+(((-0.300000000000000)*(cj3)*(x226))))) != 0)?((IKReal)1/(((((-0.0450000000000000)*(sj3)*(x227)))+(((-0.0450000000000000)*(sj3)*(x226)))+(((-0.550000000000000)*(x226)))+(((-0.550000000000000)*(x227)))+(((-0.300000000000000)*(cj3)*(x227)))+(((-0.300000000000000)*(cj3)*(x226)))))):(IKReal)1.0e30))));
                                                                            sj1array[0]=IKsin(j1array[0]);
                                                                            cj1array[0]=IKcos(j1array[0]);
                                                                            if( j1array[0] > IKPI )
                                                                                j1array[0]-=IK2PI;
                                                                            else if( j1array[0] < -IKPI )
                                                                                j1array[0]+=IK2PI;
                                                                            j1valid[0] = true;
                                                                            for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                            {
                                                                            if( !j1valid[ij1] ) { continue; }
                                                                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j1array[1], cj1array[1], sj1array[1];
                                                                bool j1valid[1]={false};
                                                                j1array[0]=IKatan2(((-1.00000000000000)*(pz)*(((IKabs(((((0.550000000000000)*(pz)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3))))) != 0)?((IKReal)1/(((((0.550000000000000)*(pz)))+(((0.300000000000000)*(cj3)*(pz)))+(((0.0450000000000000)*(pz)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(py)*(sj0)))+(((-1.00000000000000)*(cj0)*(px)))))), ((pz)*(((IKabs(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3))))) != 0)?((IKReal)1/(((0.550000000000000)+(((0.0450000000000000)*(sj3)))+(((0.300000000000000)*(cj3)))))):(IKReal)1.0e30))));
                                                                sj1array[0]=IKsin(j1array[0]);
                                                                cj1array[0]=IKcos(j1array[0]);
                                                                if( j1array[0] > IKPI )
                                                                    j1array[0]-=IK2PI;
                                                                else if( j1array[0] < -IKPI )
                                                                    j1array[0]+=IK2PI;
                                                                j1valid[0] = true;
                                                                for(int ij1 = 0; ij1 < 1; ++ij1)
                                                                {
                                                                if( !j1valid[ij1] ) { continue; }
                                                                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j1array[1], cj1array[1], sj1array[1];
                                                    bool j1valid[1]={false};
                                                    IKReal x229=((0.0450000000000000)*(sj3));
                                                    IKReal x230=((0.300000000000000)*(cj3));
                                                    IKReal x231=((0.550000000000000)+(x229)+(x230));
                                                    IKReal x232=((IKabs(x231) != 0)?((IKReal)1/(x231)):(IKReal)1.0e30);
                                                    j1array[0]=IKatan2(((x232)*(((((cj0)*(px)))+(((py)*(sj0)))))), ((pz)*(x232)));
                                                    sj1array[0]=IKsin(j1array[0]);
                                                    cj1array[0]=IKcos(j1array[0]);
                                                    if( j1array[0] > IKPI )
                                                        j1array[0]-=IK2PI;
                                                    else if( j1array[0] < -IKPI )
                                                        j1array[0]+=IK2PI;
                                                    j1valid[0] = true;
                                                    for(int ij1 = 0; ij1 < 1; ++ij1)
                                                    {
                                                    if( !j1valid[ij1] ) { continue; }
                                                    j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                            if( 1 )
                                            {
                                            continue;
                                        
                                        } else
                                        {
                                        }
                                        }
                                        }
                                        }
                                    
                                    } else
                                    {
                                        {
                                        IKReal j1array[1], cj1array[1], sj1array[1];
                                        bool j1valid[1]={false};
                                        IKReal x233=x2;
                                        IKReal x234=(sj0)*(sj0);
                                        IKReal x235=((cj2)*(x233)*(x234));
                                        IKReal x236=(cj0)*(cj0);
                                        IKReal x237=x4;
                                        IKReal x238=((cj2)*(x236)*(x237));
                                        IKReal x239=((2.00000000000000)*(cj0)*(cj2)*(px)*(py)*(sj0));
                                        IKReal x240=x0;
                                        IKReal x241=((cj2)*(x240));
                                        IKReal x242=((x238)+(x239)+(x235)+(x241));
                                        IKReal x243=((IKabs(x242) != 0)?((IKReal)1/(x242)):(IKReal)1.0e30);
                                        IKReal x244=((0.300000000000000)*(cj3));
                                        IKReal x245=((0.0450000000000000)*(sj3));
                                        IKReal x246=((0.550000000000000)+(x245)+(x244));
                                        IKReal x247=((cj0)*(py)*(sj2));
                                        IKReal x248=((0.300000000000000)*(sj3));
                                        IKReal x249=((0.0450000000000000)+(x247)+(x248));
                                        IKReal x250=((0.0450000000000000)*(cj3));
                                        IKReal x251=((px)*(sj0)*(sj2));
                                        IKReal x252=((x250)+(x251));
                                        IKReal x253=((x249)+(((-1.00000000000000)*(x252))));
                                        j1array[0]=IKatan2(((x243)*(((((x246)*(((((cj2)*(py)*(sj0)))+(((cj0)*(cj2)*(px)))))))+(((-1.00000000000000)*(pz)*(x253)))))), ((x243)*(((((x253)*(((((cj0)*(px)))+(((py)*(sj0)))))))+(((cj2)*(pz)*(x246)))))));
                                        sj1array[0]=IKsin(j1array[0]);
                                        cj1array[0]=IKcos(j1array[0]);
                                        if( j1array[0] > IKPI )
                                            j1array[0]-=IK2PI;
                                        else if( j1array[0] < -IKPI )
                                            j1array[0]+=IK2PI;
                                        j1valid[0] = true;
                                        for(int ij1 = 0; ij1 < 1; ++ij1)
                                        {
                                        if( !j1valid[ij1] ) { continue; }
                                        j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                                        
                                        rotationfunction0(vsolutions);
                                        }
                                        }
                                    
                                    }
                                
                                }
                            
                            } else
                            {
                            }
                            }
                            }
                            }
                        
                        } else
                        {
                            {
                            IKReal j1array[1], cj1array[1], sj1array[1];
                            bool j1valid[1]={false};
                            IKReal x254=x0;
                            IKReal x255=((sj2)*(x254));
                            IKReal x256=(cj0)*(cj0);
                            IKReal x257=x4;
                            IKReal x258=((sj2)*(x256)*(x257));
                            IKReal x259=((2.00000000000000)*(cj0)*(px)*(py)*(sj0)*(sj2));
                            IKReal x260=x2;
                            IKReal x261=(sj0)*(sj0);
                            IKReal x262=((sj2)*(x260)*(x261));
                            IKReal x263=((x258)+(x259)+(x255)+(x262));
                            IKReal x264=((IKabs(x263) != 0)?((IKReal)1/(x263)):(IKReal)1.0e30);
                            IKReal x265=((cj2)*(px)*(sj0));
                            IKReal x266=((cj0)*(cj2)*(py));
                            IKReal x267=((x265)+(((-1.00000000000000)*(x266))));
                            IKReal x268=((0.300000000000000)*(cj3));
                            IKReal x269=((0.0450000000000000)*(sj3));
                            IKReal x270=((0.550000000000000)+(x269)+(x268));
                            j1array[0]=IKatan2(((x264)*(((((x270)*(((((cj0)*(px)*(sj2)))+(((py)*(sj0)*(sj2)))))))+(((-1.00000000000000)*(pz)*(x267)))))), ((x264)*(((((pz)*(sj2)*(x270)))+(((x267)*(((((cj0)*(px)))+(((py)*(sj0)))))))))));
                            sj1array[0]=IKsin(j1array[0]);
                            cj1array[0]=IKcos(j1array[0]);
                            if( j1array[0] > IKPI )
                                j1array[0]-=IK2PI;
                            else if( j1array[0] < -IKPI )
                                j1array[0]+=IK2PI;
                            j1valid[0] = true;
                            for(int ij1 = 0; ij1 < 1; ++ij1)
                            {
                            if( !j1valid[ij1] ) { continue; }
                            j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                            
                            rotationfunction0(vsolutions);
                            }
                            }
                        
                        }
                    
                    }
                
                } else
                {
                }
                }
            
            } else
            {
                {
                IKReal j1array[1], cj1array[1], sj1array[1];
                bool j1valid[1]={false};
                IKReal x271=x2;
                IKReal x272=(sj0)*(sj0);
                IKReal x273=((x271)*(x272));
                IKReal x274=((2.00000000000000)*(cj0)*(px)*(py)*(sj0));
                IKReal x275=x0;
                IKReal x276=(cj0)*(cj0);
                IKReal x277=x4;
                IKReal x278=((x276)*(x277));
                IKReal x279=((x274)+(x275)+(x273)+(x278));
                IKReal x280=((IKabs(x279) != 0)?((IKReal)1/(x279)):(IKReal)1.0e30);
                IKReal x281=((cj0)*(px));
                IKReal x282=((py)*(sj0));
                IKReal x283=((x282)+(x281));
                IKReal x284=((0.0450000000000000)*(cj2));
                IKReal x285=((0.300000000000000)*(cj2)*(sj3));
                IKReal x286=((x285)+(x284));
                IKReal x287=((1.00000000000000)*(cj3)*(x284));
                IKReal x288=((x286)+(((-1.00000000000000)*(x287))));
                IKReal x289=((0.300000000000000)*(cj3));
                IKReal x290=((0.0450000000000000)*(sj3));
                IKReal x291=((0.550000000000000)+(x290)+(x289));
                j1array[0]=IKatan2(((x280)*(((((x283)*(x291)))+(((-1.00000000000000)*(pz)*(x288)))))), ((x280)*(((((pz)*(x291)))+(((x283)*(x288)))))));
                sj1array[0]=IKsin(j1array[0]);
                cj1array[0]=IKcos(j1array[0]);
                if( j1array[0] > IKPI )
                    j1array[0]-=IK2PI;
                else if( j1array[0] < -IKPI )
                    j1array[0]+=IK2PI;
                j1valid[0] = true;
                for(int ij1 = 0; ij1 < 1; ++ij1)
                {
                if( !j1valid[ij1] ) { continue; }
                j1 = j1array[ij1]; cj1 = cj1array[ij1]; sj1 = sj1array[ij1];
                
                rotationfunction0(vsolutions);
                }
                }
            
            }
        
        }
        }
        }
        }
        }
    
    } else
    {
    }
    }
}
return vsolutions.size()>0;
}
    inline void rotationfunction0(std::vector<IKSolution>& vsolutions) {
        IKReal x29=((cj1)*(sj3));
        IKReal x30=((cj2)*(cj3)*(sj1));
        IKReal x31=((x30)+(x29));
        IKReal x32=((-1.00000000000000)*(x31));
        IKReal x33=((cj1)*(cj2)*(cj3)*(sj0));
        IKReal x34=((cj0)*(cj3)*(sj2));
        IKReal x35=((sj0)*(sj1)*(sj3));
        IKReal x36=((x35)+(x34));
        IKReal x37=((((-1.00000000000000)*(x36)))+(x33));
        IKReal x38=((cj0)*(cj1)*(cj2)*(cj3));
        IKReal x39=((cj3)*(sj0)*(sj2));
        IKReal x40=((x39)+(x38));
        IKReal x41=((cj0)*(sj1)*(sj3));
        IKReal x42=((((-1.00000000000000)*(x41)))+(x40));
        IKReal x43=((cj0)*(sj2)*(sj3));
        IKReal x44=((cj2)*(sj0)*(x29));
        IKReal x45=((cj3)*(sj0)*(sj1));
        IKReal x46=((x44)+(x45));
        IKReal x47=((((-1.00000000000000)*(x46)))+(x43));
        IKReal x48=((cj0)*(cj3)*(sj1));
        IKReal x49=((cj0)*(cj2)*(x29));
        IKReal x50=((sj0)*(sj2)*(sj3));
        IKReal x51=((x48)+(x49)+(x50));
        IKReal x52=((-1.00000000000000)*(x51));
        IKReal x53=((cj2)*(sj1)*(sj3));
        IKReal x54=((cj1)*(cj3));
        IKReal x55=((x53)+(((-1.00000000000000)*(x54))));
        IKReal x56=((cj0)*(cj1)*(sj2));
        IKReal x57=((cj2)*(sj0));
        IKReal x58=((x56)+(((-1.00000000000000)*(x57))));
        IKReal x59=((cj0)*(cj2));
        IKReal x60=((cj1)*(sj0)*(sj2));
        IKReal x61=((x59)+(x60));
        new_r00=((((r20)*(x32)))+(((r00)*(x42)))+(((r10)*(x37))));
        new_r01=((((r01)*(x42)))+(((r11)*(x37)))+(((r21)*(x32))));
        new_r02=((((r22)*(x32)))+(((r02)*(x42)))+(((r12)*(x37))));
        new_r10=((((r10)*(x47)))+(((r00)*(x52)))+(((r20)*(x55))));
        new_r11=((((r01)*(x52)))+(((r21)*(x55)))+(((r11)*(x47))));
        new_r12=((((r22)*(x55)))+(((r02)*(x52)))+(((r12)*(x47))));
        new_r20=((((r10)*(x61)))+(((-1.00000000000000)*(r20)*(sj1)*(sj2)))+(((r00)*(x58))));
        new_r21=((((r11)*(x61)))+(((-1.00000000000000)*(r21)*(sj1)*(sj2)))+(((r01)*(x58))));
        new_r22=((((r02)*(x58)))+(((-1.00000000000000)*(r22)*(sj1)*(sj2)))+(((r12)*(x61))));
        {
        IKReal j5eval;
        j5eval=new_r11;
        if( j5eval >= -1.000010 && j5eval <= -0.999990 )
        {
            {
            j5 = 3.141593; sj5 = 0.000000; cj5 = -1.000000;
            IKReal j4mul = 1;
            j4=0;
            IKReal j6mul=-1.00000000000000;
            j6=((((-1.00000000000000)*(IKatan2(((-1.00000000000000)*(new_r02)), new_r00))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[4].fmul = j4mul;
            solution.basesol[4].freeind = 0;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.basesol[6].fmul = j6mul;
            solution.basesol[6].freeind = 0;
            solution.vfree.resize(1);
            solution.vfree[0] = 4;
            }
        
        } else
        if( j5eval >= 0.999990 && j5eval <= 1.000010 )
        {
            {
            j5 = 0.000000; sj5 = 0.000000; cj5 = 1.000000;
            IKReal j4mul = 1;
            j4=0;
            IKReal j6mul=1.00000000000000;
            j6=((-1.00000000000000)*(IKatan2(new_r20, new_r22)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(7);
            solution.basesol[0].foffset = j0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[4].fmul = j4mul;
            solution.basesol[4].freeind = 0;
            solution.basesol[5].foffset = j5;
            solution.basesol[6].foffset = j6;
            solution.basesol[6].fmul = j6mul;
            solution.basesol[6].freeind = 0;
            solution.vfree.resize(1);
            solution.vfree[0] = 4;
            }
        
        } else
        {
            {
            IKReal j4array[2], cj4array[2], sj4array[2];
            bool j4valid[2]={false};
            j4array[0]=IKatan2(((-1.00000000000000)*(new_r21)), ((-1.00000000000000)*(new_r01)));
            sj4array[0]=IKsin(j4array[0]);
            cj4array[0]=IKcos(j4array[0]);
            j4array[1] = j4array[0] > 0 ? j4array[0]-IKPI : j4array[0]+IKPI;
            sj4array[1] = -sj4array[0];
            cj4array[1] = -cj4array[0];
            if( j4array[0] > IKPI )
                j4array[0]-=IK2PI;
            else if( j4array[0] < -IKPI )
                j4array[0]+=IK2PI;
            j4valid[0] = true;
            if( j4array[1] > IKPI )
                j4array[1]-=IK2PI;
            else if( j4array[1] < -IKPI )
                j4array[1]+=IK2PI;
            j4valid[1] = true;
            if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
                j4valid[1]=false;
            for(int ij4 = 0; ij4 < 2; ++ij4)
            {
            if( !j4valid[ij4] ) { continue; }
            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
            
            {
            IKReal evalcond[1];
                evalcond[0]=IKcos(j4);
                if( IKabs(evalcond[0]) < 0.000010  )
                {
                {
                IKReal j5array[1], cj5array[1], sj5array[1];
                bool j5valid[1]={false};
                j5array[0]=IKatan2(((-1.00000000000000)*(new_r21)*(((IKabs(IKsin(j4)) != 0)?((IKReal)1/(IKsin(j4))):(IKReal)1.0e30))), new_r11);
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                for(int ij5 = 0; ij5 < 1; ++ij5)
                {
                if( !j5valid[ij5] ) { continue; }
                j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
                
                {
                IKReal j6array[1], cj6array[1], sj6array[1];
                bool j6valid[1]={false};
                IKReal x62=IKsin(j4);
                IKReal x63=((IKabs(x62) != 0)?((IKReal)1/(x62)):(IKReal)1.0e30);
                j6array[0]=IKatan2(((new_r00)*(x63)), ((-1.00000000000000)*(new_r02)*(x63)));
                sj6array[0]=IKsin(j6array[0]);
                cj6array[0]=IKcos(j6array[0]);
                if( j6array[0] > IKPI )
                    j6array[0]-=IK2PI;
                else if( j6array[0] < -IKPI )
                    j6array[0]+=IK2PI;
                j6valid[0] = true;
                for(int ij6 = 0; ij6 < 1; ++ij6)
                {
                if( !j6valid[ij6] ) { continue; }
                j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
                
                vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                solution.basesol.resize(7);
                solution.basesol[0].foffset = j0;
                solution.basesol[1].foffset = j1;
                solution.basesol[2].foffset = j2;
                solution.basesol[3].foffset = j3;
                solution.basesol[4].foffset = j4;
                solution.basesol[5].foffset = j5;
                solution.basesol[6].foffset = j6;
                solution.vfree.resize(0);
                }
                }
                }
                }
            
            } else
            {
                if( 1 )
                {
                {
                IKReal j5array[1], cj5array[1], sj5array[1];
                bool j5valid[1]={false};
                j5array[0]=IKatan2(((-1.00000000000000)*(new_r01)*(((IKabs(cj4) != 0)?((IKReal)1/(cj4)):(IKReal)1.0e30))), new_r11);
                sj5array[0]=IKsin(j5array[0]);
                cj5array[0]=IKcos(j5array[0]);
                if( j5array[0] > IKPI )
                    j5array[0]-=IK2PI;
                else if( j5array[0] < -IKPI )
                    j5array[0]+=IK2PI;
                j5valid[0] = true;
                for(int ij5 = 0; ij5 < 1; ++ij5)
                {
                if( !j5valid[ij5] ) { continue; }
                j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
                
                {
                IKReal evalcond[1];
                    evalcond[0]=IKsin(j5);
                    if( IKabs(evalcond[0]) < 0.000010  )
                    {
                    {
                    IKReal j6array[1], cj6array[1], sj6array[1];
                    bool j6valid[1]={false};
                    IKReal x64=IKcos(j5);
                    IKReal x65=((IKabs(x64) != 0)?((IKReal)1/(x64)):(IKReal)1.0e30);
                    IKReal x66=IKcos(j4);
                    IKReal x67=IKsin(j4);
                    j6array[0]=IKatan2(((((new_r02)*(x65)*(x66)))+(((new_r22)*(x65)*(x67)))), ((((new_r22)*(x66)))+(((-1.00000000000000)*(new_r02)*(x67)))));
                    sj6array[0]=IKsin(j6array[0]);
                    cj6array[0]=IKcos(j6array[0]);
                    if( j6array[0] > IKPI )
                        j6array[0]-=IK2PI;
                    else if( j6array[0] < -IKPI )
                        j6array[0]+=IK2PI;
                    j6valid[0] = true;
                    for(int ij6 = 0; ij6 < 1; ++ij6)
                    {
                    if( !j6valid[ij6] ) { continue; }
                    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                
                } else
                {
                    if( 1 )
                    {
                    {
                    IKReal j6array[1], cj6array[1], sj6array[1];
                    bool j6valid[1]={false};
                    IKReal x68=IKsin(j5);
                    IKReal x69=((IKabs(x68) != 0)?((IKReal)1/(x68)):(IKReal)1.0e30);
                    j6array[0]=IKatan2(((new_r12)*(x69)), ((new_r10)*(x69)));
                    sj6array[0]=IKsin(j6array[0]);
                    cj6array[0]=IKcos(j6array[0]);
                    if( j6array[0] > IKPI )
                        j6array[0]-=IK2PI;
                    else if( j6array[0] < -IKPI )
                        j6array[0]+=IK2PI;
                    j6valid[0] = true;
                    for(int ij6 = 0; ij6 < 1; ++ij6)
                    {
                    if( !j6valid[ij6] ) { continue; }
                    j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(7);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.vfree.resize(0);
                    }
                    }
                
                } else
                {
                }
                }
                }
                }
                }
            
            } else
            {
            }
            }
            }
            }
            }
        
        }
        }
    }};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ik(const IKReal* eetrans, const IKReal* eerot, const IKReal* pfree, std::vector<IKSolution>& vsolutions) {
IKSolver solver;
return solver.ik(eetrans,eerot,pfree,vsolutions);
}

IKFAST_API const char* getKinematicsHash() { return "b4ff7e5e04780df16a6bda565e714741"; }

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv)
{
    if( argc != 12+getNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",getNumFreeParameters());
        return 1;
    }

    std::vector<IKSolution> vsolutions;
    std::vector<IKReal> vfree(getNumFreeParameters());
    IKReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ik(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, vsolutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)vsolutions.size());
    std::vector<IKReal> sol(getNumJoints());
    for(size_t i = 0; i < vsolutions.size(); ++i) {
        printf("sol%d (free=%d): ", (int)i, (int)vsolutions[i].GetFree().size());
        std::vector<IKReal> vsolfree(vsolutions[i].GetFree().size());
        vsolutions[i].GetSolution(&sol[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( size_t j = 0; j < sol.size(); ++j)
            printf("%f, ", (float)sol[j]);
        printf("\n");
    }
    return 0;
}

#endif
