/// autogenerated analytical inverse kinematics code from ikfast program
/// \author Rosen Diankov
///
/// Licensed under the Apache License, Version 2.0 (the "License");
/// you may not use this file except in compliance with the License.
/// You may obtain a copy of the License at
///     http://www.apache.org/licenses/LICENSE-2.0
/// 
/// Unless required by applicable law or agreed to in writing, software
/// distributed under the License is distributed on an "AS IS" BASIS,
/// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
/// See the License for the specific language governing permissions and
/// limitations under the License.
///
/// generated 2010-11-03 15:57:00.548100
/// To compile with gcc:
///     gcc -lstdc++ ik.cpp
/// To compile without any main function as a shared object:
///     gcc -fPIC -lstdc++ -DIKFAST_NO_MAIN -shared -Wl,-soname,ik.so -o ik.so ik.cpp
#include <cmath>
#include <cassert>
#include <vector>
#include <limits>
#include <algorithm>
#include <complex>

#define IK2PI  6.28318530717959
#define IKPI  3.14159265358979
#define IKPI_2  1.57079632679490

#ifdef _MSC_VER
#ifndef isnan
#define isnan _isnan
#endif
#endif // _MSC_VER

// defined when creating a shared object/dll
#ifdef IKFAST_CLIBRARY
#ifdef _MSC_VER
#define IKFAST_API extern "C" __declspec(dllexport)
#else
#define IKFAST_API extern "C"
#endif
#else
#define IKFAST_API
#endif

typedef double IKReal;
class IKSolution
{
public:
    /// Gets a solution given its free parameters
    /// \param pfree The free parameters required, range is in [-pi,pi]
    void GetSolution(IKReal* psolution, const IKReal* pfree) const {
        for(size_t i = 0; i < basesol.size(); ++i) {
            if( basesol[i].freeind < 0 )
                psolution[i] = basesol[i].foffset;
            else {
                assert(pfree != NULL);
                psolution[i] = pfree[basesol[i].freeind]*basesol[i].fmul + basesol[i].foffset;
                if( psolution[i] > IKPI )
                    psolution[i] -= IK2PI;
                else if( psolution[i] < -IKPI )
                    psolution[i] += IK2PI;
            }
        }
    }

    /// Gets the free parameters the solution requires to be set before a full solution can be returned
    /// \return vector of indices indicating the free parameters
    const std::vector<int>& GetFree() const { return vfree; }

    struct VARIABLE
    {
        VARIABLE() : freeind(-1), fmul(0), foffset(0) {}
        VARIABLE(int freeind, IKReal fmul, IKReal foffset) : freeind(freeind), fmul(fmul), foffset(foffset) {}
        int freeind;
        IKReal fmul, foffset; ///< joint value is fmul*sol[freeind]+foffset
    };

    std::vector<VARIABLE> basesol;       ///< solution and their offsets if joints are mimiced
    std::vector<int> vfree;
};

inline float IKabs(float f) { return fabsf(f); }
inline double IKabs(double f) { return fabs(f); }

inline float IKlog(float f) { return logf(f); }
inline double IKlog(double f) { return log(f); }

inline float IKasin(float f)
{
assert( f > -1.001f && f < 1.001f ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asinf(f);
}
inline double IKasin(double f)
{
assert( f > -1.001 && f < 1.001 ); // any more error implies something is wrong with the solver
if( f <= -1 ) return -IKPI_2;
else if( f >= 1 ) return IKPI_2;
return asin(f);
}

// return positive value in [0,y)
inline float IKfmod(float x, float y)
{
    while(x < 0) {
        x += y;
    }
    return fmodf(x,y);
}

// return positive value in [0,y)
inline float IKfmod(double x, double y)
{
    while(x < 0) {
        x += y;
    }
    return fmod(x,y);
}

inline float IKacos(float f)
{
assert( f > -1.001f && f < 1.001f ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0.0f;
return acosf(f);
}
inline double IKacos(double f)
{
assert( f > -1.001 && f < 1.001 ); // any more error implies something is wrong with the solver
if( f <= -1 ) return IKPI;
else if( f >= 1 ) return 0.0;
return acos(f);
}
inline float IKsin(float f) { return sinf(f); }
inline double IKsin(double f) { return sin(f); }
inline float IKcos(float f) { return cosf(f); }
inline double IKcos(double f) { return cos(f); }
inline float IKsqrt(float f) { if( f <= 0.0f ) return 0.0f; return sqrtf(f); }
inline double IKsqrt(double f) { if( f <= 0.0 ) return 0.0; return sqrt(f); }
inline float IKatan2(float fy, float fx) {
    if( isnan(fy) ) {
        assert(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) )
        return 0;
    return atan2f(fy,fx);
}
inline double IKatan2(double fy, double fx) {
    if( isnan(fy) ) {
        assert(!isnan(fx)); // if both are nan, probably wrong value will be returned
        return IKPI_2;
    }
    else if( isnan(fx) )
        return 0;
    return atan2(fy,fx);
}

IKFAST_API int getNumFreeParameters() { return 2; }
IKFAST_API int* getFreeParameters() { static int freeparams[] = {0, 3}; return freeparams; }
IKFAST_API int getNumJoints() { return 8; }

IKFAST_API int getIKRealSize() { return sizeof(IKReal); }

IKFAST_API int getIKType() { return 1; }

/// solves the forward kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API void fk(const IKReal* j, IKReal* eetrans, IKReal* eerot) {
    IKReal x0,x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13;
    x0=IKcos(j[1]);
    x1=IKsin(j[2]);
    x2=IKsin(j[3]);
    x3=IKsin(j[5]);
    x4=IKsin(j[6]);
    x5=IKcos(j[6]);
    x6=IKcos(j[4]);
    x7=IKcos(j[3]);
    x8=IKcos(j[5]);
    x9=IKsin(j[1]);
    x10=IKsin(j[4]);
    x11=IKcos(j[2]);
    x12=IKcos(j[7]);
    x13=IKsin(j[7]);
    eerot[0]=((((-0.999135601564979)*(x0)*(x1)*(x10)*(x5)*(x7)))+(((0.0415698169996759)*(x0)*(x1)*(x2)*(x4)*(x6)*(x8)))+(((x0)*(x11)*(x5)*(x6)))+(((0.0415698169996759)*(x2)*(x3)*(x4)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x11)*(x4)*(x8)))+(((-0.999135601564979)*(x2)*(x4)*(x6)*(x8)*(x9)))+(((-0.0415698169996759)*(x10)*(x5)*(x7)*(x9)))+(((0.0415698169996759)*(x0)*(x1)*(x10)*(x2)*(x5)))+(((0.999135601564979)*(x0)*(x1)*(x2)*(x3)*(x4)))+(((-0.0415698169996759)*(x4)*(x6)*(x7)*(x8)*(x9)))+(((-0.999135601564979)*(x3)*(x4)*(x7)*(x9)))+(((-0.999135601564979)*(x0)*(x1)*(x4)*(x6)*(x7)*(x8)))+(((-0.999135601564979)*(x10)*(x2)*(x5)*(x9)))+(((0.0415698169996759)*(x0)*(x1)*(x3)*(x4)*(x7))));
    eerot[1]=((((x0)*(x10)*(x11)*(x13)*(x5)*(x8)))+(((0.999135601564979)*(x0)*(x1)*(x12)*(x3)*(x6)*(x7)))+(((-0.999135601564979)*(x0)*(x1)*(x10)*(x13)*(x4)*(x7)))+(((0.0415698169996759)*(x12)*(x2)*(x8)*(x9)))+(((-0.999135601564979)*(x10)*(x13)*(x2)*(x4)*(x9)))+(((0.999135601564979)*(x0)*(x1)*(x13)*(x5)*(x6)*(x7)*(x8)))+(((0.0415698169996759)*(x0)*(x1)*(x12)*(x7)*(x8)))+(((-0.0415698169996759)*(x0)*(x1)*(x13)*(x3)*(x5)*(x7)))+(((-0.999135601564979)*(x12)*(x7)*(x8)*(x9)))+(((0.999135601564979)*(x13)*(x2)*(x5)*(x6)*(x8)*(x9)))+(((-0.0415698169996759)*(x13)*(x2)*(x3)*(x5)*(x9)))+(((0.0415698169996759)*(x13)*(x5)*(x6)*(x7)*(x8)*(x9)))+(((x0)*(x10)*(x11)*(x12)*(x3)))+(((-0.0415698169996759)*(x0)*(x1)*(x12)*(x2)*(x3)*(x6)))+(((0.999135601564979)*(x12)*(x2)*(x3)*(x6)*(x9)))+(((-0.999135601564979)*(x0)*(x1)*(x13)*(x2)*(x3)*(x5)))+(((0.0415698169996759)*(x0)*(x1)*(x10)*(x13)*(x2)*(x4)))+(((-0.0415698169996759)*(x0)*(x1)*(x13)*(x2)*(x5)*(x6)*(x8)))+(((0.999135601564979)*(x13)*(x3)*(x5)*(x7)*(x9)))+(((0.999135601564979)*(x0)*(x1)*(x12)*(x2)*(x8)))+(((-0.0415698169996759)*(x10)*(x13)*(x4)*(x7)*(x9)))+(((0.0415698169996759)*(x12)*(x3)*(x6)*(x7)*(x9)))+(((x0)*(x11)*(x13)*(x4)*(x6))));
    eerot[2]=((((-0.999135601564979)*(x0)*(x1)*(x10)*(x12)*(x4)*(x7)))+(((-0.999135601564979)*(x0)*(x1)*(x12)*(x2)*(x3)*(x5)))+(((-0.999135601564979)*(x13)*(x2)*(x3)*(x6)*(x9)))+(((-0.999135601564979)*(x10)*(x12)*(x2)*(x4)*(x9)))+(((-0.999135601564979)*(x0)*(x1)*(x13)*(x2)*(x8)))+(((0.0415698169996759)*(x0)*(x1)*(x13)*(x2)*(x3)*(x6)))+(((-0.0415698169996759)*(x10)*(x12)*(x4)*(x7)*(x9)))+(((x0)*(x10)*(x11)*(x12)*(x5)*(x8)))+(((0.999135601564979)*(x12)*(x3)*(x5)*(x7)*(x9)))+(((-0.0415698169996759)*(x13)*(x3)*(x6)*(x7)*(x9)))+(((-0.0415698169996759)*(x0)*(x1)*(x12)*(x3)*(x5)*(x7)))+(((x0)*(x11)*(x12)*(x4)*(x6)))+(((-0.0415698169996759)*(x0)*(x1)*(x12)*(x2)*(x5)*(x6)*(x8)))+(((0.999135601564979)*(x0)*(x1)*(x12)*(x5)*(x6)*(x7)*(x8)))+(((0.0415698169996759)*(x12)*(x5)*(x6)*(x7)*(x8)*(x9)))+(((0.999135601564979)*(x12)*(x2)*(x5)*(x6)*(x8)*(x9)))+(((0.0415698169996759)*(x0)*(x1)*(x10)*(x12)*(x2)*(x4)))+(((-0.0415698169996759)*(x12)*(x2)*(x3)*(x5)*(x9)))+(((-0.999135601564979)*(x0)*(x1)*(x13)*(x3)*(x6)*(x7)))+(((-0.0415698169996759)*(x0)*(x1)*(x13)*(x7)*(x8)))+(((0.999135601564979)*(x13)*(x7)*(x8)*(x9)))+(((-1.00000000000000)*(x0)*(x10)*(x11)*(x13)*(x3)))+(((-0.0415698169996759)*(x13)*(x2)*(x8)*(x9))));
    eetrans[0]=((-0.0500000000000000)+(((-0.320722528102358)*(x10)*(x2)*(x9)))+(((0.400000000000000)*(x0)*(x11)))+(((0.00748256705994167)*(x2)*(x3)*(x4)*(x9)))+(((0.00748256705994167)*(x0)*(x1)*(x3)*(x4)*(x7)))+(((-0.180000000000000)*(x0)*(x10)*(x11)*(x4)*(x8)))+(((0.0133439112568960)*(x0)*(x1)*(x10)*(x2)))+(((0.179844408281696)*(x0)*(x1)*(x2)*(x3)*(x4)))+(((-0.320722528102358)*(x0)*(x1)*(x10)*(x7)))+(((-0.179844408281696)*(x10)*(x2)*(x5)*(x9)))+(((-0.179844408281696)*(x2)*(x4)*(x6)*(x8)*(x9)))+(((0.100000000000000)*(x0)))+(((-0.0133439112568960)*(x10)*(x7)*(x9)))+(((0.00748256705994167)*(x0)*(x1)*(x10)*(x2)*(x5)))+(((-0.00748256705994167)*(x10)*(x5)*(x7)*(x9)))+(((0.321000000000000)*(x0)*(x11)*(x6)))+(((0.180000000000000)*(x0)*(x11)*(x5)*(x6)))+(((-0.179844408281696)*(x0)*(x1)*(x10)*(x5)*(x7)))+(((-0.00748256705994167)*(x4)*(x6)*(x7)*(x8)*(x9)))+(((0.00748256705994167)*(x0)*(x1)*(x2)*(x4)*(x6)*(x8)))+(((-0.179844408281696)*(x3)*(x4)*(x7)*(x9)))+(((-0.179844408281696)*(x0)*(x1)*(x4)*(x6)*(x7)*(x8))));
    eerot[3]=((((0.0415698169996759)*(x1)*(x10)*(x2)*(x5)*(x9)))+(((-0.0415698169996759)*(x0)*(x2)*(x3)*(x4)))+(((0.999135601564979)*(x0)*(x3)*(x4)*(x7)))+(((x11)*(x5)*(x6)*(x9)))+(((0.999135601564979)*(x0)*(x2)*(x4)*(x6)*(x8)))+(((0.0415698169996759)*(x1)*(x2)*(x4)*(x6)*(x8)*(x9)))+(((0.999135601564979)*(x0)*(x10)*(x2)*(x5)))+(((-1.00000000000000)*(x10)*(x11)*(x4)*(x8)*(x9)))+(((0.0415698169996759)*(x0)*(x10)*(x5)*(x7)))+(((-0.999135601564979)*(x1)*(x10)*(x5)*(x7)*(x9)))+(((-0.999135601564979)*(x1)*(x4)*(x6)*(x7)*(x8)*(x9)))+(((0.999135601564979)*(x1)*(x2)*(x3)*(x4)*(x9)))+(((0.0415698169996759)*(x1)*(x3)*(x4)*(x7)*(x9)))+(((0.0415698169996759)*(x0)*(x4)*(x6)*(x7)*(x8))));
    eerot[4]=((((-0.0415698169996759)*(x1)*(x13)*(x3)*(x5)*(x7)*(x9)))+(((x10)*(x11)*(x12)*(x3)*(x9)))+(((0.0415698169996759)*(x0)*(x10)*(x13)*(x4)*(x7)))+(((0.999135601564979)*(x0)*(x10)*(x13)*(x2)*(x4)))+(((-0.0415698169996759)*(x1)*(x13)*(x2)*(x5)*(x6)*(x8)*(x9)))+(((0.0415698169996759)*(x0)*(x13)*(x2)*(x3)*(x5)))+(((-0.999135601564979)*(x1)*(x10)*(x13)*(x4)*(x7)*(x9)))+(((x10)*(x11)*(x13)*(x5)*(x8)*(x9)))+(((0.999135601564979)*(x1)*(x12)*(x3)*(x6)*(x7)*(x9)))+(((-0.0415698169996759)*(x0)*(x13)*(x5)*(x6)*(x7)*(x8)))+(((0.999135601564979)*(x0)*(x12)*(x7)*(x8)))+(((-0.0415698169996759)*(x0)*(x12)*(x2)*(x8)))+(((-0.999135601564979)*(x0)*(x12)*(x2)*(x3)*(x6)))+(((-0.0415698169996759)*(x1)*(x12)*(x2)*(x3)*(x6)*(x9)))+(((-0.0415698169996759)*(x0)*(x12)*(x3)*(x6)*(x7)))+(((x11)*(x13)*(x4)*(x6)*(x9)))+(((0.0415698169996759)*(x1)*(x10)*(x13)*(x2)*(x4)*(x9)))+(((0.999135601564979)*(x1)*(x13)*(x5)*(x6)*(x7)*(x8)*(x9)))+(((0.999135601564979)*(x1)*(x12)*(x2)*(x8)*(x9)))+(((0.0415698169996759)*(x1)*(x12)*(x7)*(x8)*(x9)))+(((-0.999135601564979)*(x0)*(x13)*(x3)*(x5)*(x7)))+(((-0.999135601564979)*(x1)*(x13)*(x2)*(x3)*(x5)*(x9)))+(((-0.999135601564979)*(x0)*(x13)*(x2)*(x5)*(x6)*(x8))));
    eerot[5]=((((-1.00000000000000)*(x10)*(x11)*(x13)*(x3)*(x9)))+(((-0.999135601564979)*(x0)*(x12)*(x2)*(x5)*(x6)*(x8)))+(((0.999135601564979)*(x0)*(x10)*(x12)*(x2)*(x4)))+(((0.999135601564979)*(x0)*(x13)*(x2)*(x3)*(x6)))+(((x10)*(x11)*(x12)*(x5)*(x8)*(x9)))+(((0.0415698169996759)*(x0)*(x13)*(x3)*(x6)*(x7)))+(((0.0415698169996759)*(x0)*(x10)*(x12)*(x4)*(x7)))+(((-0.999135601564979)*(x0)*(x12)*(x3)*(x5)*(x7)))+(((-0.999135601564979)*(x1)*(x10)*(x12)*(x4)*(x7)*(x9)))+(((0.0415698169996759)*(x1)*(x13)*(x2)*(x3)*(x6)*(x9)))+(((-0.999135601564979)*(x1)*(x12)*(x2)*(x3)*(x5)*(x9)))+(((-0.0415698169996759)*(x1)*(x12)*(x2)*(x5)*(x6)*(x8)*(x9)))+(((-0.999135601564979)*(x1)*(x13)*(x3)*(x6)*(x7)*(x9)))+(((-0.0415698169996759)*(x1)*(x13)*(x7)*(x8)*(x9)))+(((-0.0415698169996759)*(x1)*(x12)*(x3)*(x5)*(x7)*(x9)))+(((0.0415698169996759)*(x0)*(x12)*(x2)*(x3)*(x5)))+(((-0.0415698169996759)*(x0)*(x12)*(x5)*(x6)*(x7)*(x8)))+(((-0.999135601564979)*(x1)*(x13)*(x2)*(x8)*(x9)))+(((x11)*(x12)*(x4)*(x6)*(x9)))+(((-0.999135601564979)*(x0)*(x13)*(x7)*(x8)))+(((0.0415698169996759)*(x0)*(x13)*(x2)*(x8)))+(((0.0415698169996759)*(x1)*(x10)*(x12)*(x2)*(x4)*(x9)))+(((0.999135601564979)*(x1)*(x12)*(x5)*(x6)*(x7)*(x8)*(x9))));
    eetrans[1]=((0.188000000000000)+(((0.179844408281696)*(x0)*(x10)*(x2)*(x5)))+(((0.00748256705994167)*(x1)*(x2)*(x4)*(x6)*(x8)*(x9)))+(((0.180000000000000)*(x11)*(x5)*(x6)*(x9)))+(((0.321000000000000)*(x11)*(x6)*(x9)))+(((-0.00748256705994167)*(x0)*(x2)*(x3)*(x4)))+(((0.0133439112568960)*(x1)*(x10)*(x2)*(x9)))+(((0.179844408281696)*(x0)*(x2)*(x4)*(x6)*(x8)))+(((0.100000000000000)*(x9)))+(((0.00748256705994167)*(x0)*(x4)*(x6)*(x7)*(x8)))+(((-0.180000000000000)*(x10)*(x11)*(x4)*(x8)*(x9)))+(((0.179844408281696)*(x0)*(x3)*(x4)*(x7)))+(((0.00748256705994167)*(x1)*(x3)*(x4)*(x7)*(x9)))+(((-0.179844408281696)*(x1)*(x4)*(x6)*(x7)*(x8)*(x9)))+(((0.0133439112568960)*(x0)*(x10)*(x7)))+(((0.00748256705994167)*(x1)*(x10)*(x2)*(x5)*(x9)))+(((0.179844408281696)*(x1)*(x2)*(x3)*(x4)*(x9)))+(((-0.179844408281696)*(x1)*(x10)*(x5)*(x7)*(x9)))+(((-0.320722528102358)*(x1)*(x10)*(x7)*(x9)))+(((0.00748256705994167)*(x0)*(x10)*(x5)*(x7)))+(((0.400000000000000)*(x11)*(x9)))+(((0.320722528102358)*(x0)*(x10)*(x2))));
    eerot[6]=((((0.999135601564979)*(x11)*(x2)*(x3)*(x4)))+(((0.0415698169996759)*(x10)*(x11)*(x2)*(x5)))+(((0.0415698169996759)*(x11)*(x2)*(x4)*(x6)*(x8)))+(((x1)*(x10)*(x4)*(x8)))+(((-1.00000000000000)*(x1)*(x5)*(x6)))+(((-0.999135601564979)*(x11)*(x4)*(x6)*(x7)*(x8)))+(((0.0415698169996759)*(x11)*(x3)*(x4)*(x7)))+(((-0.999135601564979)*(x10)*(x11)*(x5)*(x7))));
    eerot[7]=((((-0.999135601564979)*(x10)*(x11)*(x13)*(x4)*(x7)))+(((0.999135601564979)*(x11)*(x13)*(x5)*(x6)*(x7)*(x8)))+(((-0.0415698169996759)*(x11)*(x13)*(x2)*(x5)*(x6)*(x8)))+(((0.999135601564979)*(x11)*(x12)*(x3)*(x6)*(x7)))+(((0.999135601564979)*(x11)*(x12)*(x2)*(x8)))+(((-1.00000000000000)*(x1)*(x10)*(x13)*(x5)*(x8)))+(((0.0415698169996759)*(x11)*(x12)*(x7)*(x8)))+(((-0.0415698169996759)*(x11)*(x12)*(x2)*(x3)*(x6)))+(((-0.0415698169996759)*(x11)*(x13)*(x3)*(x5)*(x7)))+(((-1.00000000000000)*(x1)*(x13)*(x4)*(x6)))+(((-1.00000000000000)*(x1)*(x10)*(x12)*(x3)))+(((0.0415698169996759)*(x10)*(x11)*(x13)*(x2)*(x4)))+(((-0.999135601564979)*(x11)*(x13)*(x2)*(x3)*(x5))));
    eerot[8]=((((-0.0415698169996759)*(x11)*(x12)*(x3)*(x5)*(x7)))+(((-0.999135601564979)*(x10)*(x11)*(x12)*(x4)*(x7)))+(((-0.999135601564979)*(x11)*(x12)*(x2)*(x3)*(x5)))+(((0.999135601564979)*(x11)*(x12)*(x5)*(x6)*(x7)*(x8)))+(((0.0415698169996759)*(x10)*(x11)*(x12)*(x2)*(x4)))+(((-1.00000000000000)*(x1)*(x12)*(x4)*(x6)))+(((-0.0415698169996759)*(x11)*(x12)*(x2)*(x5)*(x6)*(x8)))+(((0.0415698169996759)*(x11)*(x13)*(x2)*(x3)*(x6)))+(((x1)*(x10)*(x13)*(x3)))+(((-0.999135601564979)*(x11)*(x13)*(x3)*(x6)*(x7)))+(((-0.999135601564979)*(x11)*(x13)*(x2)*(x8)))+(((-1.00000000000000)*(x1)*(x10)*(x12)*(x5)*(x8)))+(((-0.0415698169996759)*(x11)*(x13)*(x7)*(x8))));
    eetrans[2]=((0.739680000000000)+(((-0.180000000000000)*(x1)*(x5)*(x6)))+(((0.179844408281696)*(x11)*(x2)*(x3)*(x4)))+(((-0.320722528102358)*(x10)*(x11)*(x7)))+(((0.00748256705994167)*(x11)*(x3)*(x4)*(x7)))+(((0.180000000000000)*(x1)*(x10)*(x4)*(x8)))+(((-0.179844408281696)*(x10)*(x11)*(x5)*(x7)))+(((0.00748256705994167)*(x11)*(x2)*(x4)*(x6)*(x8)))+(((-0.321000000000000)*(x1)*(x6)))+(((0.0133439112568960)*(x10)*(x11)*(x2)))+(j[0])+(((-0.400000000000000)*(x1)))+(((-0.179844408281696)*(x11)*(x4)*(x6)*(x7)*(x8)))+(((0.00748256705994167)*(x10)*(x11)*(x2)*(x5))));
}

class IKSolver {
public:
IKReal j1, cj1, sj1,
j2, cj2, sj2,
j4, cj4, sj4,
j5, cj5, sj5,
j6, cj6, sj6,
j7, cj7, sj7,
j0, cj0, sj0,
j3, cj3, sj3,
new_r00, r00,
new_r01, r01,
new_r02, r02,
new_r10, r10,
new_r11, r11,
new_r12, r12,
new_r20, r20,
new_r21, r21,
new_r22, r22,
new_px, new_py, new_pz, px, py, pz;

bool ik(const IKReal* eetrans, const IKReal* eerot, const IKReal* pfree, std::vector<IKSolution>& vsolutions) {
for(int dummyiter = 0; dummyiter < 1; ++dummyiter) {
    vsolutions.resize(0); vsolutions.reserve(8);
    j0=pfree[0]; cj0=cos(pfree[0]); sj0=sin(pfree[0]);
    j3=pfree[1]; cj3=cos(pfree[1]); sj3=sin(pfree[1]);
    r00 = eerot[0*3+0];
    r01 = eerot[0*3+1];
    r02 = eerot[0*3+2];
    r10 = eerot[1*3+0];
    r11 = eerot[1*3+1];
    r12 = eerot[1*3+2];
    r20 = eerot[2*3+0];
    r21 = eerot[2*3+1];
    r22 = eerot[2*3+2];
    px = eetrans[0]; py = eetrans[1]; pz = eetrans[2];
    
    new_r00=r00;
    new_r01=r01;
    new_r02=r02;
    new_px=((0.0500000000000000)+(px)+(((-0.180000000000000)*(r00))));
    new_r10=r10;
    new_r11=r11;
    new_r12=r12;
    new_py=((-0.188000000000000)+(py)+(((-0.180000000000000)*(r10))));
    new_r20=r20;
    new_r21=r21;
    new_r22=r22;
    new_pz=((-0.739680000000000)+(pz)+(((-0.180000000000000)*(r20))));
    r00 = new_r00; r01 = new_r01; r02 = new_r02; r10 = new_r10; r11 = new_r11; r12 = new_r12; r20 = new_r20; r21 = new_r21; r22 = new_r22; px = new_px; py = new_py; pz = new_pz;
    {
        if( 1 )
        {
        IKReal coniccoeffs[6], j1array[4];
        int numsolutions;
        IKReal x0=(py)*(py);
        IKReal x1=(px)*(px);
        IKReal x2=(sj3)*(sj3);
        IKReal x3=(cj3)*(cj3);
        IKReal x4=(j0)*(j0);
        IKReal x5=(pz)*(pz);
        IKReal x6=(px)*(px)*(px);
        IKReal x7=(py)*(py)*(py);
        IKReal x8=(j0)*(j0)*(j0);
        IKReal x9=(pz)*(pz)*(pz);
        IKReal x10=((0.0164865600000000)*(x1));
        IKReal x11=(j0)*(j0)*(j0)*(j0);
        IKReal x12=(py)*(py)*(py)*(py);
        IKReal x13=(pz)*(pz)*(pz)*(pz);
        IKReal x14=(px)*(px)*(px)*(px);
        coniccoeffs[0]=((((1.78059967631915e-6)*(x0)*(x3)))+(((0.00102862940032368)*(x0)*(x2)))+(((-8.55938590617039e-5)*(cj3)*(sj3)*(x1)))+(((-0.0164865600000000)*(x0)))+(x10)+(((-0.00102862940032368)*(x1)*(x2)))+(((8.55938590617039e-5)*(cj3)*(sj3)*(x0)))+(((-1.78059967631915e-6)*(x1)*(x3))));
        coniccoeffs[1]=((((-1.78059967631915e-6)*(px)*(py)*(x3)))+(((0.0164865600000000)*(px)*(py)))+(((-8.55938590617039e-5)*(cj3)*(px)*(py)*(sj3)))+(((-0.00102862940032368)*(px)*(py)*(x2))));
        coniccoeffs[2]=((((-0.00130142706043652)*(px)*(x2)))+(((-2.25282361347737e-6)*(px)*(x3)))+(((0.00514314700161840)*(px)*(x0)*(x2)))+(((-0.000855938590617039)*(cj3)*(j0)*(px)*(pz)*(sj3)))+(((0.00514314700161840)*(px)*(x1)*(x2)))+(((-0.0102862940032368)*(j0)*(px)*(pz)*(x2)))+(((8.90299838159575e-6)*(px)*(x3)*(x4)))+(((8.90299838159575e-6)*(px)*(x3)*(x5)))+(((8.90299838159575e-6)*(px)*(x1)*(x3)))+(((0.000427969295308520)*(cj3)*(px)*(sj3)*(x4)))+(((0.000427969295308520)*(cj3)*(px)*(sj3)*(x5)))+(((0.000427969295308520)*(cj3)*(px)*(sj3)*(x0)))+(((0.000427969295308520)*(cj3)*(px)*(sj3)*(x1)))+(((0.00514314700161840)*(px)*(x2)*(x4)))+(((0.00514314700161840)*(px)*(x2)*(x5)))+(((-0.000108293778454163)*(cj3)*(px)*(sj3)))+(((8.90299838159575e-6)*(px)*(x0)*(x3)))+(((-1.78059967631915e-5)*(j0)*(px)*(pz)*(x3))));
        coniccoeffs[3]=0;
        coniccoeffs[4]=((((-0.0102862940032368)*(j0)*(py)*(pz)*(x2)))+(((-1.78059967631915e-5)*(j0)*(py)*(pz)*(x3)))+(((-0.000855938590617039)*(cj3)*(j0)*(py)*(pz)*(sj3)))+(((0.00514314700161840)*(py)*(x2)*(x4)))+(((0.00514314700161840)*(py)*(x2)*(x5)))+(((-0.00130142706043652)*(py)*(x2)))+(((0.00514314700161840)*(py)*(x0)*(x2)))+(((-2.25282361347737e-6)*(py)*(x3)))+(((8.90299838159575e-6)*(py)*(x3)*(x5)))+(((8.90299838159575e-6)*(py)*(x3)*(x4)))+(((8.90299838159575e-6)*(py)*(x1)*(x3)))+(((0.00514314700161840)*(py)*(x1)*(x2)))+(((8.90299838159575e-6)*(py)*(x0)*(x3)))+(((-0.000108293778454163)*(cj3)*(py)*(sj3)))+(((0.000427969295308520)*(cj3)*(py)*(sj3)*(x1)))+(((0.000427969295308520)*(cj3)*(py)*(sj3)*(x0)))+(((0.000427969295308520)*(cj3)*(py)*(sj3)*(x5)))+(((0.000427969295308520)*(cj3)*(py)*(sj3)*(x4))));
        coniccoeffs[5]=((((-0.00427969295308519)*(cj3)*(sj3)*(x1)*(x5)))+(((-0.00427969295308519)*(cj3)*(sj3)*(x1)*(x4)))+(((8.53126400719886e-8)*(x3)))+(((-0.0514314700161840)*(x0)*(x2)*(x5)))+(((-0.0514314700161840)*(x0)*(x2)*(x4)))+(((-1.00000000000000)*(x10)))+(((0.000178059967631915)*(j0)*(pz)*(x1)*(x3)))+(((0.0130142706043652)*(x1)*(x2)))+(((0.102862940032368)*(j0)*(pz)*(x2)*(x4)))+(((0.102862940032368)*(j0)*(pz)*(x2)*(x5)))+(((2.25282361347737e-5)*(x1)*(x3)))+(((0.000997343925479927)*(cj3)*(sj3)*(x0)))+(((0.000178059967631915)*(j0)*(pz)*(x0)*(x3)))+(((0.00108293778454163)*(cj3)*(sj3)*(x1)))+(((0.00108293778454163)*(cj3)*(sj3)*(x4)))+(((0.00108293778454163)*(cj3)*(sj3)*(x5)))+(((-4.45149919079788e-5)*(x11)*(x3)))+(((-0.00427969295308519)*(cj3)*(sj3)*(x0)*(x5)))+(((-0.00427969295308519)*(cj3)*(sj3)*(x0)*(x4)))+(((-0.00427969295308519)*(cj3)*(sj3)*(x0)*(x1)))+(((-8.90299838159575e-5)*(x1)*(x3)*(x5)))+(((-8.90299838159575e-5)*(x1)*(x3)*(x4)))+(((0.0130142706043652)*(x2)*(x5)))+(((0.0130142706043652)*(x2)*(x4)))+(((-4.45149919079788e-5)*(x13)*(x3)))+(((-0.154294410048552)*(x2)*(x4)*(x5)))+(((-0.0514314700161840)*(x0)*(x1)*(x2)))+(((-0.000267089951447873)*(x3)*(x4)*(x5)))+(((0.00855938590617039)*(cj3)*(j0)*(pz)*(sj3)*(x0)))+(((0.00855938590617039)*(cj3)*(j0)*(pz)*(sj3)*(x1)))+(((0.00855938590617039)*(cj3)*(j0)*(pz)*(sj3)*(x4)))+(((0.00855938590617039)*(cj3)*(j0)*(pz)*(sj3)*(x5)))+(((-8.90299838159575e-5)*(x0)*(x1)*(x3)))+(((-0.0257157350080920)*(x13)*(x2)))+(((-0.00216587556908326)*(cj3)*(j0)*(pz)*(sj3)))+(((0.102862940032368)*(j0)*(pz)*(x0)*(x2)))+(((-0.0260285412087305)*(j0)*(pz)*(x2)))+(((0.000178059967631915)*(j0)*(pz)*(x3)*(x5)))+(((0.000178059967631915)*(j0)*(pz)*(x3)*(x4)))+(((4.10099933613311e-6)*(cj3)*(sj3)))+(((-0.0257157350080920)*(x14)*(x2)))+(((-0.0514314700161840)*(x1)*(x2)*(x4)))+(((-0.0514314700161840)*(x1)*(x2)*(x5)))+(((-0.0257157350080920)*(x11)*(x2)))+(((-4.45149919079788e-5)*(x12)*(x3)))+(((2.07476364584546e-5)*(x0)*(x3)))+(((4.92840086204478e-5)*(x2)))+(((-0.0257157350080920)*(x12)*(x2)))+(((0.102862940032368)*(j0)*(pz)*(x1)*(x2)))+(((-0.00213984647654260)*(cj3)*(sj3)*(x11)))+(((-0.00213984647654260)*(cj3)*(sj3)*(x12)))+(((-0.00213984647654260)*(cj3)*(sj3)*(x13)))+(((-0.00213984647654260)*(cj3)*(sj3)*(x14)))+(((-4.50564722695474e-5)*(j0)*(pz)*(x3)))+(((-4.45149919079788e-5)*(x14)*(x3)))+(((0.0119856412040415)*(x0)*(x2)))+(((-0.0128390788592556)*(cj3)*(sj3)*(x4)*(x5)))+(((-8.90299838159575e-5)*(x0)*(x3)*(x4)))+(((-8.90299838159575e-5)*(x0)*(x3)*(x5)))+(((2.25282361347737e-5)*(x3)*(x4)))+(((2.25282361347737e-5)*(x3)*(x5))));
        conicsolver(coniccoeffs,j1array,numsolutions);
        for(int ij1 = 0; ij1 < numsolutions; ++ij1)
        {
            j1 = j1array[ij1]; cj1 = IKcos(j1); sj1 = IKsin(j1);
        
        {
            IKReal j4eval[1];
            j4eval[0]=((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))));
            if( IKabs(j4eval[0]) < 0.000001  )
            {
                {
                IKReal evalcond[1];
                    evalcond[0]=((IKfmod(((0.0415817987708408)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                    if( IKabs(evalcond[0]) < 0.000010  )
                    {
                    {
                    IKReal j4array[2], cj4array[2], sj4array[2];
                    bool j4valid[2]={false};
                    sj4array[0]=((((5.76460752303423e+17)*(px)*(sj1)))+(((-5.76460752303423e+17)*(cj1)*(py))));
                    if( sj4array[0] >= -1.0001 && sj4array[0] <= 1.0001 )
                    {
                        j4valid[0] = j4valid[1] = true;
                        j4array[0] = IKasin(sj4array[0]);
                        cj4array[0] = IKcos(j4array[0]);
                        sj4array[1] = sj4array[0];
                        j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
                        cj4array[1] = -cj4array[0];
                    }
                    else if( isnan(sj4array[0]) )
                    {
                        // probably any value will work
                        j4valid[0] = true;
                        cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                    }
                    if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
                        j4valid[1]=false;
                    for(int ij4 = 0; ij4 < 2; ++ij4)
                    {
                    if( !j4valid[ij4] ) { continue; }
                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                    
                    {
                        IKReal j2eval[1];
                        j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                        if( IKabs(j2eval[0]) < 0.000001  )
                        {
                            {
                                if( 1 )
                                {
                                {
                                    IKReal j2eval[1];
                                    j2eval[0]=((((-0.160000000000000)*(cj1)))+(((-0.256800000000000)*(cj1)*(cj4)))+(((-0.103041000000000)*(cj1)*((sj4)*(sj4))))+(((-0.103041000000000)*(cj1)*((cj4)*(cj4)))));
                                    if( IKabs(j2eval[0]) < 0.000001  )
                                    {
                                        {
                                        IKReal evalcond[4];
                                            IKReal x15=((1.73472347597681e-18)*(sj4));
                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((-1.57079632679490)+(3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979)))));
                                            evalcond[1]=((((-1.00000000000000)*(x15)))+(px));
                                            evalcond[2]=((0.253041000000000)+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((0.200000000000000)*(py)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                            evalcond[3]=((x15)+(((-1.00000000000000)*(px))));
                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                            {
                                            {
                                                IKReal j2eval[1];
                                                j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                if( IKabs(j2eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.0400000000000000)+(((-0.400000000000000)*(py)))+(((-0.321000000000000)*(cj4)*(py)))+(((-0.321000000000000)*(pz)*(sj4)))+(((0.321000000000000)*(j0)*(sj4)))+(((0.0321000000000000)*(cj4))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((-0.321000000000000)*(py)*(sj4)))+(((0.0321000000000000)*(sj4)))+(((0.321000000000000)*(cj4)*(pz))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                continue;
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x16=((((-1.00000000000000)*(pz)))+(j0));
                                                                            IKReal x17=((0.400000000000000)*(pz));
                                                                            IKReal x18=((0.0321000000000000)*(sj4));
                                                                            IKReal x19=((0.321000000000000)*(cj4)*(pz));
                                                                            IKReal x20=((x17)+(x19)+(x18));
                                                                            IKReal x21=((0.400000000000000)*(j0));
                                                                            IKReal x22=((0.321000000000000)*(py)*(sj4));
                                                                            IKReal x23=((0.321000000000000)*(cj4)*(j0));
                                                                            IKReal x24=((x21)+(x22)+(x23));
                                                                            IKReal x25=((((-1.00000000000000)*(x24)))+(x20));
                                                                            IKReal x26=((IKabs(x25) != 0)?((IKReal)1/(x25)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x26)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x16)*(x16))))))), ((x26)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x16)*(((0.100000000000000)+(((-1.00000000000000)*(py)))))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x90=((0.100000000000000)+(((-1.00000000000000)*(py))));
                                                                IKReal x91=((0.321000000000000)*(j0)*(sj4));
                                                                IKReal x92=((0.0321000000000000)*(cj4));
                                                                IKReal x93=((0.0400000000000000)+(x91)+(x92));
                                                                IKReal x94=((0.400000000000000)*(py));
                                                                IKReal x95=((0.321000000000000)*(pz)*(sj4));
                                                                IKReal x96=((0.321000000000000)*(cj4)*(py));
                                                                IKReal x97=((x95)+(x94)+(x96));
                                                                IKReal x98=((x93)+(((-1.00000000000000)*(x97))));
                                                                IKReal x99=((IKabs(x98) != 0)?((IKReal)1/(x98)):(IKReal)1.0e30);
                                                                j2array[0]=IKatan2(((x99)*(((((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x90)*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x99)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x90)*(x90))))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                                    bool j2valid[1]={false};
                                                    IKReal x100=((0.256800000000000)*(cj4));
                                                    IKReal x101=(sj4)*(sj4);
                                                    IKReal x102=((0.103041000000000)*(x101));
                                                    IKReal x103=(cj4)*(cj4);
                                                    IKReal x104=((0.103041000000000)*(x103));
                                                    IKReal x105=((0.160000000000000)+(x104)+(x100)+(x102));
                                                    IKReal x106=((IKabs(x105) != 0)?((IKReal)1/(x105)):(IKReal)1.0e30);
                                                    IKReal x107=((((-1.00000000000000)*(pz)))+(j0));
                                                    IKReal x108=((0.321000000000000)*(cj4));
                                                    IKReal x109=((0.400000000000000)+(x108));
                                                    IKReal x110=((0.100000000000000)+(((-1.00000000000000)*(py))));
                                                    j2array[0]=IKatan2(((-1.00000000000000)*(x106)*(((((-0.321000000000000)*(sj4)*(x110)))+(((-1.00000000000000)*(x107)*(x109)))))), ((-1.00000000000000)*(x106)*(((((-0.321000000000000)*(sj4)*(x107)))+(((x109)*(x110)))))));
                                                    sj2array[0]=IKsin(j2array[0]);
                                                    cj2array[0]=IKcos(j2array[0]);
                                                    if( j2array[0] > IKPI )
                                                        j2array[0]-=IK2PI;
                                                    else if( j2array[0] < -IKPI )
                                                        j2array[0]+=IK2PI;
                                                    j2valid[0] = true;
                                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                                    {
                                                    if( !j2valid[ij2] ) { continue; }
                                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                            IKReal x111=((1.73472347597681e-18)*(sj4));
                                            IKReal x112=((x111)+(px));
                                            IKReal x113=((-1.00000000000000)*(x112));
                                            evalcond[0]=((IKfmod(((1.57079632679490)+(3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                            evalcond[1]=x113;
                                            evalcond[2]=((0.253041000000000)+(((-0.200000000000000)*(py)))+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                            evalcond[3]=x113;
                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                            {
                                            {
                                                IKReal j2eval[1];
                                                j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                if( IKabs(j2eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.0400000000000000)+(((-0.321000000000000)*(pz)*(sj4)))+(((0.400000000000000)*(py)))+(((0.321000000000000)*(j0)*(sj4)))+(((0.321000000000000)*(cj4)*(py)))+(((0.0321000000000000)*(cj4))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((((0.321000000000000)*(py)*(sj4)))+(((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((0.0321000000000000)*(sj4)))+(((0.321000000000000)*(cj4)*(pz))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                continue;
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x114=((((-1.00000000000000)*(pz)))+(j0));
                                                                            IKReal x115=((0.321000000000000)*(py)*(sj4));
                                                                            IKReal x116=((0.400000000000000)*(pz));
                                                                            IKReal x117=((0.0321000000000000)*(sj4));
                                                                            IKReal x118=((0.321000000000000)*(cj4)*(pz));
                                                                            IKReal x119=((x117)+(x116)+(x115)+(x118));
                                                                            IKReal x120=((0.400000000000000)*(j0));
                                                                            IKReal x121=((0.321000000000000)*(cj4)*(j0));
                                                                            IKReal x122=((x120)+(x121));
                                                                            IKReal x123=((((-1.00000000000000)*(x122)))+(x119));
                                                                            IKReal x124=((IKabs(x123) != 0)?((IKReal)1/(x123)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x124)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x114)*(x114))))))), ((x124)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x114)*(((0.100000000000000)+(py)))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x125=((0.100000000000000)+(py));
                                                                IKReal x126=((0.400000000000000)*(py));
                                                                IKReal x127=((0.321000000000000)*(j0)*(sj4));
                                                                IKReal x128=((0.321000000000000)*(cj4)*(py));
                                                                IKReal x129=((0.0321000000000000)*(cj4));
                                                                IKReal x130=((0.0400000000000000)+(x126)+(x127)+(x128)+(x129));
                                                                IKReal x131=((0.321000000000000)*(pz)*(sj4));
                                                                IKReal x132=((x130)+(((-1.00000000000000)*(x131))));
                                                                IKReal x133=((IKabs(x132) != 0)?((IKReal)1/(x132)):(IKReal)1.0e30);
                                                                j2array[0]=IKatan2(((x133)*(((((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x125)*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x133)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x125)*(x125))))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                                    bool j2valid[1]={false};
                                                    IKReal x134=((0.256800000000000)*(cj4));
                                                    IKReal x135=(sj4)*(sj4);
                                                    IKReal x136=((0.103041000000000)*(x135));
                                                    IKReal x137=(cj4)*(cj4);
                                                    IKReal x138=((0.103041000000000)*(x137));
                                                    IKReal x139=((0.160000000000000)+(x134)+(x136)+(x138));
                                                    IKReal x140=((IKabs(x139) != 0)?((IKReal)1/(x139)):(IKReal)1.0e30);
                                                    IKReal x141=((0.100000000000000)+(py));
                                                    IKReal x142=((0.321000000000000)*(cj4));
                                                    IKReal x143=((0.400000000000000)+(x142));
                                                    IKReal x144=((((-1.00000000000000)*(pz)))+(j0));
                                                    j2array[0]=IKatan2(((-1.00000000000000)*(x140)*(((((-1.00000000000000)*(x143)*(x144)))+(((-0.321000000000000)*(sj4)*(x141)))))), ((-1.00000000000000)*(x140)*(((((x141)*(x143)))+(((-0.321000000000000)*(sj4)*(x144)))))));
                                                    sj2array[0]=IKsin(j2array[0]);
                                                    cj2array[0]=IKcos(j2array[0]);
                                                    if( j2array[0] > IKPI )
                                                        j2array[0]-=IK2PI;
                                                    else if( j2array[0] < -IKPI )
                                                        j2array[0]+=IK2PI;
                                                    j2valid[0] = true;
                                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                                    {
                                                    if( !j2valid[ij2] ) { continue; }
                                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                            if( 1 )
                                            {
                                            {
                                                IKReal j2eval[1];
                                                j2eval[0]=((((-0.103041000000000)*(sj1)*((sj4)*(sj4))))+(((-0.256800000000000)*(cj4)*(sj1)))+(((-0.160000000000000)*(sj1)))+(((-0.103041000000000)*(sj1)*((cj4)*(cj4)))));
                                                if( IKabs(j2eval[0]) < 0.000001  )
                                                {
                                                    {
                                                    IKReal evalcond[4];
                                                        IKReal x145=((1.73472347597681e-18)*(sj4));
                                                        IKReal x146=((x145)+(py));
                                                        IKReal x147=((-1.00000000000000)*(x146));
                                                        evalcond[0]=((IKfmod(((3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                        evalcond[1]=x147;
                                                        evalcond[2]=((0.253041000000000)+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((0.200000000000000)*(px)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                        evalcond[3]=x147;
                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((0.0400000000000000)+(((-0.400000000000000)*(px)))+(((-0.321000000000000)*(cj4)*(px)))+(((-0.321000000000000)*(pz)*(sj4)))+(((0.321000000000000)*(j0)*(sj4)))+(((0.0321000000000000)*(cj4))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                {
                                                                                    IKReal j2eval[1];
                                                                                    j2eval[0]=((((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((-0.321000000000000)*(px)*(sj4)))+(((0.0321000000000000)*(sj4)))+(((0.321000000000000)*(cj4)*(pz))));
                                                                                    if( IKabs(j2eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            continue;
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j2array[1], cj2array[1], sj2array[1];
                                                                                        bool j2valid[1]={false};
                                                                                        IKReal x148=((((-1.00000000000000)*(pz)))+(j0));
                                                                                        IKReal x149=((0.400000000000000)*(pz));
                                                                                        IKReal x150=((0.0321000000000000)*(sj4));
                                                                                        IKReal x151=((0.321000000000000)*(cj4)*(pz));
                                                                                        IKReal x152=((x151)+(x150)+(x149));
                                                                                        IKReal x153=((0.400000000000000)*(j0));
                                                                                        IKReal x154=((0.321000000000000)*(cj4)*(j0));
                                                                                        IKReal x155=((0.321000000000000)*(px)*(sj4));
                                                                                        IKReal x156=((x155)+(x154)+(x153));
                                                                                        IKReal x157=((((-1.00000000000000)*(x156)))+(x152));
                                                                                        IKReal x158=((IKabs(x157) != 0)?((IKReal)1/(x157)):(IKReal)1.0e30);
                                                                                        j2array[0]=IKatan2(((x158)*(((((-1.00000000000000)*((x148)*(x148))))+(((0.103041000000000)*((sj4)*(sj4))))))), ((x158)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x148)*(((0.100000000000000)+(((-1.00000000000000)*(px)))))))))));
                                                                                        sj2array[0]=IKsin(j2array[0]);
                                                                                        cj2array[0]=IKcos(j2array[0]);
                                                                                        if( j2array[0] > IKPI )
                                                                                            j2array[0]-=IK2PI;
                                                                                        else if( j2array[0] < -IKPI )
                                                                                            j2array[0]+=IK2PI;
                                                                                        j2valid[0] = true;
                                                                                        for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                                        {
                                                                                        if( !j2valid[ij2] ) { continue; }
                                                                                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x159=((0.100000000000000)+(((-1.00000000000000)*(px))));
                                                                            IKReal x160=((0.321000000000000)*(j0)*(sj4));
                                                                            IKReal x161=((0.0321000000000000)*(cj4));
                                                                            IKReal x162=((0.0400000000000000)+(x160)+(x161));
                                                                            IKReal x163=((0.400000000000000)*(px));
                                                                            IKReal x164=((0.321000000000000)*(pz)*(sj4));
                                                                            IKReal x165=((0.321000000000000)*(cj4)*(px));
                                                                            IKReal x166=((x163)+(x164)+(x165));
                                                                            IKReal x167=((((-1.00000000000000)*(x166)))+(x162));
                                                                            IKReal x168=((IKabs(x167) != 0)?((IKReal)1/(x167)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x168)*(((((x159)*(((((-1.00000000000000)*(pz)))+(j0)))))+(((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))))), ((x168)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x159)*(x159))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x169=((0.256800000000000)*(cj4));
                                                                IKReal x170=(sj4)*(sj4);
                                                                IKReal x171=((0.103041000000000)*(x170));
                                                                IKReal x172=(cj4)*(cj4);
                                                                IKReal x173=((0.103041000000000)*(x172));
                                                                IKReal x174=((0.160000000000000)+(x171)+(x173)+(x169));
                                                                IKReal x175=((IKabs(x174) != 0)?((IKReal)1/(x174)):(IKReal)1.0e30);
                                                                IKReal x176=((((-1.00000000000000)*(pz)))+(j0));
                                                                IKReal x177=((0.321000000000000)*(cj4));
                                                                IKReal x178=((0.400000000000000)+(x177));
                                                                IKReal x179=((0.100000000000000)+(((-1.00000000000000)*(px))));
                                                                j2array[0]=IKatan2(((-1.00000000000000)*(x175)*(((((-1.00000000000000)*(x176)*(x178)))+(((-0.321000000000000)*(sj4)*(x179)))))), ((-1.00000000000000)*(x175)*(((((x178)*(x179)))+(((-0.321000000000000)*(sj4)*(x176)))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                        IKReal x180=((1.73472347597681e-18)*(sj4));
                                                        evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                        evalcond[1]=((((-1.00000000000000)*(x180)))+(py));
                                                        evalcond[2]=((0.253041000000000)+(((-0.200000000000000)*(px)))+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                        evalcond[3]=((x180)+(((-1.00000000000000)*(py))));
                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((0.0400000000000000)+(((-0.321000000000000)*(pz)*(sj4)))+(((0.400000000000000)*(px)))+(((0.321000000000000)*(j0)*(sj4)))+(((0.321000000000000)*(cj4)*(px)))+(((0.0321000000000000)*(cj4))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                {
                                                                                    IKReal j2eval[1];
                                                                                    j2eval[0]=((((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((0.0321000000000000)*(sj4)))+(((0.321000000000000)*(cj4)*(pz)))+(((0.321000000000000)*(px)*(sj4))));
                                                                                    if( IKabs(j2eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            continue;
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j2array[1], cj2array[1], sj2array[1];
                                                                                        bool j2valid[1]={false};
                                                                                        IKReal x181=((((-1.00000000000000)*(pz)))+(j0));
                                                                                        IKReal x182=((0.400000000000000)*(pz));
                                                                                        IKReal x183=((0.0321000000000000)*(sj4));
                                                                                        IKReal x184=((0.321000000000000)*(cj4)*(pz));
                                                                                        IKReal x185=((0.321000000000000)*(px)*(sj4));
                                                                                        IKReal x186=((x184)+(x185)+(x182)+(x183));
                                                                                        IKReal x187=((0.400000000000000)*(j0));
                                                                                        IKReal x188=((0.321000000000000)*(cj4)*(j0));
                                                                                        IKReal x189=((x187)+(x188));
                                                                                        IKReal x190=((((-1.00000000000000)*(x189)))+(x186));
                                                                                        IKReal x191=((IKabs(x190) != 0)?((IKReal)1/(x190)):(IKReal)1.0e30);
                                                                                        j2array[0]=IKatan2(((x191)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x181)*(x181))))))), ((x191)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x181)*(((0.100000000000000)+(px)))))))));
                                                                                        sj2array[0]=IKsin(j2array[0]);
                                                                                        cj2array[0]=IKcos(j2array[0]);
                                                                                        if( j2array[0] > IKPI )
                                                                                            j2array[0]-=IK2PI;
                                                                                        else if( j2array[0] < -IKPI )
                                                                                            j2array[0]+=IK2PI;
                                                                                        j2valid[0] = true;
                                                                                        for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                                        {
                                                                                        if( !j2valid[ij2] ) { continue; }
                                                                                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x192=((0.100000000000000)+(px));
                                                                            IKReal x193=((0.400000000000000)*(px));
                                                                            IKReal x194=((0.321000000000000)*(j0)*(sj4));
                                                                            IKReal x195=((0.321000000000000)*(cj4)*(px));
                                                                            IKReal x196=((0.0321000000000000)*(cj4));
                                                                            IKReal x197=((0.0400000000000000)+(x193)+(x196)+(x195)+(x194));
                                                                            IKReal x198=((0.321000000000000)*(pz)*(sj4));
                                                                            IKReal x199=((((-1.00000000000000)*(x198)))+(x197));
                                                                            IKReal x200=((IKabs(x199) != 0)?((IKReal)1/(x199)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x200)*(((((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x192)*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x200)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x192)*(x192))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x201=((0.256800000000000)*(cj4));
                                                                IKReal x202=(sj4)*(sj4);
                                                                IKReal x203=((0.103041000000000)*(x202));
                                                                IKReal x204=(cj4)*(cj4);
                                                                IKReal x205=((0.103041000000000)*(x204));
                                                                IKReal x206=((0.160000000000000)+(x205)+(x203)+(x201));
                                                                IKReal x207=((IKabs(x206) != 0)?((IKReal)1/(x206)):(IKReal)1.0e30);
                                                                IKReal x208=((((-1.00000000000000)*(pz)))+(j0));
                                                                IKReal x209=((0.100000000000000)+(px));
                                                                IKReal x210=((0.321000000000000)*(cj4));
                                                                IKReal x211=((0.400000000000000)+(x210));
                                                                j2array[0]=IKatan2(((-1.00000000000000)*(x207)*(((((-0.321000000000000)*(sj4)*(x209)))+(((-1.00000000000000)*(x208)*(x211)))))), ((-1.00000000000000)*(x207)*(((((-0.321000000000000)*(sj4)*(x208)))+(((x209)*(x211)))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                        if( 1 )
                                                        {
                                                        continue;
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                                    bool j2valid[1]={false};
                                                    IKReal x212=((0.100000000000000)*(sj1));
                                                    IKReal x213=((1.73472347597681e-18)*(cj1)*(sj4));
                                                    IKReal x214=((x213)+(py));
                                                    IKReal x215=((x212)+(((-1.00000000000000)*(x214))));
                                                    IKReal x216=(cj4)*(cj4);
                                                    IKReal x217=((0.103041000000000)*(sj1)*(x216));
                                                    IKReal x218=(sj4)*(sj4);
                                                    IKReal x219=((0.103041000000000)*(sj1)*(x218));
                                                    IKReal x220=((0.256800000000000)*(cj4)*(sj1));
                                                    IKReal x221=((0.160000000000000)*(sj1));
                                                    IKReal x222=((x217)+(x219)+(x221)+(x220));
                                                    IKReal x223=((-1.00000000000000)*(x222));
                                                    IKReal x224=((IKabs(x223) != 0)?((IKReal)1/(x223)):(IKReal)1.0e30);
                                                    j2array[0]=IKatan2(((x224)*(((((-0.321000000000000)*(sj4)*(x215)))+(((((pz)+(((-1.00000000000000)*(j0)))))*(((((0.400000000000000)*(sj1)))+(((0.321000000000000)*(cj4)*(sj1)))))))))), ((x224)*(((((-0.321000000000000)*(sj1)*(sj4)*(((((-1.00000000000000)*(pz)))+(j0)))))+(((x215)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))))));
                                                    sj2array[0]=IKsin(j2array[0]);
                                                    cj2array[0]=IKcos(j2array[0]);
                                                    if( j2array[0] > IKPI )
                                                        j2array[0]-=IK2PI;
                                                    else if( j2array[0] < -IKPI )
                                                        j2array[0]+=IK2PI;
                                                    j2valid[0] = true;
                                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                                    {
                                                    if( !j2valid[ij2] ) { continue; }
                                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                        }
                                        }
                                        }
                                        }
                                    
                                    } else
                                    {
                                        {
                                        IKReal j2array[1], cj2array[1], sj2array[1];
                                        bool j2valid[1]={false};
                                        IKReal x225=((1.73472347597681e-18)*(sj1)*(sj4));
                                        IKReal x226=((0.100000000000000)*(cj1));
                                        IKReal x227=((x225)+(x226));
                                        IKReal x228=((x227)+(((-1.00000000000000)*(px))));
                                        IKReal x229=((0.256800000000000)*(cj1)*(cj4));
                                        IKReal x230=(sj4)*(sj4);
                                        IKReal x231=((0.103041000000000)*(cj1)*(x230));
                                        IKReal x232=((0.160000000000000)*(cj1));
                                        IKReal x233=(cj4)*(cj4);
                                        IKReal x234=((0.103041000000000)*(cj1)*(x233));
                                        IKReal x235=((x229)+(x232)+(x231)+(x234));
                                        IKReal x236=((-1.00000000000000)*(x235));
                                        IKReal x237=((IKabs(x236) != 0)?((IKReal)1/(x236)):(IKReal)1.0e30);
                                        j2array[0]=IKatan2(((x237)*(((((((pz)+(((-1.00000000000000)*(j0)))))*(((((0.400000000000000)*(cj1)))+(((0.321000000000000)*(cj1)*(cj4)))))))+(((-0.321000000000000)*(sj4)*(x228)))))), ((x237)*(((((x228)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((-0.321000000000000)*(cj1)*(sj4)*(((((-1.00000000000000)*(pz)))+(j0)))))))));
                                        sj2array[0]=IKsin(j2array[0]);
                                        cj2array[0]=IKcos(j2array[0]);
                                        if( j2array[0] > IKPI )
                                            j2array[0]-=IK2PI;
                                        else if( j2array[0] < -IKPI )
                                            j2array[0]+=IK2PI;
                                        j2valid[0] = true;
                                        for(int ij2 = 0; ij2 < 1; ++ij2)
                                        {
                                        if( !j2valid[ij2] ) { continue; }
                                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                        
                                        rotationfunction0(vsolutions);
                                        }
                                        }
                                    
                                    }
                                
                                }
                            
                            } else
                            {
                            }
                            }
                        
                        } else
                        {
                            {
                            IKReal j2array[1], cj2array[1], sj2array[1];
                            bool j2valid[1]={false};
                            IKReal x238=((0.256800000000000)*(cj4));
                            IKReal x239=(sj4)*(sj4);
                            IKReal x240=((0.103041000000000)*(x239));
                            IKReal x241=(cj4)*(cj4);
                            IKReal x242=((0.103041000000000)*(x241));
                            IKReal x243=((0.160000000000000)+(x238)+(x242)+(x240));
                            IKReal x244=((IKabs(x243) != 0)?((IKReal)1/(x243)):(IKReal)1.0e30);
                            IKReal x245=((0.321000000000000)*(cj4));
                            IKReal x246=((0.400000000000000)+(x245));
                            IKReal x247=((cj1)*(px));
                            IKReal x248=((py)*(sj1));
                            IKReal x249=((x247)+(x248));
                            IKReal x250=((0.100000000000000)+(((-1.00000000000000)*(x249))));
                            IKReal x251=((((-1.00000000000000)*(pz)))+(j0));
                            j2array[0]=IKatan2(((-1.00000000000000)*(x244)*(((((-1.00000000000000)*(x246)*(x251)))+(((-0.321000000000000)*(sj4)*(x250)))))), ((-1.00000000000000)*(x244)*(((((x246)*(x250)))+(((-0.321000000000000)*(sj4)*(x251)))))));
                            sj2array[0]=IKsin(j2array[0]);
                            cj2array[0]=IKcos(j2array[0]);
                            if( j2array[0] > IKPI )
                                j2array[0]-=IK2PI;
                            else if( j2array[0] < -IKPI )
                                j2array[0]+=IK2PI;
                            j2valid[0] = true;
                            for(int ij2 = 0; ij2 < 1; ++ij2)
                            {
                            if( !j2valid[ij2] ) { continue; }
                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                            
                            rotationfunction0(vsolutions);
                            }
                            }
                        
                        }
                    
                    }
                    }
                    }
                
                } else
                {
                    evalcond[0]=((IKfmod(((-3.10001085481895)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                    if( IKabs(evalcond[0]) < 0.000010  )
                    {
                    {
                    IKReal j4array[2], cj4array[2], sj4array[2];
                    bool j4valid[2]={false};
                    sj4array[0]=((((-1.69547280089242e+16)*(px)*(sj1)))+(((1.69547280089242e+16)*(cj1)*(py))));
                    if( sj4array[0] >= -1.0001 && sj4array[0] <= 1.0001 )
                    {
                        j4valid[0] = j4valid[1] = true;
                        j4array[0] = IKasin(sj4array[0]);
                        cj4array[0] = IKcos(j4array[0]);
                        sj4array[1] = sj4array[0];
                        j4array[1] = j4array[0] > 0 ? (IKPI-j4array[0]) : (-IKPI-j4array[0]);
                        cj4array[1] = -cj4array[0];
                    }
                    else if( isnan(sj4array[0]) )
                    {
                        // probably any value will work
                        j4valid[0] = true;
                        cj4array[0] = 1; sj4array[0] = 0; j4array[0] = 0;
                    }
                    if( j4valid[0] && j4valid[1] && IKabs(cj4array[0]-cj4array[1]) < 0.0001 && IKabs(sj4array[0]-sj4array[1]) < 0.0001 )
                        j4valid[1]=false;
                    for(int ij4 = 0; ij4 < 2; ++ij4)
                    {
                    if( !j4valid[ij4] ) { continue; }
                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                    
                    {
                        IKReal j2eval[1];
                        j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                        if( IKabs(j2eval[0]) < 0.000001  )
                        {
                            {
                                if( 1 )
                                {
                                {
                                    IKReal j2eval[1];
                                    j2eval[0]=((((-0.160000000000000)*(cj1)))+(((-0.256800000000000)*(cj1)*(cj4)))+(((-0.103041000000000)*(cj1)*((sj4)*(sj4))))+(((-0.103041000000000)*(cj1)*((cj4)*(cj4)))));
                                    if( IKabs(j2eval[0]) < 0.000001  )
                                    {
                                        {
                                        IKReal evalcond[4];
                                            IKReal x252=((5.89805981832114e-17)*(sj4));
                                            IKReal x253=((x252)+(px));
                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((-1.57079632679490)+(3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979)))));
                                            evalcond[1]=x253;
                                            evalcond[2]=((0.253041000000000)+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((0.200000000000000)*(py)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                            evalcond[3]=((-1.00000000000000)*(x253));
                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                            {
                                            {
                                                IKReal j2eval[1];
                                                j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                if( IKabs(j2eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.0400000000000000)+(((-0.400000000000000)*(py)))+(((-0.321000000000000)*(cj4)*(py)))+(((-0.321000000000000)*(j0)*(sj4)))+(((0.321000000000000)*(pz)*(sj4)))+(((0.0321000000000000)*(cj4))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((((0.321000000000000)*(py)*(sj4)))+(((-0.0321000000000000)*(sj4)))+(((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((0.321000000000000)*(cj4)*(pz))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                continue;
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x254=((((-1.00000000000000)*(pz)))+(j0));
                                                                            IKReal x255=((0.321000000000000)*(py)*(sj4));
                                                                            IKReal x256=((0.400000000000000)*(pz));
                                                                            IKReal x257=((0.321000000000000)*(cj4)*(pz));
                                                                            IKReal x258=((x255)+(x256)+(x257));
                                                                            IKReal x259=((0.400000000000000)*(j0));
                                                                            IKReal x260=((0.321000000000000)*(cj4)*(j0));
                                                                            IKReal x261=((0.0321000000000000)*(sj4));
                                                                            IKReal x262=((x259)+(x261)+(x260));
                                                                            IKReal x263=((x258)+(((-1.00000000000000)*(x262))));
                                                                            IKReal x264=((IKabs(x263) != 0)?((IKReal)1/(x263)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x264)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x254)*(x254))))))), ((x264)*(((((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x254)*(((0.100000000000000)+(((-1.00000000000000)*(py)))))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x265=((0.100000000000000)+(((-1.00000000000000)*(py))));
                                                                IKReal x266=((0.321000000000000)*(pz)*(sj4));
                                                                IKReal x267=((0.0321000000000000)*(cj4));
                                                                IKReal x268=((0.0400000000000000)+(x267)+(x266));
                                                                IKReal x269=((0.400000000000000)*(py));
                                                                IKReal x270=((0.321000000000000)*(j0)*(sj4));
                                                                IKReal x271=((0.321000000000000)*(cj4)*(py));
                                                                IKReal x272=((x269)+(x270)+(x271));
                                                                IKReal x273=((x268)+(((-1.00000000000000)*(x272))));
                                                                IKReal x274=((IKabs(x273) != 0)?((IKReal)1/(x273)):(IKReal)1.0e30);
                                                                j2array[0]=IKatan2(((x274)*(((((x265)*(((((-1.00000000000000)*(pz)))+(j0)))))+(((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))))), ((x274)*(((((-1.00000000000000)*((x265)*(x265))))+(((0.103041000000000)*((sj4)*(sj4))))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                                    bool j2valid[1]={false};
                                                    IKReal x275=((0.256800000000000)*(cj4));
                                                    IKReal x276=(sj4)*(sj4);
                                                    IKReal x277=((0.103041000000000)*(x276));
                                                    IKReal x278=(cj4)*(cj4);
                                                    IKReal x279=((0.103041000000000)*(x278));
                                                    IKReal x280=((0.160000000000000)+(x277)+(x275)+(x279));
                                                    IKReal x281=((IKabs(x280) != 0)?((IKReal)1/(x280)):(IKReal)1.0e30);
                                                    IKReal x282=((0.321000000000000)*(cj4));
                                                    IKReal x283=((0.400000000000000)+(x282));
                                                    IKReal x284=((0.100000000000000)+(((-1.00000000000000)*(py))));
                                                    IKReal x285=((((-1.00000000000000)*(pz)))+(j0));
                                                    j2array[0]=IKatan2(((-1.00000000000000)*(x281)*(((((0.321000000000000)*(sj4)*(x284)))+(((-1.00000000000000)*(x283)*(x285)))))), ((-1.00000000000000)*(x281)*(((((0.321000000000000)*(sj4)*(x285)))+(((x283)*(x284)))))));
                                                    sj2array[0]=IKsin(j2array[0]);
                                                    cj2array[0]=IKcos(j2array[0]);
                                                    if( j2array[0] > IKPI )
                                                        j2array[0]-=IK2PI;
                                                    else if( j2array[0] < -IKPI )
                                                        j2array[0]+=IK2PI;
                                                    j2valid[0] = true;
                                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                                    {
                                                    if( !j2valid[ij2] ) { continue; }
                                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                            IKReal x286=((5.89805981832114e-17)*(sj4));
                                            IKReal x287=((x286)+(((-1.00000000000000)*(px))));
                                            evalcond[0]=((IKfmod(((1.57079632679490)+(3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                            evalcond[1]=x287;
                                            evalcond[2]=((0.253041000000000)+(((-0.200000000000000)*(py)))+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                            evalcond[3]=x287;
                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                            {
                                            {
                                                IKReal j2eval[1];
                                                j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                if( IKabs(j2eval[0]) < 0.000001  )
                                                {
                                                    {
                                                        if( 1 )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.0400000000000000)+(((-0.321000000000000)*(j0)*(sj4)))+(((0.400000000000000)*(py)))+(((0.321000000000000)*(pz)*(sj4)))+(((0.321000000000000)*(cj4)*(py)))+(((0.0321000000000000)*(cj4))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((((-0.0321000000000000)*(sj4)))+(((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((-0.321000000000000)*(py)*(sj4)))+(((0.321000000000000)*(cj4)*(pz))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                continue;
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x288=((((-1.00000000000000)*(pz)))+(j0));
                                                                            IKReal x289=((0.400000000000000)*(pz));
                                                                            IKReal x290=((0.321000000000000)*(cj4)*(pz));
                                                                            IKReal x291=((x290)+(x289));
                                                                            IKReal x292=((0.400000000000000)*(j0));
                                                                            IKReal x293=((0.321000000000000)*(py)*(sj4));
                                                                            IKReal x294=((0.321000000000000)*(cj4)*(j0));
                                                                            IKReal x295=((0.0321000000000000)*(sj4));
                                                                            IKReal x296=((x292)+(x293)+(x294)+(x295));
                                                                            IKReal x297=((((-1.00000000000000)*(x296)))+(x291));
                                                                            IKReal x298=((IKabs(x297) != 0)?((IKReal)1/(x297)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x298)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x288)*(x288))))))), ((x298)*(((((x288)*(((0.100000000000000)+(py)))))+(((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x299=((0.100000000000000)+(py));
                                                                IKReal x300=((0.400000000000000)*(py));
                                                                IKReal x301=((0.321000000000000)*(pz)*(sj4));
                                                                IKReal x302=((0.321000000000000)*(cj4)*(py));
                                                                IKReal x303=((0.0321000000000000)*(cj4));
                                                                IKReal x304=((0.0400000000000000)+(x302)+(x303)+(x300)+(x301));
                                                                IKReal x305=((0.321000000000000)*(j0)*(sj4));
                                                                IKReal x306=((x304)+(((-1.00000000000000)*(x305))));
                                                                IKReal x307=((IKabs(x306) != 0)?((IKReal)1/(x306)):(IKReal)1.0e30);
                                                                j2array[0]=IKatan2(((x307)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x299)*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x307)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x299)*(x299))))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                                    bool j2valid[1]={false};
                                                    IKReal x308=((0.256800000000000)*(cj4));
                                                    IKReal x309=(sj4)*(sj4);
                                                    IKReal x310=((0.103041000000000)*(x309));
                                                    IKReal x311=(cj4)*(cj4);
                                                    IKReal x312=((0.103041000000000)*(x311));
                                                    IKReal x313=((0.160000000000000)+(x308)+(x310)+(x312));
                                                    IKReal x314=((IKabs(x313) != 0)?((IKReal)1/(x313)):(IKReal)1.0e30);
                                                    IKReal x315=((0.100000000000000)+(py));
                                                    IKReal x316=((0.321000000000000)*(cj4));
                                                    IKReal x317=((0.400000000000000)+(x316));
                                                    IKReal x318=((((-1.00000000000000)*(pz)))+(j0));
                                                    j2array[0]=IKatan2(((-1.00000000000000)*(x314)*(((((0.321000000000000)*(sj4)*(x315)))+(((-1.00000000000000)*(x317)*(x318)))))), ((-1.00000000000000)*(x314)*(((((x315)*(x317)))+(((0.321000000000000)*(sj4)*(x318)))))));
                                                    sj2array[0]=IKsin(j2array[0]);
                                                    cj2array[0]=IKcos(j2array[0]);
                                                    if( j2array[0] > IKPI )
                                                        j2array[0]-=IK2PI;
                                                    else if( j2array[0] < -IKPI )
                                                        j2array[0]+=IK2PI;
                                                    j2valid[0] = true;
                                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                                    {
                                                    if( !j2valid[ij2] ) { continue; }
                                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                            if( 1 )
                                            {
                                            {
                                                IKReal j2eval[1];
                                                j2eval[0]=((((-0.103041000000000)*(sj1)*((sj4)*(sj4))))+(((-0.256800000000000)*(cj4)*(sj1)))+(((-0.160000000000000)*(sj1)))+(((-0.103041000000000)*(sj1)*((cj4)*(cj4)))));
                                                if( IKabs(j2eval[0]) < 0.000001  )
                                                {
                                                    {
                                                    IKReal evalcond[4];
                                                        IKReal x319=((5.89805981832114e-17)*(sj4));
                                                        IKReal x320=((x319)+(((-1.00000000000000)*(py))));
                                                        evalcond[0]=((IKfmod(((3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                        evalcond[1]=x320;
                                                        evalcond[2]=((0.253041000000000)+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((0.200000000000000)*(px)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                        evalcond[3]=x320;
                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((0.0400000000000000)+(((-0.400000000000000)*(px)))+(((-0.321000000000000)*(cj4)*(px)))+(((-0.321000000000000)*(j0)*(sj4)))+(((0.321000000000000)*(pz)*(sj4)))+(((0.0321000000000000)*(cj4))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                {
                                                                                    IKReal j2eval[1];
                                                                                    j2eval[0]=((((-0.0321000000000000)*(sj4)))+(((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((0.321000000000000)*(cj4)*(pz)))+(((0.321000000000000)*(px)*(sj4))));
                                                                                    if( IKabs(j2eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            continue;
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j2array[1], cj2array[1], sj2array[1];
                                                                                        bool j2valid[1]={false};
                                                                                        IKReal x321=((((-1.00000000000000)*(pz)))+(j0));
                                                                                        IKReal x322=((0.400000000000000)*(pz));
                                                                                        IKReal x323=((0.321000000000000)*(cj4)*(pz));
                                                                                        IKReal x324=((0.321000000000000)*(px)*(sj4));
                                                                                        IKReal x325=((x324)+(x322)+(x323));
                                                                                        IKReal x326=((0.400000000000000)*(j0));
                                                                                        IKReal x327=((0.321000000000000)*(cj4)*(j0));
                                                                                        IKReal x328=((0.0321000000000000)*(sj4));
                                                                                        IKReal x329=((x326)+(x327)+(x328));
                                                                                        IKReal x330=((((-1.00000000000000)*(x329)))+(x325));
                                                                                        IKReal x331=((IKabs(x330) != 0)?((IKReal)1/(x330)):(IKReal)1.0e30);
                                                                                        j2array[0]=IKatan2(((x331)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x321)*(x321))))))), ((x331)*(((((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x321)*(((0.100000000000000)+(((-1.00000000000000)*(px)))))))))));
                                                                                        sj2array[0]=IKsin(j2array[0]);
                                                                                        cj2array[0]=IKcos(j2array[0]);
                                                                                        if( j2array[0] > IKPI )
                                                                                            j2array[0]-=IK2PI;
                                                                                        else if( j2array[0] < -IKPI )
                                                                                            j2array[0]+=IK2PI;
                                                                                        j2valid[0] = true;
                                                                                        for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                                        {
                                                                                        if( !j2valid[ij2] ) { continue; }
                                                                                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x332=((0.100000000000000)+(((-1.00000000000000)*(px))));
                                                                            IKReal x333=((0.321000000000000)*(pz)*(sj4));
                                                                            IKReal x334=((0.0321000000000000)*(cj4));
                                                                            IKReal x335=((0.0400000000000000)+(x333)+(x334));
                                                                            IKReal x336=((0.400000000000000)*(px));
                                                                            IKReal x337=((0.321000000000000)*(j0)*(sj4));
                                                                            IKReal x338=((0.321000000000000)*(cj4)*(px));
                                                                            IKReal x339=((x338)+(x337)+(x336));
                                                                            IKReal x340=((x335)+(((-1.00000000000000)*(x339))));
                                                                            IKReal x341=((IKabs(x340) != 0)?((IKReal)1/(x340)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x341)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x332)*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x341)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x332)*(x332))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x342=((0.256800000000000)*(cj4));
                                                                IKReal x343=(sj4)*(sj4);
                                                                IKReal x344=((0.103041000000000)*(x343));
                                                                IKReal x345=(cj4)*(cj4);
                                                                IKReal x346=((0.103041000000000)*(x345));
                                                                IKReal x347=((0.160000000000000)+(x342)+(x346)+(x344));
                                                                IKReal x348=((IKabs(x347) != 0)?((IKReal)1/(x347)):(IKReal)1.0e30);
                                                                IKReal x349=((((-1.00000000000000)*(pz)))+(j0));
                                                                IKReal x350=((0.321000000000000)*(cj4));
                                                                IKReal x351=((0.400000000000000)+(x350));
                                                                IKReal x352=((0.100000000000000)+(((-1.00000000000000)*(px))));
                                                                j2array[0]=IKatan2(((-1.00000000000000)*(x348)*(((((-1.00000000000000)*(x349)*(x351)))+(((0.321000000000000)*(sj4)*(x352)))))), ((-1.00000000000000)*(x348)*(((((x351)*(x352)))+(((0.321000000000000)*(sj4)*(x349)))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                        IKReal x353=((5.89805981832114e-17)*(sj4));
                                                        IKReal x354=((x353)+(py));
                                                        evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j1)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                        evalcond[1]=x354;
                                                        evalcond[2]=((0.253041000000000)+(((-0.200000000000000)*(px)))+(((0.256800000000000)*(cj4)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                        evalcond[3]=((-1.00000000000000)*(x354));
                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  && IKabs(evalcond[3]) < 0.000010  )
                                                        {
                                                        {
                                                            IKReal j2eval[1];
                                                            j2eval[0]=((0.160000000000000)+(((0.256800000000000)*(cj4)))+(((0.103041000000000)*((sj4)*(sj4))))+(((0.103041000000000)*((cj4)*(cj4)))));
                                                            if( IKabs(j2eval[0]) < 0.000001  )
                                                            {
                                                                {
                                                                    if( 1 )
                                                                    {
                                                                    {
                                                                        IKReal j2eval[1];
                                                                        j2eval[0]=((0.0400000000000000)+(((-0.321000000000000)*(j0)*(sj4)))+(((0.400000000000000)*(px)))+(((0.321000000000000)*(pz)*(sj4)))+(((0.321000000000000)*(cj4)*(px)))+(((0.0321000000000000)*(cj4))));
                                                                        if( IKabs(j2eval[0]) < 0.000001  )
                                                                        {
                                                                            {
                                                                                if( 1 )
                                                                                {
                                                                                {
                                                                                    IKReal j2eval[1];
                                                                                    j2eval[0]=((((-0.0321000000000000)*(sj4)))+(((-0.400000000000000)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.321000000000000)*(cj4)*(j0)))+(((-0.321000000000000)*(px)*(sj4)))+(((0.321000000000000)*(cj4)*(pz))));
                                                                                    if( IKabs(j2eval[0]) < 0.000001  )
                                                                                    {
                                                                                        {
                                                                                            if( 1 )
                                                                                            {
                                                                                            continue;
                                                                                        
                                                                                        } else
                                                                                        {
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        {
                                                                                        IKReal j2array[1], cj2array[1], sj2array[1];
                                                                                        bool j2valid[1]={false};
                                                                                        IKReal x355=((((-1.00000000000000)*(pz)))+(j0));
                                                                                        IKReal x356=((0.400000000000000)*(pz));
                                                                                        IKReal x357=((0.321000000000000)*(cj4)*(pz));
                                                                                        IKReal x358=((x357)+(x356));
                                                                                        IKReal x359=((0.400000000000000)*(j0));
                                                                                        IKReal x360=((0.321000000000000)*(cj4)*(j0));
                                                                                        IKReal x361=((0.0321000000000000)*(sj4));
                                                                                        IKReal x362=((0.321000000000000)*(px)*(sj4));
                                                                                        IKReal x363=((x359)+(x360)+(x361)+(x362));
                                                                                        IKReal x364=((((-1.00000000000000)*(x363)))+(x358));
                                                                                        IKReal x365=((IKabs(x364) != 0)?((IKReal)1/(x364)):(IKReal)1.0e30);
                                                                                        j2array[0]=IKatan2(((x365)*(((((-1.00000000000000)*((x355)*(x355))))+(((0.103041000000000)*((sj4)*(sj4))))))), ((x365)*(((((0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x355)*(((0.100000000000000)+(px)))))))));
                                                                                        sj2array[0]=IKsin(j2array[0]);
                                                                                        cj2array[0]=IKcos(j2array[0]);
                                                                                        if( j2array[0] > IKPI )
                                                                                            j2array[0]-=IK2PI;
                                                                                        else if( j2array[0] < -IKPI )
                                                                                            j2array[0]+=IK2PI;
                                                                                        j2valid[0] = true;
                                                                                        for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                                        {
                                                                                        if( !j2valid[ij2] ) { continue; }
                                                                                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            } else
                                                                            {
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            {
                                                                            IKReal j2array[1], cj2array[1], sj2array[1];
                                                                            bool j2valid[1]={false};
                                                                            IKReal x366=((0.100000000000000)+(px));
                                                                            IKReal x367=((0.400000000000000)*(px));
                                                                            IKReal x368=((0.321000000000000)*(pz)*(sj4));
                                                                            IKReal x369=((0.321000000000000)*(cj4)*(px));
                                                                            IKReal x370=((0.0321000000000000)*(cj4));
                                                                            IKReal x371=((0.0400000000000000)+(x370)+(x368)+(x369)+(x367));
                                                                            IKReal x372=((0.321000000000000)*(j0)*(sj4));
                                                                            IKReal x373=((((-1.00000000000000)*(x372)))+(x371));
                                                                            IKReal x374=((IKabs(x373) != 0)?((IKReal)1/(x373)):(IKReal)1.0e30);
                                                                            j2array[0]=IKatan2(((x374)*(((((-0.321000000000000)*(sj4)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x366)*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x374)*(((((0.103041000000000)*((sj4)*(sj4))))+(((-1.00000000000000)*((x366)*(x366))))))));
                                                                            sj2array[0]=IKsin(j2array[0]);
                                                                            cj2array[0]=IKcos(j2array[0]);
                                                                            if( j2array[0] > IKPI )
                                                                                j2array[0]-=IK2PI;
                                                                            else if( j2array[0] < -IKPI )
                                                                                j2array[0]+=IK2PI;
                                                                            j2valid[0] = true;
                                                                            for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                            {
                                                                            if( !j2valid[ij2] ) { continue; }
                                                                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                } else
                                                                {
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                {
                                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                                bool j2valid[1]={false};
                                                                IKReal x375=((0.256800000000000)*(cj4));
                                                                IKReal x376=(sj4)*(sj4);
                                                                IKReal x377=((0.103041000000000)*(x376));
                                                                IKReal x378=(cj4)*(cj4);
                                                                IKReal x379=((0.103041000000000)*(x378));
                                                                IKReal x380=((0.160000000000000)+(x379)+(x377)+(x375));
                                                                IKReal x381=((IKabs(x380) != 0)?((IKReal)1/(x380)):(IKReal)1.0e30);
                                                                IKReal x382=((0.100000000000000)+(px));
                                                                IKReal x383=((0.321000000000000)*(cj4));
                                                                IKReal x384=((0.400000000000000)+(x383));
                                                                IKReal x385=((((-1.00000000000000)*(pz)))+(j0));
                                                                j2array[0]=IKatan2(((-1.00000000000000)*(x381)*(((((0.321000000000000)*(sj4)*(x382)))+(((-1.00000000000000)*(x384)*(x385)))))), ((-1.00000000000000)*(x381)*(((((0.321000000000000)*(sj4)*(x385)))+(((x382)*(x384)))))));
                                                                sj2array[0]=IKsin(j2array[0]);
                                                                cj2array[0]=IKcos(j2array[0]);
                                                                if( j2array[0] > IKPI )
                                                                    j2array[0]-=IK2PI;
                                                                else if( j2array[0] < -IKPI )
                                                                    j2array[0]+=IK2PI;
                                                                j2valid[0] = true;
                                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                                {
                                                                if( !j2valid[ij2] ) { continue; }
                                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            }
                                                        
                                                        }
                                                    
                                                    } else
                                                    {
                                                        if( 1 )
                                                        {
                                                        continue;
                                                    
                                                    } else
                                                    {
                                                    }
                                                    }
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    {
                                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                                    bool j2valid[1]={false};
                                                    IKReal x386=((5.89805981832114e-17)*(cj1)*(sj4));
                                                    IKReal x387=((0.100000000000000)*(sj1));
                                                    IKReal x388=((x386)+(x387));
                                                    IKReal x389=((x388)+(((-1.00000000000000)*(py))));
                                                    IKReal x390=(cj4)*(cj4);
                                                    IKReal x391=((0.103041000000000)*(sj1)*(x390));
                                                    IKReal x392=(sj4)*(sj4);
                                                    IKReal x393=((0.103041000000000)*(sj1)*(x392));
                                                    IKReal x394=((0.256800000000000)*(cj4)*(sj1));
                                                    IKReal x395=((0.160000000000000)*(sj1));
                                                    IKReal x396=((x395)+(x394)+(x391)+(x393));
                                                    IKReal x397=((-1.00000000000000)*(x396));
                                                    IKReal x398=((IKabs(x397) != 0)?((IKReal)1/(x397)):(IKReal)1.0e30);
                                                    j2array[0]=IKatan2(((x398)*(((((0.321000000000000)*(sj4)*(x389)))+(((((pz)+(((-1.00000000000000)*(j0)))))*(((((0.400000000000000)*(sj1)))+(((0.321000000000000)*(cj4)*(sj1)))))))))), ((x398)*(((((x389)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((0.321000000000000)*(sj1)*(sj4)*(((((-1.00000000000000)*(pz)))+(j0)))))))));
                                                    sj2array[0]=IKsin(j2array[0]);
                                                    cj2array[0]=IKcos(j2array[0]);
                                                    if( j2array[0] > IKPI )
                                                        j2array[0]-=IK2PI;
                                                    else if( j2array[0] < -IKPI )
                                                        j2array[0]+=IK2PI;
                                                    j2valid[0] = true;
                                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                                    {
                                                    if( !j2valid[ij2] ) { continue; }
                                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                }
                                            
                                            }
                                        
                                        } else
                                        {
                                        }
                                        }
                                        }
                                        }
                                    
                                    } else
                                    {
                                        {
                                        IKReal j2array[1], cj2array[1], sj2array[1];
                                        bool j2valid[1]={false};
                                        IKReal x399=((0.100000000000000)*(cj1));
                                        IKReal x400=((5.89805981832114e-17)*(sj1)*(sj4));
                                        IKReal x401=((x400)+(px));
                                        IKReal x402=((x399)+(((-1.00000000000000)*(x401))));
                                        IKReal x403=((0.256800000000000)*(cj1)*(cj4));
                                        IKReal x404=(sj4)*(sj4);
                                        IKReal x405=((0.103041000000000)*(cj1)*(x404));
                                        IKReal x406=((0.160000000000000)*(cj1));
                                        IKReal x407=(cj4)*(cj4);
                                        IKReal x408=((0.103041000000000)*(cj1)*(x407));
                                        IKReal x409=((x403)+(x405)+(x406)+(x408));
                                        IKReal x410=((-1.00000000000000)*(x409));
                                        IKReal x411=((IKabs(x410) != 0)?((IKReal)1/(x410)):(IKReal)1.0e30);
                                        j2array[0]=IKatan2(((x411)*(((((((pz)+(((-1.00000000000000)*(j0)))))*(((((0.400000000000000)*(cj1)))+(((0.321000000000000)*(cj1)*(cj4)))))))+(((0.321000000000000)*(sj4)*(x402)))))), ((x411)*(((((x402)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((0.321000000000000)*(cj1)*(sj4)*(((((-1.00000000000000)*(pz)))+(j0)))))))));
                                        sj2array[0]=IKsin(j2array[0]);
                                        cj2array[0]=IKcos(j2array[0]);
                                        if( j2array[0] > IKPI )
                                            j2array[0]-=IK2PI;
                                        else if( j2array[0] < -IKPI )
                                            j2array[0]+=IK2PI;
                                        j2valid[0] = true;
                                        for(int ij2 = 0; ij2 < 1; ++ij2)
                                        {
                                        if( !j2valid[ij2] ) { continue; }
                                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                        
                                        rotationfunction0(vsolutions);
                                        }
                                        }
                                    
                                    }
                                
                                }
                            
                            } else
                            {
                            }
                            }
                        
                        } else
                        {
                            {
                            IKReal j2array[1], cj2array[1], sj2array[1];
                            bool j2valid[1]={false};
                            IKReal x412=((0.256800000000000)*(cj4));
                            IKReal x413=(sj4)*(sj4);
                            IKReal x414=((0.103041000000000)*(x413));
                            IKReal x415=(cj4)*(cj4);
                            IKReal x416=((0.103041000000000)*(x415));
                            IKReal x417=((0.160000000000000)+(x412)+(x416)+(x414));
                            IKReal x418=((IKabs(x417) != 0)?((IKReal)1/(x417)):(IKReal)1.0e30);
                            IKReal x419=((0.321000000000000)*(cj4));
                            IKReal x420=((0.400000000000000)+(x419));
                            IKReal x421=((cj1)*(px));
                            IKReal x422=((py)*(sj1));
                            IKReal x423=((x422)+(x421));
                            IKReal x424=((0.100000000000000)+(((-1.00000000000000)*(x423))));
                            IKReal x425=((((-1.00000000000000)*(pz)))+(j0));
                            j2array[0]=IKatan2(((-1.00000000000000)*(x418)*(((((0.321000000000000)*(sj4)*(x424)))+(((-1.00000000000000)*(x420)*(x425)))))), ((-1.00000000000000)*(x418)*(((((0.321000000000000)*(sj4)*(x425)))+(((x420)*(x424)))))));
                            sj2array[0]=IKsin(j2array[0]);
                            cj2array[0]=IKcos(j2array[0]);
                            if( j2array[0] > IKPI )
                                j2array[0]-=IK2PI;
                            else if( j2array[0] < -IKPI )
                                j2array[0]+=IK2PI;
                            j2valid[0] = true;
                            for(int ij2 = 0; ij2 < 1; ++ij2)
                            {
                            if( !j2valid[ij2] ) { continue; }
                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                            
                            rotationfunction0(vsolutions);
                            }
                            }
                        
                        }
                    
                    }
                    }
                    }
                
                } else
                {
                    if( 1 )
                    {
                    {
                        IKReal j2eval[1];
                        IKReal x426=x0;
                        IKReal x427=(sj1)*(sj1);
                        IKReal x428=(cj1)*(cj1);
                        IKReal x429=x1;
                        IKReal x430=x5;
                        IKReal x431=x4;
                        j2eval[0]=((((0.159861696250397)*(cj1)*(px)*(sj3)))+(((0.0665117071994815)*(cj3)*(j0)*(pz)))+(((0.00665117071994815)*(cj1)*(cj3)*(px)))+(((1.59861696250397)*(j0)*(pz)*(sj3)))+(((-0.0665117071994815)*(cj1)*(cj3)*(px)*(py)*(sj1)))+(((-0.0332558535997407)*(cj3)*(x431)))+(((-0.0332558535997407)*(cj3)*(x430)))+(((-0.799308481251984)*(sj3)*(x428)*(x429)))+(((-0.00799308481251984)*(sj3)))+(((0.159861696250397)*(py)*(sj1)*(sj3)))+(((-0.0332558535997407)*(cj3)*(x428)*(x429)))+(((-0.000332558535997407)*(cj3)))+(((-0.799308481251984)*(sj3)*(x431)))+(((-0.799308481251984)*(sj3)*(x430)))+(((-1.59861696250397)*(cj1)*(px)*(py)*(sj1)*(sj3)))+(((-0.0332558535997407)*(cj3)*(x426)*(x427)))+(((0.00665117071994815)*(cj3)*(py)*(sj1)))+(((-0.799308481251984)*(sj3)*(x426)*(x427))));
                        if( IKabs(j2eval[0]) < 0.000001  )
                        {
                            {
                                if( 1 )
                                {
                                continue;
                            
                            } else
                            {
                            }
                            }
                        
                        } else
                        {
                            {
                            IKReal j2array[1], cj2array[1], sj2array[1];
                            bool j2valid[1]={false};
                            IKReal x432=x0;
                            IKReal x433=x5;
                            IKReal x434=(sj1)*(sj1);
                            IKReal x435=x1;
                            IKReal x436=(cj1)*(cj1);
                            IKReal x437=x4;
                            IKReal x438=((0.0669590000000000)+(x432)+(x433)+(x435)+(x437));
                            IKReal x439=((2.00000000000000)*(j0)*(pz));
                            IKReal x440=((0.200000000000000)*(cj1)*(px));
                            IKReal x441=((0.200000000000000)*(py)*(sj1));
                            IKReal x442=((x439)+(x441)+(x440));
                            IKReal x443=((((-1.00000000000000)*(x442)))+(x438));
                            IKReal x444=((0.0415698169996759)*(px)*(sj1)*(sj3));
                            IKReal x445=((0.999135601564979)*(cj1)*(cj3)*(py));
                            IKReal x446=((x445)+(x444));
                            IKReal x447=((0.0415698169996759)*(cj1)*(py)*(sj3));
                            IKReal x448=((0.999135601564979)*(cj3)*(px)*(sj1));
                            IKReal x449=((x448)+(x447));
                            IKReal x450=((((-1.00000000000000)*(x449)))+(x446));
                            IKReal x451=((0.159861696250397)*(cj1)*(px)*(sj3));
                            IKReal x452=((0.0665117071994815)*(cj3)*(j0)*(pz));
                            IKReal x453=((0.00665117071994815)*(cj1)*(cj3)*(px));
                            IKReal x454=((1.59861696250397)*(j0)*(pz)*(sj3));
                            IKReal x455=((0.159861696250397)*(py)*(sj1)*(sj3));
                            IKReal x456=((0.00665117071994815)*(cj3)*(py)*(sj1));
                            IKReal x457=((x456)+(x454)+(x455)+(x452)+(x453)+(x451));
                            IKReal x458=((0.0332558535997407)*(cj3)*(x432)*(x434));
                            IKReal x459=((0.799308481251984)*(sj3)*(x433));
                            IKReal x460=((0.799308481251984)*(sj3)*(x432)*(x434));
                            IKReal x461=((0.799308481251984)*(sj3)*(x435)*(x436));
                            IKReal x462=((0.0332558535997407)*(cj3)*(x435)*(x436));
                            IKReal x463=((0.0332558535997407)*(cj3)*(x433));
                            IKReal x464=((0.000332558535997407)*(cj3));
                            IKReal x465=((0.0332558535997407)*(cj3)*(x437));
                            IKReal x466=((0.0665117071994815)*(cj1)*(cj3)*(px)*(py)*(sj1));
                            IKReal x467=((1.59861696250397)*(cj1)*(px)*(py)*(sj1)*(sj3));
                            IKReal x468=((0.00799308481251984)*(sj3));
                            IKReal x469=((0.799308481251984)*(sj3)*(x437));
                            IKReal x470=((x458)+(x459)+(x469)+(x468)+(x463)+(x462)+(x461)+(x460)+(x467)+(x466)+(x465)+(x464));
                            IKReal x471=((((-1.00000000000000)*(x470)))+(x457));
                            IKReal x472=((IKabs(x471) != 0)?((IKReal)1/(x471)):(IKReal)1.0e30);
                            j2array[0]=IKatan2(((x472)*(((((-1.00000000000000)*(x450)*(((0.0800000000000000)+(((-0.800000000000000)*(py)*(sj1)))+(((-0.800000000000000)*(cj1)*(px)))))))+(((x443)*(((((0.0415698169996759)*(cj3)*(pz)))+(((-0.999135601564979)*(j0)*(sj3)))+(((-0.0415698169996759)*(cj3)*(j0)))+(((0.999135601564979)*(pz)*(sj3)))))))))), ((x472)*(((((x443)*(((((-0.999135601564979)*(py)*(sj1)*(sj3)))+(((-0.0415698169996759)*(cj3)*(py)*(sj1)))+(((0.0999135601564980)*(sj3)))+(((-0.999135601564979)*(cj1)*(px)*(sj3)))+(((0.00415698169996759)*(cj3)))+(((-0.0415698169996759)*(cj1)*(cj3)*(px)))))))+(((x450)*(((((-0.800000000000000)*(j0)))+(((0.800000000000000)*(pz)))))))))));
                            sj2array[0]=IKsin(j2array[0]);
                            cj2array[0]=IKcos(j2array[0]);
                            if( j2array[0] > IKPI )
                                j2array[0]-=IK2PI;
                            else if( j2array[0] < -IKPI )
                                j2array[0]+=IK2PI;
                            j2valid[0] = true;
                            for(int ij2 = 0; ij2 < 1; ++ij2)
                            {
                            if( !j2valid[ij2] ) { continue; }
                            j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                            
                            {
                                IKReal j4eval[1];
                                j4eval[0]=((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))));
                                if( IKabs(j4eval[0]) < 0.000001  )
                                {
                                    {
                                    IKReal evalcond[3];
                                        evalcond[0]=((IKfmod(((0.0415817987708408)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                        evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(j0)*(sj2)))+(((-0.0800000000000000)*(cj2)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-0.800000000000000)*(pz)*(sj2)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((0.800000000000000)*(cj2)*(py)*(sj1)))+(((0.800000000000000)*(cj1)*(cj2)*(px))));
                                        evalcond[2]=((((-6.93889390390723e-18)*(cj2)*(pz)))+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)*(sj2)))+(((8.67361737988404e-19)*(sj2)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(cj2)*(j0)))+(((-6.93889390390723e-18)*(cj1)*(px)*(sj2))));
                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                        {
                                        {
                                            IKReal j4eval[1];
                                            j4eval[0]=sj2;
                                            if( IKabs(j4eval[0]) < 0.000001  )
                                            {
                                                {
                                                IKReal evalcond[3];
                                                    evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                    evalcond[1]=((-0.146959000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((cj1)*(px)))+(((2.00000000000000)*(j0)*(pz)))+(((py)*(sj1))));
                                                    evalcond[2]=((((-1.00000000000000)*(px)*(sj1)))+(((6.93889390390723e-18)*(j0)))+(((cj1)*(py)))+(((-6.93889390390723e-18)*(pz))));
                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                    {
                                                    {
                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                    bool j4valid[1]={false};
                                                    j4array[0]=IKatan2(((((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))), ((-1.55763239875389)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))));
                                                    sj4array[0]=IKsin(j4array[0]);
                                                    cj4array[0]=IKcos(j4array[0]);
                                                    if( j4array[0] > IKPI )
                                                        j4array[0]-=IK2PI;
                                                    else if( j4array[0] < -IKPI )
                                                        j4array[0]+=IK2PI;
                                                    j4valid[0] = true;
                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                    {
                                                    if( !j4valid[ij4] ) { continue; }
                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                    evalcond[1]=((0.0130410000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.600000000000000)*(cj1)*(px)))+(((-0.600000000000000)*(py)*(sj1))));
                                                    evalcond[2]=((((6.93889390390723e-18)*(pz)))+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(j0)))+(((cj1)*(py))));
                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                    {
                                                    {
                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                    bool j4valid[1]={false};
                                                    j4array[0]=IKatan2(((((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))), ((-0.934579439252337)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))));
                                                    sj4array[0]=IKsin(j4array[0]);
                                                    cj4array[0]=IKcos(j4array[0]);
                                                    if( j4array[0] > IKPI )
                                                        j4array[0]-=IK2PI;
                                                    else if( j4array[0] < -IKPI )
                                                        j4array[0]+=IK2PI;
                                                    j4valid[0] = true;
                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                    {
                                                    if( !j4valid[ij4] ) { continue; }
                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    if( 1 )
                                                    {
                                                    {
                                                        IKReal j4eval[1];
                                                        j4eval[0]=cj2;
                                                        if( IKabs(j4eval[0]) < 0.000001  )
                                                        {
                                                            {
                                                            IKReal evalcond[3];
                                                                evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                evalcond[2]=((8.67361737988404e-19)+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)))+(((-6.93889390390723e-18)*(cj1)*(px)))+(((cj1)*(py))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                evalcond[2]=((-8.67361737988404e-19)+(((6.93889390390723e-18)*(cj1)*(px)))+(((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(py)*(sj1))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                if( 1 )
                                                                {
                                                                {
                                                                    IKReal j4eval[1];
                                                                    j4eval[0]=cj2;
                                                                    if( IKabs(j4eval[0]) < 0.000001  )
                                                                    {
                                                                        {
                                                                        IKReal evalcond[3];
                                                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                            evalcond[2]=((8.67361737988404e-19)+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)))+(((-6.93889390390723e-18)*(cj1)*(px)))+(((cj1)*(py))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                            evalcond[2]=((-8.67361737988404e-19)+(((6.93889390390723e-18)*(cj1)*(px)))+(((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(py)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            if( 1 )
                                                                            {
                                                                            continue;
                                                                        
                                                                        } else
                                                                        {
                                                                        }
                                                                        }
                                                                        }
                                                                        }
                                                                    
                                                                    } else
                                                                    {
                                                                        {
                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                        bool j4valid[1]={false};
                                                                        j4array[0]=IKatan2(((((-3.11526479750779)*(cj2)*(pz)))+(((-3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(j0)))+(((-3.11526479750779)*(py)*(sj1)*(sj2)))+(((0.311526479750779)*(sj2)))), ((-9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((0.0321000000000000)+(((-0.321000000000000)*(cj1)*(px)))+(((0.128400000000000)*(cj2)))+(((-0.321000000000000)*(sj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((-0.321000000000000)*(py)*(sj1)))))));
                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                        if( j4array[0] > IKPI )
                                                                            j4array[0]-=IK2PI;
                                                                        else if( j4array[0] < -IKPI )
                                                                            j4array[0]+=IK2PI;
                                                                        j4valid[0] = true;
                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                        {
                                                                        if( !j4valid[ij4] ) { continue; }
                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                        
                                                                        rotationfunction0(vsolutions);
                                                                        }
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                }
                                                            
                                                            } else
                                                            {
                                                            }
                                                            }
                                                            }
                                                            }
                                                        
                                                        } else
                                                        {
                                                            {
                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                            bool j4valid[1]={false};
                                                            j4array[0]=IKatan2(((-9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((((-0.321000000000000)*(j0)))+(((-0.321000000000000)*(sj2)*(((0.400000000000000)+(((-1.00000000000000)*(cj2)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(cj2)*(px)))+(((pz)*(sj2)))+(((-1.00000000000000)*(j0)*(sj2)))+(((0.100000000000000)*(cj2)))))))+(((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))))), ((-1.24610591900312)+(((3.11526479750779)*(cj2)*(py)*(sj1)))+(((-3.11526479750779)*(pz)*(sj2)))+(((3.11526479750779)*(j0)*(sj2)))+(((3.11526479750779)*(cj1)*(cj2)*(px)))+(((-0.311526479750779)*(cj2)))));
                                                            sj4array[0]=IKsin(j4array[0]);
                                                            cj4array[0]=IKcos(j4array[0]);
                                                            if( j4array[0] > IKPI )
                                                                j4array[0]-=IK2PI;
                                                            else if( j4array[0] < -IKPI )
                                                                j4array[0]+=IK2PI;
                                                            j4valid[0] = true;
                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                            {
                                                            if( !j4valid[ij4] ) { continue; }
                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                            
                                                            rotationfunction0(vsolutions);
                                                            }
                                                            }
                                                        
                                                        }
                                                    
                                                    }
                                                
                                                } else
                                                {
                                                }
                                                }
                                                }
                                                }
                                            
                                            } else
                                            {
                                                {
                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                bool j4valid[1]={false};
                                                j4array[0]=IKatan2(((((-3.11526479750779)*(cj2)*(pz)))+(((-3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(j0)))+(((-3.11526479750779)*(py)*(sj1)*(sj2)))+(((0.311526479750779)*(sj2)))), ((9.70487475859124)*(((IKabs(sj2) != 0)?((IKReal)1/(sj2)):(IKReal)1.0e30))*(((((-0.321000000000000)*(cj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((0.321000000000000)*(j0)))+(((-0.321000000000000)*(pz)))+(((-0.128400000000000)*(sj2)))))));
                                                sj4array[0]=IKsin(j4array[0]);
                                                cj4array[0]=IKcos(j4array[0]);
                                                if( j4array[0] > IKPI )
                                                    j4array[0]-=IK2PI;
                                                else if( j4array[0] < -IKPI )
                                                    j4array[0]+=IK2PI;
                                                j4valid[0] = true;
                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                {
                                                if( !j4valid[ij4] ) { continue; }
                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                
                                                rotationfunction0(vsolutions);
                                                }
                                                }
                                            
                                            }
                                        
                                        }
                                    
                                    } else
                                    {
                                        evalcond[0]=((IKfmod(((-3.10001085481895)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                        evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(j0)*(sj2)))+(((-0.0800000000000000)*(cj2)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-0.800000000000000)*(pz)*(sj2)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((0.800000000000000)*(cj2)*(py)*(sj1)))+(((0.800000000000000)*(cj1)*(cj2)*(px))));
                                        evalcond[2]=((((1.87350135405495e-16)*(cj2)*(pz)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)*(sj2)))+(((-1.90819582357449e-17)*(sj2)))+(((px)*(sj1)))+(((-1.87350135405495e-16)*(cj2)*(j0)))+(((1.87350135405495e-16)*(py)*(sj1)*(sj2))));
                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                        {
                                        {
                                            IKReal j4eval[1];
                                            j4eval[0]=sj2;
                                            if( IKabs(j4eval[0]) < 0.000001  )
                                            {
                                                {
                                                IKReal evalcond[3];
                                                    evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                    evalcond[1]=((-0.146959000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((cj1)*(px)))+(((2.00000000000000)*(j0)*(pz)))+(((py)*(sj1))));
                                                    evalcond[2]=((((-1.87350135405495e-16)*(j0)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(pz)))+(((px)*(sj1))));
                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                    {
                                                    {
                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                    bool j4valid[1]={false};
                                                    j4array[0]=IKatan2(((((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))), ((-1.55763239875389)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))));
                                                    sj4array[0]=IKsin(j4array[0]);
                                                    cj4array[0]=IKcos(j4array[0]);
                                                    if( j4array[0] > IKPI )
                                                        j4array[0]-=IK2PI;
                                                    else if( j4array[0] < -IKPI )
                                                        j4array[0]+=IK2PI;
                                                    j4valid[0] = true;
                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                    {
                                                    if( !j4valid[ij4] ) { continue; }
                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                    evalcond[1]=((0.0130410000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.600000000000000)*(cj1)*(px)))+(((-0.600000000000000)*(py)*(sj1))));
                                                    evalcond[2]=((((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(j0)))+(((-1.87350135405495e-16)*(pz)))+(((px)*(sj1))));
                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                    {
                                                    {
                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                    bool j4valid[1]={false};
                                                    j4array[0]=IKatan2(((((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))), ((-0.934579439252337)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))));
                                                    sj4array[0]=IKsin(j4array[0]);
                                                    cj4array[0]=IKcos(j4array[0]);
                                                    if( j4array[0] > IKPI )
                                                        j4array[0]-=IK2PI;
                                                    else if( j4array[0] < -IKPI )
                                                        j4array[0]+=IK2PI;
                                                    j4valid[0] = true;
                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                    {
                                                    if( !j4valid[ij4] ) { continue; }
                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                    
                                                    rotationfunction0(vsolutions);
                                                    }
                                                    }
                                                
                                                } else
                                                {
                                                    if( 1 )
                                                    {
                                                    {
                                                        IKReal j4eval[1];
                                                        j4eval[0]=cj2;
                                                        if( IKabs(j4eval[0]) < 0.000001  )
                                                        {
                                                            {
                                                            IKReal evalcond[3];
                                                                evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                evalcond[2]=((-1.90819582357449e-17)+(((1.87350135405495e-16)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)))+(((px)*(sj1))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                evalcond[2]=((1.90819582357449e-17)+(((-1.00000000000000)*(cj1)*(py)))+(((-1.87350135405495e-16)*(cj1)*(px)))+(((-1.87350135405495e-16)*(py)*(sj1)))+(((px)*(sj1))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                if( 1 )
                                                                {
                                                                {
                                                                    IKReal j4eval[1];
                                                                    j4eval[0]=cj2;
                                                                    if( IKabs(j4eval[0]) < 0.000001  )
                                                                    {
                                                                        {
                                                                        IKReal evalcond[3];
                                                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                            evalcond[2]=((-1.90819582357449e-17)+(((1.87350135405495e-16)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)))+(((px)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                            evalcond[2]=((1.90819582357449e-17)+(((-1.00000000000000)*(cj1)*(py)))+(((-1.87350135405495e-16)*(cj1)*(px)))+(((-1.87350135405495e-16)*(py)*(sj1)))+(((px)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            if( 1 )
                                                                            {
                                                                            continue;
                                                                        
                                                                        } else
                                                                        {
                                                                        }
                                                                        }
                                                                        }
                                                                        }
                                                                    
                                                                    } else
                                                                    {
                                                                        {
                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                        bool j4valid[1]={false};
                                                                        j4array[0]=IKatan2(((((3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(pz)))+(((3.11526479750779)*(py)*(sj1)*(sj2)))+(((-0.311526479750779)*(sj2)))+(((-3.11526479750779)*(cj2)*(j0)))), ((-9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((0.0321000000000000)+(((-0.321000000000000)*(cj1)*(px)))+(((0.128400000000000)*(cj2)))+(((-0.321000000000000)*(sj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((-0.321000000000000)*(py)*(sj1)))))));
                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                        if( j4array[0] > IKPI )
                                                                            j4array[0]-=IK2PI;
                                                                        else if( j4array[0] < -IKPI )
                                                                            j4array[0]+=IK2PI;
                                                                        j4valid[0] = true;
                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                        {
                                                                        if( !j4valid[ij4] ) { continue; }
                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                        
                                                                        rotationfunction0(vsolutions);
                                                                        }
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                }
                                                            
                                                            } else
                                                            {
                                                            }
                                                            }
                                                            }
                                                            }
                                                        
                                                        } else
                                                        {
                                                            {
                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                            bool j4valid[1]={false};
                                                            j4array[0]=IKatan2(((9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((((-0.321000000000000)*(j0)))+(((-0.321000000000000)*(sj2)*(((0.400000000000000)+(((-1.00000000000000)*(cj2)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(cj2)*(px)))+(((pz)*(sj2)))+(((-1.00000000000000)*(j0)*(sj2)))+(((0.100000000000000)*(cj2)))))))+(((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))))), ((-1.24610591900312)+(((3.11526479750779)*(cj2)*(py)*(sj1)))+(((-3.11526479750779)*(pz)*(sj2)))+(((3.11526479750779)*(j0)*(sj2)))+(((3.11526479750779)*(cj1)*(cj2)*(px)))+(((-0.311526479750779)*(cj2)))));
                                                            sj4array[0]=IKsin(j4array[0]);
                                                            cj4array[0]=IKcos(j4array[0]);
                                                            if( j4array[0] > IKPI )
                                                                j4array[0]-=IK2PI;
                                                            else if( j4array[0] < -IKPI )
                                                                j4array[0]+=IK2PI;
                                                            j4valid[0] = true;
                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                            {
                                                            if( !j4valid[ij4] ) { continue; }
                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                            
                                                            rotationfunction0(vsolutions);
                                                            }
                                                            }
                                                        
                                                        }
                                                    
                                                    }
                                                
                                                } else
                                                {
                                                }
                                                }
                                                }
                                                }
                                            
                                            } else
                                            {
                                                {
                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                bool j4valid[1]={false};
                                                j4array[0]=IKatan2(((((3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(pz)))+(((3.11526479750779)*(py)*(sj1)*(sj2)))+(((-0.311526479750779)*(sj2)))+(((-3.11526479750779)*(cj2)*(j0)))), ((-9.70487475859124)*(((IKabs(sj2) != 0)?((IKReal)1/(sj2)):(IKReal)1.0e30))*(((((0.321000000000000)*(cj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((-0.321000000000000)*(j0)))+(((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))))));
                                                sj4array[0]=IKsin(j4array[0]);
                                                cj4array[0]=IKcos(j4array[0]);
                                                if( j4array[0] > IKPI )
                                                    j4array[0]-=IK2PI;
                                                else if( j4array[0] < -IKPI )
                                                    j4array[0]+=IK2PI;
                                                j4valid[0] = true;
                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                {
                                                if( !j4valid[ij4] ) { continue; }
                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                
                                                rotationfunction0(vsolutions);
                                                }
                                                }
                                            
                                            }
                                        
                                        }
                                    
                                    } else
                                    {
                                        if( 1 )
                                        {
                                        {
                                            IKReal j4eval[1];
                                            j4eval[0]=((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))));
                                            if( IKabs(j4eval[0]) < 0.000001  )
                                            {
                                                {
                                                IKReal evalcond[3];
                                                    evalcond[0]=((IKfmod(((0.0415817987708408)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                    evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(j0)*(sj2)))+(((-0.0800000000000000)*(cj2)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-0.800000000000000)*(pz)*(sj2)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((0.800000000000000)*(cj2)*(py)*(sj1)))+(((0.800000000000000)*(cj1)*(cj2)*(px))));
                                                    evalcond[2]=((((-6.93889390390723e-18)*(cj2)*(pz)))+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)*(sj2)))+(((8.67361737988404e-19)*(sj2)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(cj2)*(j0)))+(((-6.93889390390723e-18)*(cj1)*(px)*(sj2))));
                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                    {
                                                    {
                                                        IKReal j4eval[1];
                                                        j4eval[0]=sj2;
                                                        if( IKabs(j4eval[0]) < 0.000001  )
                                                        {
                                                            {
                                                            IKReal evalcond[3];
                                                                evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                evalcond[1]=((-0.146959000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((cj1)*(px)))+(((2.00000000000000)*(j0)*(pz)))+(((py)*(sj1))));
                                                                evalcond[2]=((((-1.00000000000000)*(px)*(sj1)))+(((6.93889390390723e-18)*(j0)))+(((cj1)*(py)))+(((-6.93889390390723e-18)*(pz))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))), ((-1.55763239875389)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                evalcond[1]=((0.0130410000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.600000000000000)*(cj1)*(px)))+(((-0.600000000000000)*(py)*(sj1))));
                                                                evalcond[2]=((((6.93889390390723e-18)*(pz)))+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(j0)))+(((cj1)*(py))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))), ((-0.934579439252337)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                if( 1 )
                                                                {
                                                                {
                                                                    IKReal j4eval[1];
                                                                    j4eval[0]=cj2;
                                                                    if( IKabs(j4eval[0]) < 0.000001  )
                                                                    {
                                                                        {
                                                                        IKReal evalcond[3];
                                                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                            evalcond[2]=((8.67361737988404e-19)+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)))+(((-6.93889390390723e-18)*(cj1)*(px)))+(((cj1)*(py))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                            evalcond[2]=((-8.67361737988404e-19)+(((6.93889390390723e-18)*(cj1)*(px)))+(((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(py)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            if( 1 )
                                                                            {
                                                                            {
                                                                                IKReal j4eval[1];
                                                                                j4eval[0]=cj2;
                                                                                if( IKabs(j4eval[0]) < 0.000001  )
                                                                                {
                                                                                    {
                                                                                    IKReal evalcond[3];
                                                                                        evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                                        evalcond[2]=((8.67361737988404e-19)+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)))+(((-6.93889390390723e-18)*(cj1)*(px)))+(((cj1)*(py))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                                        evalcond[2]=((-8.67361737988404e-19)+(((6.93889390390723e-18)*(cj1)*(px)))+(((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(py)*(sj1))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        if( 1 )
                                                                                        {
                                                                                        continue;
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                
                                                                                } else
                                                                                {
                                                                                    {
                                                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                    bool j4valid[1]={false};
                                                                                    j4array[0]=IKatan2(((((-3.11526479750779)*(cj2)*(pz)))+(((-3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(j0)))+(((-3.11526479750779)*(py)*(sj1)*(sj2)))+(((0.311526479750779)*(sj2)))), ((-9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((0.0321000000000000)+(((-0.321000000000000)*(cj1)*(px)))+(((0.128400000000000)*(cj2)))+(((-0.321000000000000)*(sj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((-0.321000000000000)*(py)*(sj1)))))));
                                                                                    sj4array[0]=IKsin(j4array[0]);
                                                                                    cj4array[0]=IKcos(j4array[0]);
                                                                                    if( j4array[0] > IKPI )
                                                                                        j4array[0]-=IK2PI;
                                                                                    else if( j4array[0] < -IKPI )
                                                                                        j4array[0]+=IK2PI;
                                                                                    j4valid[0] = true;
                                                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                    {
                                                                                    if( !j4valid[ij4] ) { continue; }
                                                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                    
                                                                                    rotationfunction0(vsolutions);
                                                                                    }
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                        }
                                                                        }
                                                                        }
                                                                        }
                                                                    
                                                                    } else
                                                                    {
                                                                        {
                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                        bool j4valid[1]={false};
                                                                        j4array[0]=IKatan2(((-9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((((-0.321000000000000)*(j0)))+(((-0.321000000000000)*(sj2)*(((0.400000000000000)+(((-1.00000000000000)*(cj2)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(cj2)*(px)))+(((pz)*(sj2)))+(((-1.00000000000000)*(j0)*(sj2)))+(((0.100000000000000)*(cj2)))))))+(((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))))), ((-1.24610591900312)+(((3.11526479750779)*(cj2)*(py)*(sj1)))+(((-3.11526479750779)*(pz)*(sj2)))+(((3.11526479750779)*(j0)*(sj2)))+(((3.11526479750779)*(cj1)*(cj2)*(px)))+(((-0.311526479750779)*(cj2)))));
                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                        if( j4array[0] > IKPI )
                                                                            j4array[0]-=IK2PI;
                                                                        else if( j4array[0] < -IKPI )
                                                                            j4array[0]+=IK2PI;
                                                                        j4valid[0] = true;
                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                        {
                                                                        if( !j4valid[ij4] ) { continue; }
                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                        
                                                                        rotationfunction0(vsolutions);
                                                                        }
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                }
                                                            
                                                            } else
                                                            {
                                                            }
                                                            }
                                                            }
                                                            }
                                                        
                                                        } else
                                                        {
                                                            {
                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                            bool j4valid[1]={false};
                                                            j4array[0]=IKatan2(((((-3.11526479750779)*(cj2)*(pz)))+(((-3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(j0)))+(((-3.11526479750779)*(py)*(sj1)*(sj2)))+(((0.311526479750779)*(sj2)))), ((9.70487475859124)*(((IKabs(sj2) != 0)?((IKReal)1/(sj2)):(IKReal)1.0e30))*(((((-0.321000000000000)*(cj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((0.321000000000000)*(j0)))+(((-0.321000000000000)*(pz)))+(((-0.128400000000000)*(sj2)))))));
                                                            sj4array[0]=IKsin(j4array[0]);
                                                            cj4array[0]=IKcos(j4array[0]);
                                                            if( j4array[0] > IKPI )
                                                                j4array[0]-=IK2PI;
                                                            else if( j4array[0] < -IKPI )
                                                                j4array[0]+=IK2PI;
                                                            j4valid[0] = true;
                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                            {
                                                            if( !j4valid[ij4] ) { continue; }
                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                            
                                                            rotationfunction0(vsolutions);
                                                            }
                                                            }
                                                        
                                                        }
                                                    
                                                    }
                                                
                                                } else
                                                {
                                                    evalcond[0]=((IKfmod(((-3.10001085481895)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                    evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(j0)*(sj2)))+(((-0.0800000000000000)*(cj2)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-0.800000000000000)*(pz)*(sj2)))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((0.800000000000000)*(cj2)*(py)*(sj1)))+(((0.800000000000000)*(cj1)*(cj2)*(px))));
                                                    evalcond[2]=((((1.87350135405495e-16)*(cj2)*(pz)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)*(sj2)))+(((-1.90819582357449e-17)*(sj2)))+(((px)*(sj1)))+(((-1.87350135405495e-16)*(cj2)*(j0)))+(((1.87350135405495e-16)*(py)*(sj1)*(sj2))));
                                                    if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                    {
                                                    {
                                                        IKReal j4eval[1];
                                                        j4eval[0]=sj2;
                                                        if( IKabs(j4eval[0]) < 0.000001  )
                                                        {
                                                            {
                                                            IKReal evalcond[3];
                                                                evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                evalcond[1]=((-0.146959000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((cj1)*(px)))+(((2.00000000000000)*(j0)*(pz)))+(((py)*(sj1))));
                                                                evalcond[2]=((((-1.87350135405495e-16)*(j0)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(pz)))+(((px)*(sj1))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))), ((-1.55763239875389)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                evalcond[0]=((IKfmod(((-3.14159265358979)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                evalcond[1]=((0.0130410000000000)+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.600000000000000)*(cj1)*(px)))+(((-0.600000000000000)*(py)*(sj1))));
                                                                evalcond[2]=((((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(j0)))+(((-1.87350135405495e-16)*(pz)))+(((px)*(sj1))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                bool j4valid[1]={false};
                                                                j4array[0]=IKatan2(((((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))), ((-0.934579439252337)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))));
                                                                sj4array[0]=IKsin(j4array[0]);
                                                                cj4array[0]=IKcos(j4array[0]);
                                                                if( j4array[0] > IKPI )
                                                                    j4array[0]-=IK2PI;
                                                                else if( j4array[0] < -IKPI )
                                                                    j4array[0]+=IK2PI;
                                                                j4valid[0] = true;
                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                {
                                                                if( !j4valid[ij4] ) { continue; }
                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                
                                                                rotationfunction0(vsolutions);
                                                                }
                                                                }
                                                            
                                                            } else
                                                            {
                                                                if( 1 )
                                                                {
                                                                {
                                                                    IKReal j4eval[1];
                                                                    j4eval[0]=cj2;
                                                                    if( IKabs(j4eval[0]) < 0.000001  )
                                                                    {
                                                                        {
                                                                        IKReal evalcond[3];
                                                                            evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                            evalcond[2]=((-1.90819582357449e-17)+(((1.87350135405495e-16)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)))+(((px)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                            evalcond[2]=((1.90819582357449e-17)+(((-1.00000000000000)*(cj1)*(py)))+(((-1.87350135405495e-16)*(cj1)*(px)))+(((-1.87350135405495e-16)*(py)*(sj1)))+(((px)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            if( 1 )
                                                                            {
                                                                            {
                                                                                IKReal j4eval[1];
                                                                                j4eval[0]=cj2;
                                                                                if( IKabs(j4eval[0]) < 0.000001  )
                                                                                {
                                                                                    {
                                                                                    IKReal evalcond[3];
                                                                                        evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                                        evalcond[2]=((-1.90819582357449e-17)+(((1.87350135405495e-16)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)))+(((px)*(sj1))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                                        evalcond[2]=((1.90819582357449e-17)+(((-1.00000000000000)*(cj1)*(py)))+(((-1.87350135405495e-16)*(cj1)*(px)))+(((-1.87350135405495e-16)*(py)*(sj1)))+(((px)*(sj1))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        if( 1 )
                                                                                        {
                                                                                        continue;
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                
                                                                                } else
                                                                                {
                                                                                    {
                                                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                    bool j4valid[1]={false};
                                                                                    j4array[0]=IKatan2(((((3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(pz)))+(((3.11526479750779)*(py)*(sj1)*(sj2)))+(((-0.311526479750779)*(sj2)))+(((-3.11526479750779)*(cj2)*(j0)))), ((-9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((0.0321000000000000)+(((-0.321000000000000)*(cj1)*(px)))+(((0.128400000000000)*(cj2)))+(((-0.321000000000000)*(sj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((-0.321000000000000)*(py)*(sj1)))))));
                                                                                    sj4array[0]=IKsin(j4array[0]);
                                                                                    cj4array[0]=IKcos(j4array[0]);
                                                                                    if( j4array[0] > IKPI )
                                                                                        j4array[0]-=IK2PI;
                                                                                    else if( j4array[0] < -IKPI )
                                                                                        j4array[0]+=IK2PI;
                                                                                    j4valid[0] = true;
                                                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                    {
                                                                                    if( !j4valid[ij4] ) { continue; }
                                                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                    
                                                                                    rotationfunction0(vsolutions);
                                                                                    }
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                        }
                                                                        }
                                                                        }
                                                                        }
                                                                    
                                                                    } else
                                                                    {
                                                                        {
                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                        bool j4valid[1]={false};
                                                                        j4array[0]=IKatan2(((9.70487475859124)*(((IKabs(cj2) != 0)?((IKReal)1/(cj2)):(IKReal)1.0e30))*(((((-0.321000000000000)*(j0)))+(((-0.321000000000000)*(sj2)*(((0.400000000000000)+(((-1.00000000000000)*(cj2)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(cj2)*(px)))+(((pz)*(sj2)))+(((-1.00000000000000)*(j0)*(sj2)))+(((0.100000000000000)*(cj2)))))))+(((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))))), ((-1.24610591900312)+(((3.11526479750779)*(cj2)*(py)*(sj1)))+(((-3.11526479750779)*(pz)*(sj2)))+(((3.11526479750779)*(j0)*(sj2)))+(((3.11526479750779)*(cj1)*(cj2)*(px)))+(((-0.311526479750779)*(cj2)))));
                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                        if( j4array[0] > IKPI )
                                                                            j4array[0]-=IK2PI;
                                                                        else if( j4array[0] < -IKPI )
                                                                            j4array[0]+=IK2PI;
                                                                        j4valid[0] = true;
                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                        {
                                                                        if( !j4valid[ij4] ) { continue; }
                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                        
                                                                        rotationfunction0(vsolutions);
                                                                        }
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                }
                                                            
                                                            } else
                                                            {
                                                            }
                                                            }
                                                            }
                                                            }
                                                        
                                                        } else
                                                        {
                                                            {
                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                            bool j4valid[1]={false};
                                                            j4array[0]=IKatan2(((((3.11526479750779)*(cj1)*(px)*(sj2)))+(((3.11526479750779)*(cj2)*(pz)))+(((3.11526479750779)*(py)*(sj1)*(sj2)))+(((-0.311526479750779)*(sj2)))+(((-3.11526479750779)*(cj2)*(j0)))), ((-9.70487475859124)*(((IKabs(sj2) != 0)?((IKReal)1/(sj2)):(IKReal)1.0e30))*(((((0.321000000000000)*(cj2)*(((((-1.00000000000000)*(cj1)*(px)*(sj2)))+(((-1.00000000000000)*(py)*(sj1)*(sj2)))+(((0.100000000000000)*(sj2)))+(((cj2)*(j0)))+(((-1.00000000000000)*(cj2)*(pz)))))))+(((-0.321000000000000)*(j0)))+(((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))))));
                                                            sj4array[0]=IKsin(j4array[0]);
                                                            cj4array[0]=IKcos(j4array[0]);
                                                            if( j4array[0] > IKPI )
                                                                j4array[0]-=IK2PI;
                                                            else if( j4array[0] < -IKPI )
                                                                j4array[0]+=IK2PI;
                                                            j4valid[0] = true;
                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                            {
                                                            if( !j4valid[ij4] ) { continue; }
                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                            
                                                            rotationfunction0(vsolutions);
                                                            }
                                                            }
                                                        
                                                        }
                                                    
                                                    }
                                                
                                                } else
                                                {
                                                    if( 1 )
                                                    {
                                                    {
                                                        IKReal j4eval[1];
                                                        j4eval[0]=((((-0.102951931520857)*(cj2)*(cj3)))+(((0.00428339551346361)*(cj2)*(sj3))));
                                                        if( IKabs(j4eval[0]) < 0.000001  )
                                                        {
                                                            {
                                                            IKReal evalcond[3];
                                                                evalcond[0]=((IKfmod(((-1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                evalcond[2]=((((-0.999135601564979)*(cj3)*(px)*(sj1)))+(((-0.0999135601564980)*(sj3)))+(((0.999135601564979)*(cj1)*(cj3)*(py)))+(((0.0415698169996759)*(px)*(sj1)*(sj3)))+(((0.0415698169996759)*(cj1)*(cj3)*(px)))+(((-0.0415698169996759)*(cj1)*(py)*(sj3)))+(((0.999135601564979)*(py)*(sj1)*(sj3)))+(((0.0415698169996759)*(cj3)*(py)*(sj1)))+(((0.999135601564979)*(cj1)*(px)*(sj3)))+(((-0.00415698169996759)*(cj3))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                    IKReal j4eval[1];
                                                                    j4eval[0]=((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))));
                                                                    if( IKabs(j4eval[0]) < 0.000001  )
                                                                    {
                                                                        {
                                                                        IKReal evalcond[3];
                                                                            evalcond[0]=((IKfmod(((0.0415817987708408)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                            evalcond[2]=((8.67361737988404e-19)+(((-1.00000000000000)*(px)*(sj1)))+(((-6.93889390390723e-18)*(py)*(sj1)))+(((-6.93889390390723e-18)*(cj1)*(px)))+(((cj1)*(py))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            evalcond[0]=((IKfmod(((-3.10001085481895)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                            evalcond[2]=((-1.90819582357449e-17)+(((1.87350135405495e-16)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(py)))+(((1.87350135405495e-16)*(cj1)*(px)))+(((px)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            if( 1 )
                                                                            {
                                                                            {
                                                                                IKReal j4eval[1];
                                                                                j4eval[0]=((((-0.320722528102358)*(cj3)))+(((0.0133439112568960)*(sj3))));
                                                                                if( IKabs(j4eval[0]) < 0.000001  )
                                                                                {
                                                                                    {
                                                                                    IKReal evalcond[3];
                                                                                        evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((-1.52921452802406)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979)))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                                        evalcond[2]=((-0.100000000000000)+(((9.02056207507940e-17)*(px)*(sj1)))+(((cj1)*(px)))+(((-9.02056207507940e-17)*(cj1)*(py)))+(((py)*(sj1))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((((3.11526479750779)*(cj1)*(py)))+(((-3.11526479750779)*(px)*(sj1)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        evalcond[0]=((IKfmod(((-4.67080718161385)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((-0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((0.800000000000000)*(j0)))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz))));
                                                                                        evalcond[2]=((0.100000000000000)+(((-1.00000000000000)*(cj1)*(px)))+(((-1.94289029309402e-16)*(px)*(sj1)))+(((-1.00000000000000)*(py)*(sj1)))+(((1.94289029309402e-16)*(cj1)*(py))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((((3.11526479750779)*(px)*(sj1)))+(((-3.11526479750779)*(cj1)*(py)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        if( 1 )
                                                                                        {
                                                                                        {
                                                                                            IKReal j4eval[1];
                                                                                            j4eval[0]=((((-0.0133439112568960)*(cj3)*(sj1)))+(((-0.320722528102358)*(sj1)*(sj3)))+(((-0.320722528102358)*(cj1)*(cj3)))+(((0.0133439112568960)*(cj1)*(sj3))));
                                                                                            if( IKabs(j4eval[0]) < 0.000001  )
                                                                                            {
                                                                                                {
                                                                                                    if( 1 )
                                                                                                    {
                                                                                                    continue;
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                }
                                                                                                }
                                                                                            
                                                                                            } else
                                                                                            {
                                                                                                {
                                                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                                bool j4valid[1]={false};
                                                                                                j4array[0]=IKatan2(((((IKabs(((((-0.0133439112568960)*(cj3)*(sj1)))+(((-0.320722528102358)*(sj1)*(sj3)))+(((-0.320722528102358)*(cj1)*(cj3)))+(((0.0133439112568960)*(cj1)*(sj3))))) != 0)?((IKReal)1/(((((-0.0133439112568960)*(cj3)*(sj1)))+(((-0.320722528102358)*(sj1)*(sj3)))+(((-0.320722528102358)*(cj1)*(cj3)))+(((0.0133439112568960)*(cj1)*(sj3)))))):(IKReal)1.0e30))*(((((-0.100000000000000)*(cj1)))+(px)))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                                                sj4array[0]=IKsin(j4array[0]);
                                                                                                cj4array[0]=IKcos(j4array[0]);
                                                                                                if( j4array[0] > IKPI )
                                                                                                    j4array[0]-=IK2PI;
                                                                                                else if( j4array[0] < -IKPI )
                                                                                                    j4array[0]+=IK2PI;
                                                                                                j4valid[0] = true;
                                                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                                {
                                                                                                if( !j4valid[ij4] ) { continue; }
                                                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                                
                                                                                                rotationfunction0(vsolutions);
                                                                                                }
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                
                                                                                } else
                                                                                {
                                                                                    {
                                                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                    bool j4valid[1]={false};
                                                                                    j4array[0]=IKatan2(((-1.00000000000000)*(((IKabs(((((-0.320722528102358)*(cj3)))+(((0.0133439112568960)*(sj3))))) != 0)?((IKReal)1/(((((-0.320722528102358)*(cj3)))+(((0.0133439112568960)*(sj3)))))):(IKReal)1.0e30))*(((0.100000000000000)+(((-1.00000000000000)*(cj1)*(px)))+(((-1.00000000000000)*(py)*(sj1)))))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                                    sj4array[0]=IKsin(j4array[0]);
                                                                                    cj4array[0]=IKcos(j4array[0]);
                                                                                    if( j4array[0] > IKPI )
                                                                                        j4array[0]-=IK2PI;
                                                                                    else if( j4array[0] < -IKPI )
                                                                                        j4array[0]+=IK2PI;
                                                                                    j4valid[0] = true;
                                                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                    {
                                                                                    if( !j4valid[ij4] ) { continue; }
                                                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                    
                                                                                    rotationfunction0(vsolutions);
                                                                                    }
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                        }
                                                                        }
                                                                        }
                                                                        }
                                                                    
                                                                    } else
                                                                    {
                                                                        {
                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                        bool j4valid[1]={false};
                                                                        j4array[0]=IKatan2(((((IKabs(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))))) != 0)?((IKReal)1/(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))))), ((-1.24610591900312)+(((3.11526479750779)*(j0)))+(((-3.11526479750779)*(pz)))));
                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                        if( j4array[0] > IKPI )
                                                                            j4array[0]-=IK2PI;
                                                                        else if( j4array[0] < -IKPI )
                                                                            j4array[0]+=IK2PI;
                                                                        j4valid[0] = true;
                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                        {
                                                                        if( !j4valid[ij4] ) { continue; }
                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                        
                                                                        rotationfunction0(vsolutions);
                                                                        }
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                }
                                                            
                                                            } else
                                                            {
                                                                evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((1.57079632679490)+(3.14159265358979)+(j2)), ((2.00000000000000)*(3.14159265358979)))));
                                                                evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                evalcond[2]=((((-0.0415698169996759)*(cj3)*(py)*(sj1)))+(((-0.999135601564979)*(cj3)*(px)*(sj1)))+(((0.999135601564979)*(cj1)*(cj3)*(py)))+(((0.0415698169996759)*(px)*(sj1)*(sj3)))+(((-0.999135601564979)*(py)*(sj1)*(sj3)))+(((-0.0415698169996759)*(cj1)*(cj3)*(px)))+(((0.00415698169996759)*(cj3)))+(((0.0999135601564980)*(sj3)))+(((-0.999135601564979)*(cj1)*(px)*(sj3)))+(((-0.0415698169996759)*(cj1)*(py)*(sj3))));
                                                                if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                {
                                                                {
                                                                    IKReal j4eval[1];
                                                                    j4eval[0]=((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))));
                                                                    if( IKabs(j4eval[0]) < 0.000001  )
                                                                    {
                                                                        {
                                                                        IKReal evalcond[3];
                                                                            evalcond[0]=((IKfmod(((0.0415817987708408)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                            evalcond[2]=((-8.67361737988404e-19)+(((6.93889390390723e-18)*(cj1)*(px)))+(((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))+(((6.93889390390723e-18)*(py)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((-0.311526479750779)+(((3.11526479750779)*(cj1)*(px)))+(((3.11526479750779)*(py)*(sj1)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            evalcond[0]=((IKfmod(((-3.10001085481895)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                            evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                            evalcond[2]=((1.90819582357449e-17)+(((-1.00000000000000)*(cj1)*(py)))+(((-1.87350135405495e-16)*(cj1)*(px)))+(((-1.87350135405495e-16)*(py)*(sj1)))+(((px)*(sj1))));
                                                                            if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                            {
                                                                            {
                                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                                            bool j4valid[1]={false};
                                                                            j4array[0]=IKatan2(((0.311526479750779)+(((-3.11526479750779)*(py)*(sj1)))+(((-3.11526479750779)*(cj1)*(px)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                            sj4array[0]=IKsin(j4array[0]);
                                                                            cj4array[0]=IKcos(j4array[0]);
                                                                            if( j4array[0] > IKPI )
                                                                                j4array[0]-=IK2PI;
                                                                            else if( j4array[0] < -IKPI )
                                                                                j4array[0]+=IK2PI;
                                                                            j4valid[0] = true;
                                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                            {
                                                                            if( !j4valid[ij4] ) { continue; }
                                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                            
                                                                            rotationfunction0(vsolutions);
                                                                            }
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                            if( 1 )
                                                                            {
                                                                            {
                                                                                IKReal j4eval[1];
                                                                                j4eval[0]=((((-0.0133439112568960)*(sj3)))+(((0.320722528102358)*(cj3))));
                                                                                if( IKabs(j4eval[0]) < 0.000001  )
                                                                                {
                                                                                    {
                                                                                    IKReal evalcond[3];
                                                                                        evalcond[0]=((IKfmod(((1.61237812556574)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979))))+(((-1.00000000000000)*(3.14159265358979))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                                        evalcond[2]=((-0.100000000000000)+(((-6.93889390390723e-18)*(cj1)*(py)))+(((6.93889390390723e-18)*(px)*(sj1)))+(((cj1)*(px)))+(((py)*(sj1))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((((3.11526479750779)*(px)*(sj1)))+(((-3.11526479750779)*(cj1)*(py)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        evalcond[0]=((((-1.00000000000000)*(3.14159265358979)))+(IKfmod(((-1.52921452802406)+(3.14159265358979)+(j3)), ((2.00000000000000)*(3.14159265358979)))));
                                                                                        evalcond[1]=((-0.0669590000000000)+(((0.800000000000000)*(pz)))+(((0.200000000000000)*(cj1)*(px)))+(((0.200000000000000)*(py)*(sj1)))+(((-1.00000000000000)*((j0)*(j0))))+(((-1.00000000000000)*((px)*(px))))+(((-1.00000000000000)*((py)*(py))))+(((-1.00000000000000)*((pz)*(pz))))+(((2.00000000000000)*(j0)*(pz)))+(((-0.800000000000000)*(j0))));
                                                                                        evalcond[2]=((0.100000000000000)+(((-1.00000000000000)*(cj1)*(px)))+(((1.31838984174237e-16)*(cj1)*(py)))+(((-1.00000000000000)*(py)*(sj1)))+(((-1.31838984174237e-16)*(px)*(sj1))));
                                                                                        if( IKabs(evalcond[0]) < 0.000010  && IKabs(evalcond[1]) < 0.000010  && IKabs(evalcond[2]) < 0.000010  )
                                                                                        {
                                                                                        {
                                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                        bool j4valid[1]={false};
                                                                                        j4array[0]=IKatan2(((((3.11526479750779)*(cj1)*(py)))+(((-3.11526479750779)*(px)*(sj1)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                                        if( j4array[0] > IKPI )
                                                                                            j4array[0]-=IK2PI;
                                                                                        else if( j4array[0] < -IKPI )
                                                                                            j4array[0]+=IK2PI;
                                                                                        j4valid[0] = true;
                                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                        {
                                                                                        if( !j4valid[ij4] ) { continue; }
                                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                        
                                                                                        rotationfunction0(vsolutions);
                                                                                        }
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                        if( 1 )
                                                                                        {
                                                                                        {
                                                                                            IKReal j4eval[1];
                                                                                            j4eval[0]=((((-0.0133439112568960)*(cj3)*(sj1)))+(((0.320722528102358)*(cj1)*(cj3)))+(((-0.320722528102358)*(sj1)*(sj3)))+(((-0.0133439112568960)*(cj1)*(sj3))));
                                                                                            if( IKabs(j4eval[0]) < 0.000001  )
                                                                                            {
                                                                                                {
                                                                                                    if( 1 )
                                                                                                    {
                                                                                                    continue;
                                                                                                
                                                                                                } else
                                                                                                {
                                                                                                }
                                                                                                }
                                                                                            
                                                                                            } else
                                                                                            {
                                                                                                {
                                                                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                                bool j4valid[1]={false};
                                                                                                j4array[0]=IKatan2(((((IKabs(((((-0.0133439112568960)*(cj3)*(sj1)))+(((0.320722528102358)*(cj1)*(cj3)))+(((-0.320722528102358)*(sj1)*(sj3)))+(((-0.0133439112568960)*(cj1)*(sj3))))) != 0)?((IKReal)1/(((((-0.0133439112568960)*(cj3)*(sj1)))+(((0.320722528102358)*(cj1)*(cj3)))+(((-0.320722528102358)*(sj1)*(sj3)))+(((-0.0133439112568960)*(cj1)*(sj3)))))):(IKReal)1.0e30))*(((((-0.100000000000000)*(cj1)))+(px)))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                                                sj4array[0]=IKsin(j4array[0]);
                                                                                                cj4array[0]=IKcos(j4array[0]);
                                                                                                if( j4array[0] > IKPI )
                                                                                                    j4array[0]-=IK2PI;
                                                                                                else if( j4array[0] < -IKPI )
                                                                                                    j4array[0]+=IK2PI;
                                                                                                j4valid[0] = true;
                                                                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                                {
                                                                                                if( !j4valid[ij4] ) { continue; }
                                                                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                                
                                                                                                rotationfunction0(vsolutions);
                                                                                                }
                                                                                                }
                                                                                            
                                                                                            }
                                                                                        
                                                                                        }
                                                                                    
                                                                                    } else
                                                                                    {
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                    }
                                                                                
                                                                                } else
                                                                                {
                                                                                    {
                                                                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                                                                    bool j4valid[1]={false};
                                                                                    j4array[0]=IKatan2(((-1.00000000000000)*(((IKabs(((((-0.0133439112568960)*(sj3)))+(((0.320722528102358)*(cj3))))) != 0)?((IKReal)1/(((((-0.0133439112568960)*(sj3)))+(((0.320722528102358)*(cj3)))))):(IKReal)1.0e30))*(((0.100000000000000)+(((-1.00000000000000)*(cj1)*(px)))+(((-1.00000000000000)*(py)*(sj1)))))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                                    sj4array[0]=IKsin(j4array[0]);
                                                                                    cj4array[0]=IKcos(j4array[0]);
                                                                                    if( j4array[0] > IKPI )
                                                                                        j4array[0]-=IK2PI;
                                                                                    else if( j4array[0] < -IKPI )
                                                                                        j4array[0]+=IK2PI;
                                                                                    j4valid[0] = true;
                                                                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                                    {
                                                                                    if( !j4valid[ij4] ) { continue; }
                                                                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                                    
                                                                                    rotationfunction0(vsolutions);
                                                                                    }
                                                                                    }
                                                                                
                                                                                }
                                                                            
                                                                            }
                                                                        
                                                                        } else
                                                                        {
                                                                        }
                                                                        }
                                                                        }
                                                                        }
                                                                    
                                                                    } else
                                                                    {
                                                                        {
                                                                        IKReal j4array[1], cj4array[1], sj4array[1];
                                                                        bool j4valid[1]={false};
                                                                        j4array[0]=IKatan2(((((IKabs(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))))) != 0)?((IKReal)1/(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))))), ((-1.24610591900312)+(((-3.11526479750779)*(j0)))+(((3.11526479750779)*(pz)))));
                                                                        sj4array[0]=IKsin(j4array[0]);
                                                                        cj4array[0]=IKcos(j4array[0]);
                                                                        if( j4array[0] > IKPI )
                                                                            j4array[0]-=IK2PI;
                                                                        else if( j4array[0] < -IKPI )
                                                                            j4array[0]+=IK2PI;
                                                                        j4valid[0] = true;
                                                                        for(int ij4 = 0; ij4 < 1; ++ij4)
                                                                        {
                                                                        if( !j4valid[ij4] ) { continue; }
                                                                        j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                                        
                                                                        rotationfunction0(vsolutions);
                                                                        }
                                                                        }
                                                                    
                                                                    }
                                                                
                                                                }
                                                            
                                                            } else
                                                            {
                                                                if( 1 )
                                                                {
                                                                continue;
                                                            
                                                            } else
                                                            {
                                                            }
                                                            }
                                                            }
                                                            }
                                                        
                                                        } else
                                                        {
                                                            {
                                                            IKReal j4array[1], cj4array[1], sj4array[1];
                                                            bool j4valid[1]={false};
                                                            j4array[0]=IKatan2(((((IKabs(((((-0.102951931520857)*(cj2)*(cj3)))+(((0.00428339551346361)*(cj2)*(sj3))))) != 0)?((IKReal)1/(((((-0.102951931520857)*(cj2)*(cj3)))+(((0.00428339551346361)*(cj2)*(sj3)))))):(IKReal)1.0e30))*(((((0.128400000000000)*(sj2)))+(((0.321000000000000)*(pz)))+(((-0.321000000000000)*(j0)))+(((-0.321000000000000)*(sj2)*(((0.400000000000000)+(((0.100000000000000)*(cj2)))+(((-1.00000000000000)*(cj2)*(py)*(sj1)))+(((-1.00000000000000)*(cj1)*(cj2)*(px)))+(((pz)*(sj2)))+(((-1.00000000000000)*(j0)*(sj2)))))))))), ((-1.24610591900312)+(((3.11526479750779)*(cj1)*(cj2)*(px)))+(((-0.311526479750779)*(cj2)))+(((3.11526479750779)*(j0)*(sj2)))+(((3.11526479750779)*(cj2)*(py)*(sj1)))+(((-3.11526479750779)*(pz)*(sj2)))));
                                                            sj4array[0]=IKsin(j4array[0]);
                                                            cj4array[0]=IKcos(j4array[0]);
                                                            if( j4array[0] > IKPI )
                                                                j4array[0]-=IK2PI;
                                                            else if( j4array[0] < -IKPI )
                                                                j4array[0]+=IK2PI;
                                                            j4valid[0] = true;
                                                            for(int ij4 = 0; ij4 < 1; ++ij4)
                                                            {
                                                            if( !j4valid[ij4] ) { continue; }
                                                            j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                            
                                                            rotationfunction0(vsolutions);
                                                            }
                                                            }
                                                        
                                                        }
                                                    
                                                    }
                                                
                                                } else
                                                {
                                                }
                                                }
                                                }
                                                }
                                            
                                            } else
                                            {
                                                {
                                                IKReal j4array[1], cj4array[1], sj4array[1];
                                                bool j4valid[1]={false};
                                                j4array[0]=IKatan2(((((IKabs(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))))) != 0)?((IKReal)1/(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))))), ((-0.985362149532710)+(((3.89408099688473)*((py)*(py))))+(((3.89408099688473)*((j0)*(j0))))+(((3.89408099688473)*((pz)*(pz))))+(((-0.778816199376947)*(py)*(sj1)))+(((3.89408099688473)*((px)*(px))))+(((-0.778816199376947)*(cj1)*(px)))+(((-7.78816199376947)*(j0)*(pz)))));
                                                sj4array[0]=IKsin(j4array[0]);
                                                cj4array[0]=IKcos(j4array[0]);
                                                if( j4array[0] > IKPI )
                                                    j4array[0]-=IK2PI;
                                                else if( j4array[0] < -IKPI )
                                                    j4array[0]+=IK2PI;
                                                j4valid[0] = true;
                                                for(int ij4 = 0; ij4 < 1; ++ij4)
                                                {
                                                if( !j4valid[ij4] ) { continue; }
                                                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                                
                                                rotationfunction0(vsolutions);
                                                }
                                                }
                                            
                                            }
                                        
                                        }
                                    
                                    } else
                                    {
                                    }
                                    }
                                    }
                                    }
                                
                                } else
                                {
                                    {
                                    IKReal j4array[1], cj4array[1], sj4array[1];
                                    bool j4valid[1]={false};
                                    j4array[0]=IKatan2(((((IKabs(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))))) != 0)?((IKReal)1/(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))))), ((-1.24610591900312)+(((3.11526479750779)*(cj1)*(cj2)*(px)))+(((-0.311526479750779)*(cj2)))+(((3.11526479750779)*(j0)*(sj2)))+(((3.11526479750779)*(cj2)*(py)*(sj1)))+(((-3.11526479750779)*(pz)*(sj2)))));
                                    sj4array[0]=IKsin(j4array[0]);
                                    cj4array[0]=IKcos(j4array[0]);
                                    if( j4array[0] > IKPI )
                                        j4array[0]-=IK2PI;
                                    else if( j4array[0] < -IKPI )
                                        j4array[0]+=IK2PI;
                                    j4valid[0] = true;
                                    for(int ij4 = 0; ij4 < 1; ++ij4)
                                    {
                                    if( !j4valid[ij4] ) { continue; }
                                    j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                                    
                                    rotationfunction0(vsolutions);
                                    }
                                    }
                                
                                }
                            
                            }
                            }
                            }
                        
                        }
                    
                    }
                
                } else
                {
                }
                }
                }
                }
            
            } else
            {
                {
                IKReal j4array[1], cj4array[1], sj4array[1];
                bool j4valid[1]={false};
                j4array[0]=IKatan2(((((IKabs(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3))))) != 0)?((IKReal)1/(((((0.0133439112568960)*(cj3)))+(((0.320722528102358)*(sj3)))))):(IKReal)1.0e30))*(((((-1.00000000000000)*(px)*(sj1)))+(((cj1)*(py)))))), ((-0.985362149532710)+(((3.89408099688473)*((py)*(py))))+(((3.89408099688473)*((j0)*(j0))))+(((3.89408099688473)*((pz)*(pz))))+(((-0.778816199376947)*(py)*(sj1)))+(((3.89408099688473)*((px)*(px))))+(((-0.778816199376947)*(cj1)*(px)))+(((-7.78816199376947)*(j0)*(pz)))));
                sj4array[0]=IKsin(j4array[0]);
                cj4array[0]=IKcos(j4array[0]);
                if( j4array[0] > IKPI )
                    j4array[0]-=IK2PI;
                else if( j4array[0] < -IKPI )
                    j4array[0]+=IK2PI;
                j4valid[0] = true;
                for(int ij4 = 0; ij4 < 1; ++ij4)
                {
                if( !j4valid[ij4] ) { continue; }
                j4 = j4array[ij4]; cj4 = cj4array[ij4]; sj4 = sj4array[ij4];
                
                {
                    IKReal j2eval[1];
                    IKReal x473=(sj4)*(sj4);
                    j2eval[0]=((0.160000000000000)+(((0.103041000000000)*((cj4)*(cj4))))+(((0.000178059967631915)*(x473)*((sj3)*(sj3))))+(((-0.00855938590617039)*(cj3)*(sj3)*(x473)))+(((0.256800000000000)*(cj4)))+(((0.102862940032368)*(x473)*((cj3)*(cj3)))));
                    if( IKabs(j2eval[0]) < 0.000001  )
                    {
                        {
                            if( 1 )
                            {
                            {
                                IKReal j2eval[1];
                                j2eval[0]=((0.0400000000000000)+(((-0.320722528102358)*(cj3)*(pz)*(sj4)))+(((0.0321000000000000)*(cj4)))+(((-0.321000000000000)*(cj4)*(py)*(sj1)))+(((0.0133439112568960)*(pz)*(sj3)*(sj4)))+(((-0.0133439112568960)*(j0)*(sj3)*(sj4)))+(((-0.400000000000000)*(cj1)*(px)))+(((-0.321000000000000)*(cj1)*(cj4)*(px)))+(((0.320722528102358)*(cj3)*(j0)*(sj4)))+(((-0.400000000000000)*(py)*(sj1))));
                                if( IKabs(j2eval[0]) < 0.000001  )
                                {
                                    {
                                        if( 1 )
                                        {
                                        {
                                            IKReal j2eval[1];
                                            j2eval[0]=((((0.321000000000000)*(cj4)*(pz)))+(((0.0133439112568960)*(py)*(sj1)*(sj3)*(sj4)))+(((0.0320722528102358)*(cj3)*(sj4)))+(((-0.320722528102358)*(cj1)*(cj3)*(px)*(sj4)))+(((-0.400000000000000)*(j0)))+(((0.0133439112568960)*(cj1)*(px)*(sj3)*(sj4)))+(((-0.321000000000000)*(cj4)*(j0)))+(((0.400000000000000)*(pz)))+(((-0.320722528102358)*(cj3)*(py)*(sj1)*(sj4)))+(((-0.00133439112568960)*(sj3)*(sj4))));
                                            if( IKabs(j2eval[0]) < 0.000001  )
                                            {
                                                {
                                                    if( 1 )
                                                    {
                                                    continue;
                                                
                                                } else
                                                {
                                                }
                                                }
                                            
                                            } else
                                            {
                                                {
                                                IKReal j2array[1], cj2array[1], sj2array[1];
                                                bool j2valid[1]={false};
                                                IKReal x474=((((-1.00000000000000)*(pz)))+(j0));
                                                IKReal x475=((0.0133439112568960)*(sj3)*(sj4));
                                                IKReal x476=((0.320722528102358)*(cj3)*(sj4));
                                                IKReal x477=((((-1.00000000000000)*(x476)))+(x475));
                                                IKReal x478=((0.321000000000000)*(cj4)*(pz));
                                                IKReal x479=((py)*(sj1)*(x475));
                                                IKReal x480=((0.0320722528102358)*(cj3)*(sj4));
                                                IKReal x481=((cj1)*(px)*(x475));
                                                IKReal x482=((0.400000000000000)*(pz));
                                                IKReal x483=((x481)+(x480)+(x482)+(x478)+(x479));
                                                IKReal x484=((1.00000000000000)*(py)*(sj1)*(x476));
                                                IKReal x485=((0.321000000000000)*(cj4)*(j0));
                                                IKReal x486=((0.400000000000000)*(j0));
                                                IKReal x487=((0.00133439112568960)*(sj3)*(sj4));
                                                IKReal x488=((1.00000000000000)*(cj1)*(px)*(x476));
                                                IKReal x489=((x488)+(x485)+(x484)+(x487)+(x486));
                                                IKReal x490=((((-1.00000000000000)*(x489)))+(x483));
                                                IKReal x491=((IKabs(x490) != 0)?((IKReal)1/(x490)):(IKReal)1.0e30);
                                                j2array[0]=IKatan2(((x491)*((((x477)*(x477))+(((-1.00000000000000)*((x474)*(x474))))))), ((x491)*(((((x477)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))))+(((x474)*(((0.100000000000000)+(((-1.00000000000000)*(cj1)*(px)))+(((-1.00000000000000)*(py)*(sj1)))))))))));
                                                sj2array[0]=IKsin(j2array[0]);
                                                cj2array[0]=IKcos(j2array[0]);
                                                if( j2array[0] > IKPI )
                                                    j2array[0]-=IK2PI;
                                                else if( j2array[0] < -IKPI )
                                                    j2array[0]+=IK2PI;
                                                j2valid[0] = true;
                                                for(int ij2 = 0; ij2 < 1; ++ij2)
                                                {
                                                if( !j2valid[ij2] ) { continue; }
                                                j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                                
                                                rotationfunction0(vsolutions);
                                                }
                                                }
                                            
                                            }
                                        
                                        }
                                    
                                    } else
                                    {
                                    }
                                    }
                                
                                } else
                                {
                                    {
                                    IKReal j2array[1], cj2array[1], sj2array[1];
                                    bool j2valid[1]={false};
                                    IKReal x492=((pz)+(((-1.00000000000000)*(j0))));
                                    IKReal x493=((0.0321000000000000)*(cj4));
                                    IKReal x494=((0.0133439112568960)*(pz)*(sj3)*(sj4));
                                    IKReal x495=((0.320722528102358)*(cj3)*(j0)*(sj4));
                                    IKReal x496=((0.0400000000000000)+(x493)+(x494)+(x495));
                                    IKReal x497=((0.400000000000000)*(cj1)*(px));
                                    IKReal x498=((0.400000000000000)*(py)*(sj1));
                                    IKReal x499=((0.321000000000000)*(cj1)*(cj4)*(px));
                                    IKReal x500=((0.321000000000000)*(cj4)*(py)*(sj1));
                                    IKReal x501=((0.320722528102358)*(cj3)*(pz)*(sj4));
                                    IKReal x502=((0.0133439112568960)*(j0)*(sj3)*(sj4));
                                    IKReal x503=((x498)+(x499)+(x497)+(x500)+(x501)+(x502));
                                    IKReal x504=((((-1.00000000000000)*(x503)))+(x496));
                                    IKReal x505=((IKabs(x504) != 0)?((IKReal)1/(x504)):(IKReal)1.0e30);
                                    j2array[0]=IKatan2(((x505)*(((((-1.00000000000000)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))*(((((0.0133439112568960)*(sj3)*(sj4)))+(((-0.320722528102358)*(cj3)*(sj4)))))))+(((-1.00000000000000)*(x492)*(((0.100000000000000)+(((-1.00000000000000)*(cj1)*(px)))+(((-1.00000000000000)*(py)*(sj1)))))))))), ((x505)*((((x492)*(x492))+(((-1.00000000000000)*((((0.400000000000000)+(((0.321000000000000)*(cj4)))))*(((0.400000000000000)+(((0.321000000000000)*(cj4))))))))))));
                                    sj2array[0]=IKsin(j2array[0]);
                                    cj2array[0]=IKcos(j2array[0]);
                                    if( j2array[0] > IKPI )
                                        j2array[0]-=IK2PI;
                                    else if( j2array[0] < -IKPI )
                                        j2array[0]+=IK2PI;
                                    j2valid[0] = true;
                                    for(int ij2 = 0; ij2 < 1; ++ij2)
                                    {
                                    if( !j2valid[ij2] ) { continue; }
                                    j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                                    
                                    rotationfunction0(vsolutions);
                                    }
                                    }
                                
                                }
                            
                            }
                        
                        } else
                        {
                        }
                        }
                    
                    } else
                    {
                        {
                        IKReal j2array[1], cj2array[1], sj2array[1];
                        bool j2valid[1]={false};
                        IKReal x506=(sj4)*(sj4);
                        IKReal x507=((cj1)*(px));
                        IKReal x508=((py)*(sj1));
                        IKReal x509=((0.0133439112568960)*(sj3)*(sj4));
                        IKReal x510=((0.320722528102358)*(cj3)*(sj4));
                        IKReal x511=((x509)+(((-1.00000000000000)*(x510))));
                        IKReal x512=x2;
                        IKReal x513=((0.000178059967631915)*(x506)*(x512));
                        IKReal x514=x3;
                        IKReal x515=((0.102862940032368)*(x506)*(x514));
                        IKReal x516=(cj4)*(cj4);
                        IKReal x517=((0.103041000000000)*(x516));
                        IKReal x518=((0.256800000000000)*(cj4));
                        IKReal x519=((0.160000000000000)+(x518)+(x517)+(x515)+(x513));
                        IKReal x520=((0.00855938590617039)*(cj3)*(sj3)*(x506));
                        IKReal x521=((((-1.00000000000000)*(x520)))+(x519));
                        IKReal x522=((IKabs(x521) != 0)?((IKReal)1/(x521)):(IKReal)1.0e30);
                        j2array[0]=IKatan2(((x522)*(((((-1.00000000000000)*(x511)*(((0.100000000000000)+(((-1.00000000000000)*(x508)))+(((-1.00000000000000)*(x507)))))))+(((((0.400000000000000)+(((0.321000000000000)*(cj4)))))*(((((-1.00000000000000)*(pz)))+(j0)))))))), ((x522)*(((((-1.00000000000000)*(((0.400000000000000)+(((0.321000000000000)*(cj4)))))*(((0.100000000000000)+(((-1.00000000000000)*(x508)))+(((-1.00000000000000)*(x507)))))))+(((x511)*(((pz)+(((-1.00000000000000)*(j0)))))))))));
                        sj2array[0]=IKsin(j2array[0]);
                        cj2array[0]=IKcos(j2array[0]);
                        if( j2array[0] > IKPI )
                            j2array[0]-=IK2PI;
                        else if( j2array[0] < -IKPI )
                            j2array[0]+=IK2PI;
                        j2valid[0] = true;
                        for(int ij2 = 0; ij2 < 1; ++ij2)
                        {
                        if( !j2valid[ij2] ) { continue; }
                        j2 = j2array[ij2]; cj2 = cj2array[ij2]; sj2 = sj2array[ij2];
                        
                        rotationfunction0(vsolutions);
                        }
                        }
                    
                    }
                
                }
                }
                }
            
            }
        
        }
            }
    
    } else
    {
    }
    }
}
return vsolutions.size()>0;
}
    inline void rotationfunction0(std::vector<IKSolution>& vsolutions) {
        IKReal x27=((cj2)*(cj4)*(sj1));
        IKReal x28=((0.999135601564979)*(cj1)*(sj3)*(sj4));
        IKReal x29=((0.0415698169996759)*(sj1)*(sj2)*(sj3)*(sj4));
        IKReal x30=((0.0415698169996759)*(cj1)*(cj3)*(sj4));
        IKReal x31=((x30)+(x27)+(x28)+(x29));
        IKReal x32=((0.999135601564979)*(cj3)*(sj1)*(sj2)*(sj4));
        IKReal x33=((((-1.00000000000000)*(x32)))+(x31));
        IKReal x34=((0.0415698169996759)*(cj1)*(sj2)*(sj3)*(sj4));
        IKReal x35=((cj1)*(cj2)*(cj4));
        IKReal x36=((x35)+(x34));
        IKReal x37=((0.999135601564979)*(sj1)*(sj3)*(sj4));
        IKReal x38=((0.0415698169996759)*(cj3)*(sj1)*(sj4));
        IKReal x39=((0.999135601564979)*(cj1)*(cj3)*(sj2)*(sj4));
        IKReal x40=((x39)+(x38)+(x37));
        IKReal x41=((((-1.00000000000000)*(x40)))+(x36));
        IKReal x42=((0.0415698169996759)*(cj2)*(sj3)*(sj4));
        IKReal x43=((0.999135601564979)*(cj2)*(cj3)*(sj4));
        IKReal x44=((cj4)*(sj2));
        IKReal x45=((x44)+(x43));
        IKReal x46=((((-1.00000000000000)*(x45)))+(x42));
        IKReal x47=((0.999135601564979)*(sj1)*(sj2)*(sj3));
        IKReal x48=((0.0415698169996759)*(cj3)*(sj1)*(sj2));
        IKReal x49=((0.999135601564979)*(cj1)*(cj3));
        IKReal x50=((x48)+(x49)+(x47));
        IKReal x51=((0.0415698169996759)*(cj1)*(sj3));
        IKReal x52=((x50)+(((-1.00000000000000)*(x51))));
        IKReal x53=((0.0415698169996759)*(cj2)*(cj3));
        IKReal x54=((0.999135601564979)*(cj2)*(sj3));
        IKReal x55=((x54)+(x53));
        IKReal x56=((0.999135601564979)*(cj1)*(sj2)*(sj3));
        IKReal x57=((0.0415698169996759)*(sj1)*(sj3));
        IKReal x58=((0.0415698169996759)*(cj1)*(cj3)*(sj2));
        IKReal x59=((x58)+(x57)+(x56));
        IKReal x60=((0.999135601564979)*(cj3)*(sj1));
        IKReal x61=((x59)+(((-1.00000000000000)*(x60))));
        IKReal x62=((cj1)*(cj2)*(sj4));
        IKReal x63=((0.0415698169996759)*(cj3)*(cj4)*(sj1));
        IKReal x64=((x44)*(x49));
        IKReal x65=((0.999135601564979)*(cj4)*(sj1)*(sj3));
        IKReal x66=((x62)+(x63)+(x64)+(x65));
        IKReal x67=((x44)*(x51));
        IKReal x68=((x66)+(((-1.00000000000000)*(x67))));
        IKReal x69=((0.999135601564979)*(cj2)*(cj3)*(cj4));
        IKReal x70=((sj2)*(sj4));
        IKReal x71=((0.0415698169996759)*(cj2)*(cj4)*(sj3));
        IKReal x72=((x71)+(x70));
        IKReal x73=((((-1.00000000000000)*(x72)))+(x69));
        IKReal x74=((cj2)*(sj1)*(sj4));
        IKReal x75=((x44)*(x60));
        IKReal x76=((x75)+(x74));
        IKReal x77=((x44)*(x57));
        IKReal x78=((0.0415698169996759)*(cj1)*(cj3)*(cj4));
        IKReal x79=((0.999135601564979)*(cj1)*(cj4)*(sj3));
        IKReal x80=((x77)+(x79)+(x78));
        IKReal x81=((x76)+(((-1.00000000000000)*(x80))));
        new_r00=((((r20)*(x46)))+(((r00)*(x41)))+(((r10)*(x33))));
        new_r01=((((r01)*(x41)))+(((r11)*(x33)))+(((r21)*(x46))));
        new_r02=((((r22)*(x46)))+(((r02)*(x41)))+(((r12)*(x33))));
        new_r10=((((r10)*(x52)))+(((r00)*(x61)))+(((r20)*(x55))));
        new_r11=((((r11)*(x52)))+(((r01)*(x61)))+(((r21)*(x55))));
        new_r12=((((r22)*(x55)))+(((r02)*(x61)))+(((r12)*(x52))));
        new_r20=((((r10)*(x81)))+(((r00)*(x68)))+(((r20)*(x73))));
        new_r21=((((r21)*(x73)))+(((r11)*(x81)))+(((r01)*(x68))));
        new_r22=((((r12)*(x81)))+(((r02)*(x68)))+(((r22)*(x73))));
        {
        IKReal j6eval;
        j6eval=new_r00;
        if( j6eval >= -1.000010 && j6eval <= -0.999990 )
        {
            {
            j6 = 3.141593; sj6 = 0.000000; cj6 = -1.000000;
            IKReal j5mul = 1;
            j5=0;
            IKReal j7mul=1.00000000000000;
            j7=((-1.00000000000000)*(IKatan2(new_r12, new_r11)));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(8);
            solution.basesol[0].foffset = j0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[5].fmul = j5mul;
            solution.basesol[5].freeind = 0;
            solution.basesol[6].foffset = j6;
            solution.basesol[7].foffset = j7;
            solution.basesol[7].fmul = j7mul;
            solution.basesol[7].freeind = 0;
            solution.vfree.resize(1);
            solution.vfree[0] = 5;
            }
        
        } else
        if( j6eval >= 0.999990 && j6eval <= 1.000010 )
        {
            {
            j6 = 0.000000; sj6 = 0.000000; cj6 = 1.000000;
            IKReal j5mul = 1;
            j5=0;
            IKReal j7mul=-1.00000000000000;
            j7=((((-1.00000000000000)*(IKatan2(new_r21, ((-1.00000000000000)*(new_r22))))))+(((-1.00000000000000)*(3.14159265358979))));
            vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
            solution.basesol.resize(8);
            solution.basesol[0].foffset = j0;
            solution.basesol[1].foffset = j1;
            solution.basesol[2].foffset = j2;
            solution.basesol[3].foffset = j3;
            solution.basesol[4].foffset = j4;
            solution.basesol[5].foffset = j5;
            solution.basesol[5].fmul = j5mul;
            solution.basesol[5].freeind = 0;
            solution.basesol[6].foffset = j6;
            solution.basesol[7].foffset = j7;
            solution.basesol[7].fmul = j7mul;
            solution.basesol[7].freeind = 0;
            solution.vfree.resize(1);
            solution.vfree[0] = 5;
            }
        
        } else
        {
            {
            IKReal j5array[2], cj5array[2], sj5array[2];
            bool j5valid[2]={false};
            j5array[0]=IKatan2(new_r10, ((-1.00000000000000)*(new_r20)));
            sj5array[0]=IKsin(j5array[0]);
            cj5array[0]=IKcos(j5array[0]);
            j5array[1] = j5array[0] > 0 ? j5array[0]-IKPI : j5array[0]+IKPI;
            sj5array[1] = -sj5array[0];
            cj5array[1] = -cj5array[0];
            if( j5array[0] > IKPI )
                j5array[0]-=IK2PI;
            else if( j5array[0] < -IKPI )
                j5array[0]+=IK2PI;
            j5valid[0] = true;
            if( j5array[1] > IKPI )
                j5array[1]-=IK2PI;
            else if( j5array[1] < -IKPI )
                j5array[1]+=IK2PI;
            j5valid[1] = true;
            if( j5valid[0] && j5valid[1] && IKabs(cj5array[0]-cj5array[1]) < 0.0001 && IKabs(sj5array[0]-sj5array[1]) < 0.0001 )
                j5valid[1]=false;
            for(int ij5 = 0; ij5 < 2; ++ij5)
            {
            if( !j5valid[ij5] ) { continue; }
            j5 = j5array[ij5]; cj5 = cj5array[ij5]; sj5 = sj5array[ij5];
            
            {
            IKReal evalcond[1];
                evalcond[0]=IKsin(j5);
                if( IKabs(evalcond[0]) < 0.000010  )
                {
                {
                IKReal j7array[1], cj7array[1], sj7array[1];
                bool j7valid[1]={false};
                IKReal x82=IKcos(j5);
                IKReal x83=((IKabs(x82) != 0)?((IKReal)1/(x82)):(IKReal)1.0e30);
                j7array[0]=IKatan2(((-1.00000000000000)*(new_r12)*(x83)), ((new_r11)*(x83)));
                sj7array[0]=IKsin(j7array[0]);
                cj7array[0]=IKcos(j7array[0]);
                if( j7array[0] > IKPI )
                    j7array[0]-=IK2PI;
                else if( j7array[0] < -IKPI )
                    j7array[0]+=IK2PI;
                j7valid[0] = true;
                for(int ij7 = 0; ij7 < 1; ++ij7)
                {
                if( !j7valid[ij7] ) { continue; }
                j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];
                
                {
                IKReal j6array[1], cj6array[1], sj6array[1];
                bool j6valid[1]={false};
                j6array[0]=IKatan2(((-1.00000000000000)*(new_r20)*(((IKabs(IKcos(j5)) != 0)?((IKReal)1/(IKcos(j5))):(IKReal)1.0e30))), new_r00);
                sj6array[0]=IKsin(j6array[0]);
                cj6array[0]=IKcos(j6array[0]);
                if( j6array[0] > IKPI )
                    j6array[0]-=IK2PI;
                else if( j6array[0] < -IKPI )
                    j6array[0]+=IK2PI;
                j6valid[0] = true;
                for(int ij6 = 0; ij6 < 1; ++ij6)
                {
                if( !j6valid[ij6] ) { continue; }
                j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
                
                vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                solution.basesol.resize(8);
                solution.basesol[0].foffset = j0;
                solution.basesol[1].foffset = j1;
                solution.basesol[2].foffset = j2;
                solution.basesol[3].foffset = j3;
                solution.basesol[4].foffset = j4;
                solution.basesol[5].foffset = j5;
                solution.basesol[6].foffset = j6;
                solution.basesol[7].foffset = j7;
                solution.vfree.resize(0);
                }
                }
                }
                }
            
            } else
            {
                if( 1 )
                {
                {
                IKReal j6array[1], cj6array[1], sj6array[1];
                bool j6valid[1]={false};
                j6array[0]=IKatan2(((new_r10)*(((IKabs(sj5) != 0)?((IKReal)1/(sj5)):(IKReal)1.0e30))), new_r00);
                sj6array[0]=IKsin(j6array[0]);
                cj6array[0]=IKcos(j6array[0]);
                if( j6array[0] > IKPI )
                    j6array[0]-=IK2PI;
                else if( j6array[0] < -IKPI )
                    j6array[0]+=IK2PI;
                j6valid[0] = true;
                for(int ij6 = 0; ij6 < 1; ++ij6)
                {
                if( !j6valid[ij6] ) { continue; }
                j6 = j6array[ij6]; cj6 = cj6array[ij6]; sj6 = sj6array[ij6];
                
                {
                IKReal evalcond[1];
                    evalcond[0]=IKsin(j6);
                    if( IKabs(evalcond[0]) < 0.000010  )
                    {
                    {
                    IKReal j7array[1], cj7array[1], sj7array[1];
                    bool j7valid[1]={false};
                    IKReal x84=IKcos(j6);
                    IKReal x85=((IKabs(x84) != 0)?((IKReal)1/(x84)):(IKReal)1.0e30);
                    IKReal x86=IKcos(j5);
                    IKReal x87=IKsin(j5);
                    j7array[0]=IKatan2(((((new_r21)*(x85)*(x86)))+(((-1.00000000000000)*(new_r11)*(x85)*(x87)))), ((((new_r21)*(x87)))+(((new_r11)*(x86)))));
                    sj7array[0]=IKsin(j7array[0]);
                    cj7array[0]=IKcos(j7array[0]);
                    if( j7array[0] > IKPI )
                        j7array[0]-=IK2PI;
                    else if( j7array[0] < -IKPI )
                        j7array[0]+=IK2PI;
                    j7valid[0] = true;
                    for(int ij7 = 0; ij7 < 1; ++ij7)
                    {
                    if( !j7valid[ij7] ) { continue; }
                    j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(8);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.basesol[7].foffset = j7;
                    solution.vfree.resize(0);
                    }
                    }
                
                } else
                {
                    if( 1 )
                    {
                    {
                    IKReal j7array[1], cj7array[1], sj7array[1];
                    bool j7valid[1]={false};
                    IKReal x88=IKsin(j6);
                    IKReal x89=((IKabs(x88) != 0)?((IKReal)1/(x88)):(IKReal)1.0e30);
                    j7array[0]=IKatan2(((new_r01)*(x89)), ((new_r02)*(x89)));
                    sj7array[0]=IKsin(j7array[0]);
                    cj7array[0]=IKcos(j7array[0]);
                    if( j7array[0] > IKPI )
                        j7array[0]-=IK2PI;
                    else if( j7array[0] < -IKPI )
                        j7array[0]+=IK2PI;
                    j7valid[0] = true;
                    for(int ij7 = 0; ij7 < 1; ++ij7)
                    {
                    if( !j7valid[ij7] ) { continue; }
                    j7 = j7array[ij7]; cj7 = cj7array[ij7]; sj7 = sj7array[ij7];
                    
                    vsolutions.push_back(IKSolution()); IKSolution& solution = vsolutions.back();
                    solution.basesol.resize(8);
                    solution.basesol[0].foffset = j0;
                    solution.basesol[1].foffset = j1;
                    solution.basesol[2].foffset = j2;
                    solution.basesol[3].foffset = j3;
                    solution.basesol[4].foffset = j4;
                    solution.basesol[5].foffset = j5;
                    solution.basesol[6].foffset = j6;
                    solution.basesol[7].foffset = j7;
                    solution.vfree.resize(0);
                    }
                    }
                
                } else
                {
                }
                }
                }
                }
                }
            
            } else
            {
            }
            }
            }
            }
            }
        
        }
        }
    }    /// Durand-Kerner polynomial root finding method
    static void polyroots3(IKReal rawcoeffs[3+1], IKReal rawroots[3], int& numroots)
    {
        using std::complex;
        assert(rawcoeffs[0] != 0);
        const IKReal tol = 128.0*std::numeric_limits<IKReal>::epsilon();
        complex<IKReal> coeffs[3];
        const int maxsteps = 50;
        for(int i = 0; i < 3; ++i) {
            coeffs[i] = complex<IKReal>(rawcoeffs[i+1]/rawcoeffs[0]);
        }
        complex<IKReal> roots[3];
        IKReal err[3];
        roots[0] = complex<IKReal>(1,0);
        roots[1] = complex<IKReal>(0.4,0.9); // any complex number not a root of unity is works
        err[0] = 1.0;
        err[1] = 1.0;
        for(int i = 2; i < 3; ++i) {
            roots[i] = roots[i-1]*roots[1];
            err[i] = 1.0;
        }
        for(int step = 0; step < maxsteps; ++step) {
            bool changed = false;
            for(int i = 0; i < 3; ++i) {
                if ( err[i] >= tol ) {
                    changed = true;
                    // evaluate
                    complex<IKReal> x = roots[i] + coeffs[0];
                    for(int j = 1; j < 3; ++j) {
                        x = roots[i] * x + coeffs[j];
                    }
                    for(int j = 0; j < 3; ++j) {
                        if( i != j ) {
                            if( roots[i] != roots[j] ) {
                                x /= (roots[i] - roots[j]);
                            }
                        }
                    }
                    roots[i] -= x;
                    err[i] = abs(x);
                }
            }
            if( !changed ) {
                break;
            }
        }
        numroots = 0;
        for(int i = 0; i < 3; ++i) {
            if( IKabs(imag(roots[i])) < std::numeric_limits<IKReal>::epsilon() ) {
                rawroots[numroots++] = real(roots[i]);
            }
        }
    }
    static void polyroots2(IKReal rawcoeffs[2+1], IKReal rawroots[2], int& numroots) {
        IKReal det = rawcoeffs[1]*rawcoeffs[1]-4*rawcoeffs[0]*rawcoeffs[2];
        if( det < 0 ) {
            numroots=0;
        }
        else if( det == 0 ) {
            rawroots[0] = -0.5*rawcoeffs[1]/rawcoeffs[0];
            numroots = 1;
        }
        else {
            det = IKsqrt(det);
            rawroots[0] = 0.5*(-rawcoeffs[1]+det)/rawcoeffs[0];
            rawroots[1] = 0.5*(-rawcoeffs[1]-det)/rawcoeffs[0];
            numroots = 2;
        }
    }
    /// intersection of a conic and the unit circle
    static void conicsolver(IKReal C0[6], IKReal roots[4], int& numroots)
    {
        IKReal rawcoeffs[4] = {-1,
                               C0[5] - C0[0] - C0[3],
                               C0[0]*C0[5] + C0[3]*C0[5] - C0[0]*C0[3] + C0[1]*C0[1] - C0[2]*C0[2] - C0[4]*C0[4],
                               C0[0]*C0[3]*C0[5] + 2*C0[1]*C0[2]*C0[4] - C0[0]*C0[4]*C0[4] - C0[3]*C0[2]*C0[2] - C0[5]*C0[1]*C0[1]};
        IKReal proots[3];
        int numproots, numyroots;
        polyroots3(rawcoeffs,proots,numproots);
        numroots = 0;
        int iroot=0;
        IKReal a, b, c, d, e, f;
        a = C0[0]+proots[iroot]; b = C0[1]; c = C0[3]+proots[iroot]; d = C0[2]; e = C0[4]; f = C0[5]-proots[iroot];
        IKReal adjugate[9] = {c*f-e*e, -b*f+e*d, b*e-c*d, -b*f+d*e, a*f-d*d, -a*e+b*d, b*e-d*c, -a*e+d*b, a*c-b*b};
        // find the greatest absolute value of adjugate and take that column
        int maxindex = 0;
        IKReal val = IKabs(adjugate[maxindex]);
        for(int i = 1; i < 9; ++i) {
            IKReal newval = IKabs(adjugate[i]);
            if( val < newval ) {
                val = newval;
                maxindex = i;
            }
        }
        maxindex = maxindex%3;
        if( adjugate[0] > 0 || adjugate[4] > 0 || adjugate[8] > 0 || adjugate[4*maxindex] >= 0 ) {
            // according to the structure of the matrix, should be always negative if a solution exists...
            return;
        }
        IKReal bmult = 1.0/IKsqrt(-adjugate[4*maxindex]);
        IKReal p[3] = {adjugate[maxindex]*bmult, adjugate[3+maxindex]*bmult, adjugate[6+maxindex]*bmult}; // intersection point
        // C = C0 - [p_x] = 2gh^t, C is rank1
        IKReal C[9] = {a,b+p[2],d-p[1],b-p[2],c,e+p[0],d+p[1],e-p[0],f};
        maxindex = 0;
        val = IKabs(C[maxindex]);
        for(int i = 1; i < 9; ++i) {
            IKReal newval = IKabs(C[i]);
            if( val < newval ) {
                val = newval;
                maxindex = i;
            }
        }
        int row = maxindex/3;
        int col = maxindex%3;
        IKReal lineequation[3], coeffs[3], yintersections[2];
        for(int i = 0; i < 2; ++i) {
            if( i == 0 ) {
                lineequation[0] = C[3*row];
                lineequation[1] = C[3*row+1];
                lineequation[2] = C[3*row+2];
            }
            else {
                lineequation[0] = C[col];
                lineequation[1] = C[3+col];
                lineequation[2] = C[6+col];
            }
    
            if( IKabs(lineequation[0]) < std::numeric_limits<IKReal>::epsilon() ) {
                yintersections[0] = -lineequation[2]/lineequation[1];
                IKReal x = 1-yintersections[0]*yintersections[0];
                if( x <= 0 && x > -std::numeric_limits<IKReal>::epsilon() ) {
                    roots[numroots++] = yintersections[0] > 0 ? IKPI_2 : -IKPI_2;
                }
                else {
                    x = IKsqrt(x);
                    roots[numroots++] = IKatan2(yintersections[0], x);
                    roots[numroots] = IKPI - roots[numroots-1]; numroots++;
                }
            }
            else {
                coeffs[0] = lineequation[0]*lineequation[0]+lineequation[1]*lineequation[1];
                coeffs[1] = 2*lineequation[1]*lineequation[2];
                coeffs[2] = lineequation[2]*lineequation[2]-lineequation[0]*lineequation[0];
                polyroots2(coeffs,yintersections,numyroots);
                for(int j = 0; j < numyroots; ++j) {
                    roots[numroots++] = IKatan2(yintersections[j],-(lineequation[1]*yintersections[j]+lineequation[2])/lineequation[0]);
                }
            }
        }
    }
};


/// solves the inverse kinematics equations.
/// \param pfree is an array specifying the free joints of the chain.
IKFAST_API bool ik(const IKReal* eetrans, const IKReal* eerot, const IKReal* pfree, std::vector<IKSolution>& vsolutions) {
IKSolver solver;
return solver.ik(eetrans,eerot,pfree,vsolutions);
}

IKFAST_API const char* getKinematicsHash() { return "3d237bf9cd0926ca3151dfca1d0d8936"; }

#ifndef IKFAST_NO_MAIN
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char** argv)
{
    if( argc != 12+getNumFreeParameters()+1 ) {
        printf("\nUsage: ./ik r00 r01 r02 t0 r10 r11 r12 t1 r20 r21 r22 t2 free0 ...\n\n"
               "Returns the ik solutions given the transformation of the end effector specified by\n"
               "a 3x3 rotation R (rXX), and a 3x1 translation (tX).\n"
               "There are %d free parameters that have to be specified.\n\n",getNumFreeParameters());
        return 1;
    }

    std::vector<IKSolution> vsolutions;
    std::vector<IKReal> vfree(getNumFreeParameters());
    IKReal eerot[9],eetrans[3];
    eerot[0] = atof(argv[1]); eerot[1] = atof(argv[2]); eerot[2] = atof(argv[3]); eetrans[0] = atof(argv[4]);
    eerot[3] = atof(argv[5]); eerot[4] = atof(argv[6]); eerot[5] = atof(argv[7]); eetrans[1] = atof(argv[8]);
    eerot[6] = atof(argv[9]); eerot[7] = atof(argv[10]); eerot[8] = atof(argv[11]); eetrans[2] = atof(argv[12]);
    for(size_t i = 0; i < vfree.size(); ++i)
        vfree[i] = atof(argv[13+i]);
    bool bSuccess = ik(eetrans, eerot, vfree.size() > 0 ? &vfree[0] : NULL, vsolutions);

    if( !bSuccess ) {
        fprintf(stderr,"Failed to get ik solution\n");
        return -1;
    }

    printf("Found %d ik solutions:\n", (int)vsolutions.size());
    std::vector<IKReal> sol(getNumJoints());
    for(size_t i = 0; i < vsolutions.size(); ++i) {
        printf("sol%d (free=%d): ", (int)i, (int)vsolutions[i].GetFree().size());
        std::vector<IKReal> vsolfree(vsolutions[i].GetFree().size());
        vsolutions[i].GetSolution(&sol[0],vsolfree.size()>0?&vsolfree[0]:NULL);
        for( size_t j = 0; j < sol.size(); ++j)
            printf("%f, ", (float)sol[j]);
        printf("\n");
    }
    return 0;
}

#endif
